#ifndef MVIMPACT_BARCODE_H_
#define MVIMPACT_BARCODE_H_


/// \file
///

#ifndef WRAP_PYTHON
#include "mvIMPACT.h"
#endif // #ifndef WRAP_PYTHON




#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_PYTHON


#include <vl_cr1d.h>


#endif // #ifndef WRAP_PYTHON
#endif // DOXYGEN_SHOULD_SKIP_THIS


namespace mvIMPACT {

        /// \namespace mvIMPACT::barcode This namespace contains all classes and functions from mvIMPACT Barcode.
	namespace barcode {


/// \brief The checksum mode enum.

/// A checksum can either be not used at all, or being calculated and
/// transmitted or being calculated but not transmitted.
enum ChecksummingMode 
{
	/// Do not calculate checksum. All decoded characters are transmitted.
	cmNoChecksum				= mvIMPACT_C::CR1D_NO_CHECKSUM,
	/// Calculate checksum and transmit it. All decoded characters are transmitted,
	/// but only if the checksum matches.
	cmChecksumTransmit			= mvIMPACT_C::CR1D_CHECKSUM_TRANSMIT,
	/// Calculate checksum but do not transmit. The checksum character(s) are stripped
	/// from the decoded string and are not transmitted.
	cmChecksumNoTransmit		= mvIMPACT_C::CR1D_CHECKSUM_NO_TRANSMIT,
};


/// \brief This enumeration lists some predefined combinations for locator/decoder search flags.
//! The directions start at the horizontal direction, left-to-right and then count in 45 degree
//! increments clockwise.
//! The directions can be ORed or ADDed together to specify search direction combinations.
//! Some predefined combinations (horizontal only, vertical only, horizontal and vertical,
//! any direction) are also available.
enum SearchDirections
{
	/// Search in horizontal direction.
	sdHorizontal				= mvIMPACT_C::CR1D_DIRHORIZONTAL,
	/// Search in vertical direction.
	sdVertical					= mvIMPACT_C::CR1D_DIRVERTICAL,
	/// Search in horizontal and vertical direction.
	sdOrthogonal				= mvIMPACT_C::CR1D_DIRORTHOGONAL,
	/// Search in all directions.
	sdAll						= mvIMPACT_C::CR1D_DIRALL,

	/// Search in 0 degree direction (horizontal, left-to-right).
	sdDir0						= mvIMPACT_C::CR1D_DIR000,
	/// Search in 45 degree direction.
	sdDir45						= mvIMPACT_C::CR1D_DIR045,
	/// Search in 90 degree direction (vertical, top-to-bottom).
	sdDir90						= mvIMPACT_C::CR1D_DIR090,
	/// Search in 135 degree direction.
	sdDir135					= mvIMPACT_C::CR1D_DIR135,
	/// Search in 180 degree direction (horizontal, right-to-left).
	sdDir180					= mvIMPACT_C::CR1D_DIR180,
	/// Search in 225 degree direction.
	sdDir225					= mvIMPACT_C::CR1D_DIR225,
	/// Search in 270 degree direction (vertical, bottom-to-top).
	sdDir270					= mvIMPACT_C::CR1D_DIR270,
	/// Search in 315 degree direction.
	sdDir315					= mvIMPACT_C::CR1D_DIR315,
};

/// \brief Class that holds all configuration parameters of the barcode decoder.

/// BarcodeConfiguration is used to set or retrieve all the decoder specific parameters.
/// The parameters that can be set are:
/// - scanline spacing
/// - contrast
/// - gap size
/// - minimal gaps
//-----------------------------------------------------------------------------
class BarcodeConfiguration
//-----------------------------------------------------------------------------
{
private:
	/// Unassigned buffer constant.
	enum { unassigned = -1 };
	/// The buffer handle.
	mvIMPACT_C::CR1D_BUFHANDLE handle_;
	/// Pointer to the reference counter.
	mutable int* counter_;

public:
	/// \brief Set the scanline spacing.

	/// For efficiency, the image is only probed along different profiles each spaced by a given
	/// number of pixels. The spacing parameter should be chosen small enough to be sure that a
	/// sufficient number of profiles hit the barcode, for robustness, but large enough to reduce the
	/// running time. For a barcode H pixels high, to be hit at least N times, the spacing should be set
	/// to H/N. 
	/// The default value of 15 pixels is appropriate for the standard sizes.
	/// \note Odd values are always rounded to the next even value.
	/// \see mvIMPACT::barcode::BarcodeConfiguration::getSpacing
	void setSpacing(
		/// Scanline spacing.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="100"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int spacing); 
		

	/// \brief Retrieve scanline spacing.
	
	///Retrieve the value that can be set with mvIMPACT::barcode::BarcodeConfiguration::setSpacing
	/// \see mvIMPACT::barcode::BarcodeConfiguration::setSpacing
	int getSpacing() const;
	
	
	/// \brief Set gradient contrast.

	///Bars edges correspond to steep changes between the foreground color (white label) and
	///foreground marking (black ink). The contrast parameter allows selecting the edges that are
	///strong enough. Setting a low value can result in false edge detection because of noise or
	///background texture. Setting a high value can cause some of the edges to be missed. In
	///practice, a value of 20 deals with poor contrast, 30 is appropriate for normal contrast, and 40
	///or more is suited for printed samples.
	///The default value of 15 usually works fine for barcodes.
	/// \see mvIMPACT::barcode::BarcodeConfiguration::getContrast
	void setContrast(
		/// Gradient contrast.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="255"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int contrast); 
		

	/// \brief Retrieve gradient contrast.

	/// Retrieve the value that can be set with mvIMPACT::barcode::BarcodeConfiguration::setContrast
	/// \see mvIMPACT::barcode::BarcodeConfiguration::setContrast	
	int getContrast() const;
	
	

	
	
	/// \brief Set the size of a gap in a barcode.

	///The reader needs to group bars and fill the spaces between them. The gap size must be larger
	///than the size of the largest spaces, allowing the reader to properly detect neighboring bars and
	///merge them. On the opposite, a too high value might cause unwanted merging with
	///surrounding features outside the quiet zone.
	///The default value is appropriate for barcodes seen at typical resolution. Larger
	///values must be used on codes with a large magnification.
	/// \see mvIMPACT::barcode::BarcodeConfiguration::getGapSize
	void setGapSize(
		/// Gap size.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="100"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int gapSize); 
		

	/// \brief Retrieve size of a gap

	///
	/// \see mvIMPACT::barcode::BarcodeConfiguration::setGapSize	
	int getGapSize() const;
	
	
	/// \brief Set minimum number of gaps a barcode must have.

	///Typical barcodes count at least a certain number of spaces. This property allows telling text or
	///other features from symbols.
	///The default value is appropriate for most of the symbologies. Only in certain
	///circumstances should it be increased (very long strings) or reduced (short Code39, EAN8,
	///etc.)
	/// \see mvIMPACT::barcode::BarcodeConfiguration::getMinGaps
	void setMinGaps(
		/// Minimum gaps.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="100"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int minGaps); 
		

	/// \brief Retrieve minimum number of gaps a barcode must have
	
	/// Retrieve the value that was set with mvIMPACT::barcode::BarcodeConfiguration::setMinGaps
	/// \see mvIMPACT::barcode::BarcodeConfiguration::setMinGaps
	int getMinGaps() const;

	/// \brief Set noise threshold.
	///
	/// This threshold is used to separate signal gradients from noise gradients.
	void setNoiseThreshold(
		/// the noise threshold value
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="100"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int noiseThreshold); 
		
	/// \brief Get noise threshold.
	///
	/// This threshold is used to separate signal gradients from noise gradients.
	int getNoiseThreshold() const;

	/// \brief Set the locator/decoder search directions.
	///
	/// This is a bitfield that encodes search directions. The search directions
	/// start at the horizontal direction, left-to-right with bit 0 and then increase
	/// in 45 degrees clockwise with bit 1 and higher. Thus, bit 0 to 7 encode the
	/// possible directions.
	/// The directions can be ORed or ADDed together to specify search direction combinations.
	/// Some predefined combinations (horizontal only, vertical only, horizontal and vertical,
	/// any direction) are also available.
	/// The SearchDirections enumeration encodes some predefined and useful combinations.
	void setDirections(
		/// directions encoded as a bitfiled
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="255"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int directions); 
		
	/// \brief Get the locator/decoder search directions.
	///
	/// This is a bitfield that encodes search directions. The search directions
	/// start at the horizontal direction, left-to-right with bit 0 and then increase
	/// in 45 degrees clockwise with bit 1 and higher. Thus, bit 0 to 7 encode the
	/// possible directions.
	/// The directions can be ORed or ADDed together to specify search direction combinations.
	/// Some predefined combinations (horizontal only, vertical only, horizontal and vertical,
	/// any direction) are also available.
	/// The SearchDirections enumeration encodes some predefined and useful combinations.
	int getDirections() const;
	
	/// \brief Orphaned ray handling.
	///
	/// An orphaned ray is a ray candidate that is not linked
	/// to another one and thus isolated. Isolated rays are very often wrong candidates. In many
	/// cases they can be filtered out without any harm. Filtering orphaned rays increases the
	/// reader performance.
	void setFilterOrphanedRays(
		bool directions); 
		
	/// \brief Orphaned ray handling.
	///
	/// An orphaned ray is a ray candidate that is not linked
	/// to another one and thus isolated. Isolated rays are very often wrong candidates. In many
	/// cases they can be filtered out without any harm. Filtering orphaned rays increases the
	/// reader performance.
	bool getFilterOrphanedDecodes() const;
	
	/// \brief Orphaned decode handling.
	///
	/// An orphaned decode is a when a group of rays only
	/// has one decoding. Some decoders may return false reads in some cases, but the 
	/// probability that two false reads are returned is very low. Filtering orphaned rays 
	/// increases the robustness of the reader.
	void setFilterOrphanedDecodes(
		bool directions); 
		
	/// \brief Orphaned decode handling.
	///
	/// An orphaned decode is a when a group of rays only
	/// has one decoding. Some decoders may return false reads in some cases, but the 
	/// probability that two false reads are returned is very low. Filtering orphaned rays 
	/// increases the robustness of the reader.
	bool getFilterOrphanedRays() const;
	
	/// \brief Set the print quality checking mode.
	///
	/// If set to true, the barcode print quality is checked according to the 
	/// ANSI X3.182 standard. Otherwise, print quality checks are not performed.
	void setCheckPrintQuality(
		bool checkPrintQuality); 
		
	/// \brief Get the print quality checking mode.
	///
	/// If set to true, the barcode print quality is checked according to the 
	/// ANSI X3.182 standard. Otherwise, print quality checks are not performed.
	bool getCheckPrintQuality() const;

public:
	/// \brief Construct a default configuration.
	BarcodeConfiguration();
	
	/// \brief Create a BarcodeConfiguration from \a src
	BarcodeConfiguration(/// the other BarcodeConfiguration object
			     const BarcodeConfiguration& src);

#ifndef WRAP_ANY
	/// \brief Overwrite current object with \a src.
	BarcodeConfiguration& operator=(
		const BarcodeConfiguration& src);
#endif // #ifndef WRAP_ANY

	/// \brief Destruct a buffer
	virtual ~BarcodeConfiguration();
	
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY
	mvIMPACT_C::CR1D_BUFHANDLE getHandle  () const; 
	int getUseCount() const; 
	bool getIsUnique   () const; 
#endif // #ifndef WRAP_ANY
#endif
	
protected:
	/// \brief Construct an allocated BarcodeConfiguration object from a handle
	explicit BarcodeConfiguration(
		/// the buffer handle
		mvIMPACT_C::CR1D_BUFHANDLE bufhandle);
	
private:
	/// Dispose the buffer. If the reference count drops to zero,
	/// and the buffer was allocated it is freed.
	void dispose();
	
	void copy(const BarcodeConfiguration& src);	
};




/// \brief Barcode results buffer.

///This class contains the results of the decoding process, that was started with mvIMPACT::barcode::decode.
///This includes the number of detected codes and the codes themselves, the code types, the total decoding time
///and other parameters.
//-----------------------------------------------------------------------------
class BarcodeResults
//-----------------------------------------------------------------------------
{
private:
	/// Unassigned buffer constant.
	enum { unassigned = -1 };
	/// The buffer handle.
	mvIMPACT_C::CR1D_BUFHANDLE handle_;
	/// Pointer to the reference counter.
	mutable int* counter_;

public:
	/// \brief Retrieve the number of candidates that were found.
	int getNumberOfCandidates() const;
		
   
	/// \brief  Retrive the number of codes for a specific candidate.
	int getNumberOfCodes(
		///the index of the candidate
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfCandidates-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int candidateIndex) const;
		
   
	/// \brief Retrieve the type of a specific code.
	std::string getCodeType(
				///index of the candidate
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfCandidates-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int candidateIndex, 
				///index of the code
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.getNumberOfCodes(%(candidateIndex)s)-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int codeIndex=0) const;
		
   
	/// \brief Retrieve the actual code. 

	///This includes the checksum at the end of the string.
	/// \note You can select the checksum mode with mvIMPACT::barcode::setBCChecksumMode.
	/// No matter what checksum mode you choose, the checksum will always be calculated and delivered.
	/// Ignoring the checksum only disables filtering of codes, whose checksums fail.
	/// \see mvIMPACT::barcode::setBCChecksumMode
	/// \see mvIMPACT::barcode::getBCChecksumMode
	std::string getCode(
				///index of the candidate
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfCandidates-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int candidateIndex, 
			    ///index of the code
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.getNumberOfCodes(%(candidateIndex)s)-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int codeIndex=0) const;
		
   
	/// \brief Retrieve the checksum of a specific code.
	
	/// \note You can select the checksum mode with mvIMPACT::barcode::setBCChecksumMode.
	/// No matter what checksum mode you choose, the checksum will always be calculated and delivered.
	/// Ignoring the checksum only disables filtering of codes, whose checksums fail.
	/// \see mvIMPACT::barcode::setBCChecksumMode
	/// \see mvIMPACT::barcode::getBCChecksumMode
	std::string getChecksum(
				///index of the candidate
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfCandidates-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int candidateIndex, 
				///index of the code
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.getNumberOfCodes(%(candidateIndex)s)-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int codeIndex=0) const;
		
   
	/// \brief Retrieve the location of a specific candidate.

	///Returns four points, that make up the rectangle that surrounds the candidate
	std::vector<Point> getLocation(
				///index of the candidate
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfCandidates-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int candidateIndex) const;
		

	/// \brief Retrieve the code direction with respect to the candidate.

	/// Get the code direction with respect to the candidate.
	/// It is true if code runs forward, false if code runs backwards.
	bool getDirection(
				///index of the candidate
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfCandidates-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int candidateIndex) const;
	

	/// \brief Retrieve the overprint factor of a specific candidate.

	///Overprinting is returned as a floating point value, that is equal
	///to 1.0 when no overprinting has been detected. The value is bigger than 1, when overprinting
	///has been detected and it is smaller than 1 when underprinting has been detected. The value
	///returned is the quotient of a bar width divided by a space width of the same size.
	double getOverprint(
				///index of the candidate
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfCandidates-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int candidateIndex) const;
	

	/// \brief Retrieve the total decoding time for all codes in milliseconds.

	///
	/// \see mvIMPACT::barcode::BarcodeResults::getDecodingTime
	double getTotalDecodingTime () const;
	

	/// \brief Retrive decoding time for a specific candidate in milliseconds.
	
	///
	/// \see mvIMPACT::barcode::BarcodeResults::getTotalDecodingTime
	double getDecodingTime (
				///index of the candidate
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfCandidates-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int candidateIndex) const;
	
	/// \brief Retrieve the reflectance minimum for the scan.

	/// This value is part of the set of values calculated to perform
	/// print quality checking according to the ANSI X3.182 standard.
	int getReflectanceMinimum (
				///index of the candidate
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfCandidates-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int candidateIndex) const;
	
	/// \brief Retrieve the reflectance maximum for the scan.

	/// This value is part of the set of values calculated to perform
	/// print quality checking according to the ANSI X3.182 standard.
	int getReflectanceMaximum (
				///index of the candidate
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfCandidates-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int candidateIndex) const;
	
	/// \brief Retrieve the symbol contrast for the scan.

	/// This value is part of the set of values calculated to perform
	/// print quality checking according to the ANSI X3.182 standard.
	int getSymbolContrast (
				///index of the candidate
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfCandidates-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int candidateIndex) const;
	
	/// \brief Retrieve the global threshold for the scan.

	/// This value is part of the set of values calculated to perform
	/// print quality checking according to the ANSI X3.182 standard.
	int getGlobalThreshold (
				///index of the candidate
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfCandidates-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int candidateIndex) const;
	
	/// \brief Retrieve the minimum edge contrast for the scan.

	/// This value is part of the set of values calculated to perform
	/// print quality checking according to the ANSI X3.182 standard.
	int getMinimumEdgeContrast (
				///index of the candidate
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfCandidates-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int candidateIndex) const;
	
	/// \brief Retrieve the modulation for the scan.

	/// This value is part of the set of values calculated to perform
	/// print quality checking according to the ANSI X3.182 standard.
	double getModulation (
				///index of the candidate
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfCandidates-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int candidateIndex) const;
	
	/// \brief Retrieve the maximum non-uniformity for the scan.

	/// This value is part of the set of values calculated to perform
	/// print quality checking according to the ANSI X3.182 standard.
	int getMaximumNonUniformity (
				///index of the candidate
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfCandidates-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int candidateIndex) const;
	
	/// \brief Retrieve the edge determination test result for the scan.

	/// This value is part of the set of values calculated to perform
	/// print quality checking according to the ANSI X3.182 standard.
	bool getEdgeDeterminationTest (
				///index of the candidate
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfCandidates-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int candidateIndex) const;
	
	/// \brief Retrieve the minimum reflectance test result for the scan.

	/// This value is part of the set of values calculated to perform
	/// print quality checking according to the ANSI X3.182 standard.
	bool getMinimumReflectanceTest (
				///index of the candidate
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfCandidates-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int candidateIndex) const;
	
	/// \brief Retrieve the minimum edge contrast test result for the scan.

	/// This value is part of the set of values calculated to perform
	/// print quality checking according to the ANSI X3.182 standard.
	bool getMinimumEdgeContrastTest (
				///index of the candidate
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfCandidates-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int candidateIndex) const;
	
	/// \brief Retrieve the symbol contrast grade for the scan.

	/// This value is part of the set of values calculated to perform
	/// print quality checking according to the ANSI X3.182 standard.
	int getSymbolContrastGrade (
				///index of the candidate
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfCandidates-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int candidateIndex) const;
	
	/// \brief Retrieve the modulation grade for the scan.

	/// This value is part of the set of values calculated to perform
	/// print quality checking according to the ANSI X3.182 standard.
	int getModulationGrade (
				///index of the candidate
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfCandidates-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int candidateIndex) const;
	
	/// \brief Retrieve the defects grade for the scan.

	/// This value is part of the set of values calculated to perform
	/// print quality checking according to the ANSI X3.182 standard.
	int getDefectsGrade (
				///index of the candidate
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfCandidates-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int candidateIndex) const;
	
	/// \brief Retrieve the decodability grade for the scan.

	/// This value is part of the set of values calculated to perform
	/// print quality checking according to the ANSI X3.182 standard.
	int getDecodabilityGrade (
				///index of the candidate
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfCandidates-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int candidateIndex) const;
	
	/// \brief Retrieve the scan grade for the scan.

	/// This value is part of the set of values calculated to perform
	/// print quality checking according to the ANSI X3.182 standard.
	int getScanGrade (
				///index of the candidate
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfCandidates-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int candidateIndex) const;
	

public:
	/// \brief Construct a non-allocated buffer.
	BarcodeResults();
	
	/// \brief Construct a BarcodeResults object from \a src
	BarcodeResults(///the other BarcodeResults object
		       const BarcodeResults& src);

#ifndef WRAP_ANY
	/// \brief Overwrite current object with \a src.
	BarcodeResults& operator=(///the other BarcodeResults object
				  const BarcodeResults& src);
#endif // #ifndef WRAP_ANY

	/// \brief Destructor
	virtual ~BarcodeResults();

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY    
	mvIMPACT_C::CR1D_BUFHANDLE getHandle  () const; 
	int getUseCount() const; 
	bool getIsUnique   () const; 
#endif // #ifndef WRAP_ANY	
#endif
	
#ifndef DOXYGEN_SHOULD_SKIP_THIS	
#ifndef WRAP_ANY
	static BarcodeResults barcodeResultsFactory(mvIMPACT_C::CR1D_BUFHANDLE buffer_handle);
#endif // #ifndef WRAP_ANY
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

private:
	/// construct an allocated buffer from a handle
	explicit BarcodeResults(
		/// the buffer handle
		mvIMPACT_C::CR1D_BUFHANDLE bufhandle);
	
	/// Dispose the buffer. If the reference count drops to zero,
	/// and the buffer was allocated it is freed.
	void dispose();
	
	void copy(const BarcodeResults& src);

};

/// \brief Detects and decodes barcodes using standard configuration.
///
/// This function uses the standard configuration for the decoding process.
/// The built-in standard values suit for most barcodes,
/// however, these values can be overridden by the vl_cr1d.dec file!
//-----------------------------------------------------------------------------
BarcodeResults decodeBC(
	/// The source image buffer to be decoded.
	const Image& source);

#ifndef I386_LINUX
/// \brief Detects and decodes barcodes using standard configuration.
///
/// This function uses the standard configuration for the decoding process.
/// The built-in standard values suit for most barcodes,
/// however, these values can be overridden by the vl_cr1d.dec file!
/// 
/// This overload can be used to visualize the runs used by the barcode
/// locator.
//-----------------------------------------------------------------------------
BarcodeResults decodeBC(
	/// The source image buffer to be decoded.
	const Image& source,
	/// The window for the LineArrayOfInterest visualization.
	Window& window,
	/// The LineArrayOfInterest containing the runs of the barcode locator.
	LineArrayOfInterest& runs);
#endif // !I386_LINUX

/// \brief Detects and decodes barcodes using standard configuration.

/// This function uses the standard configuration for the decoding process.
/// The built-in standard values suit for most barcodes,
/// however, these values can be overridden by the vl_cr1d.dec file!
//-----------------------------------------------------------------------------
void decodeBC(
	/// The BarcodeResults object.
	BarcodeResults& dest,
	/// The source image buffer to be decoded.
	const Image& source
	);

#ifndef I386_LINUX
/// \brief Detects and decodes barcodes using standard configuration.

/// This function uses the standard configuration for the decoding process.
/// The built-in standard values suit for most barcodes,
/// however, these values can be overridden by the vl_cr1d.dec file!
/// 
/// This overload can be used to visualize the runs used by the barcode
/// locator.
//-----------------------------------------------------------------------------
void decodeBC(
	/// The BarcodeResults object.
	BarcodeResults& dest,
	/// The source image buffer to be decoded.
	const Image& source,
	/// The window for the LineArrayOfInterest visualization.
	Window& window,
	/// The LineArrayOfInterest containing the runs of the barcode locator.
	LineArrayOfInterest& runs);
#endif // !I386_LINUX

/// \brief Detects and decodes barcodes using specific configuration parameters.

/// This function decodes an image using the decoder configuration given with
/// parameter \a barcodeConfiguration. \a barcodeConfiguration overrides the built-in defaults and the 
/// default values that may be given by the vl_cr1d.dec file.
//-----------------------------------------------------------------------------
BarcodeResults decodeBC(
	/// The source image buffer to be decoded.
	const Image& source,
	/// The barcode configuration to use.
	const BarcodeConfiguration& barcodeConfiguration);

#ifndef I386_LINUX
/// \brief Detects and decodes barcodes using specific configuration parameters.

/// This function decodes an image using the decoder configuration given with
/// parameter \a barcodeConfiguration. \a barcodeConfiguration overrides the built-in defaults and the 
/// default values that may be given by the vl_cr1d.dec file.
//-----------------------------------------------------------------------------
BarcodeResults decodeBC(
	/// The source image buffer to be decoded.
	const Image& source,
	/// The barcode configuration to use.
	const BarcodeConfiguration& barcodeConfiguration,
	/// The window for the LineArrayOfInterest visualization.
	Window& window,
	/// The LineArrayOfInterest containing the runs of the barcode locator.
	LineArrayOfInterest& runs);
#endif // !I386_LINUX

/// \brief Detects and decodes barcodes using specific configuration parameters.

/// This function decodes an image using the decoder configuration given with
/// parameter \a barcodeConfiguration. \a barcodeConfiguration overrides the built-in defaults and the 
/// default values that may be given by the vl_cr1d.dec file.
//-----------------------------------------------------------------------------
void decodeBC(
	/// The results buffer.
	BarcodeResults& dest, 
	/// The source image buffer to be decoded.
	const Image& source,
	/// The barcode configuration.
	const BarcodeConfiguration& barcodeConfiguration);

#ifndef I386_LINUX
/// \brief Detects and decodes barcodes using specific configuration parameters.

/// This function decodes an image using the decoder configuration given with
/// parameter \a barcodeConfiguration. \a barcodeConfiguration overrides the built-in defaults and the 
/// default values that may be given by the vl_cr1d.dec file.
//-----------------------------------------------------------------------------
void decodeBC(
	/// The results buffer.
	BarcodeResults& dest, 
	/// The source image buffer to be decoded.
	const Image& source,
	/// The barcode configuration.
	const BarcodeConfiguration& barcodeConfiguration,
	/// The window for the LineArrayOfInterest visualization.
	Window& window,
	/// The LineArrayOfInterest containing the runs of the barcode locator.
	LineArrayOfInterest& runs);
#endif // !I386_LINUX

// \brief Decodes a barcode from a line profile.

/// This function uses the standard configuration for the decoding process.
/// The built-in standard values suit for most barcodes,
/// however, these values can be overridden by the vl_cr1d.dec file!
//-----------------------------------------------------------------------------
BarcodeResults decodeBC(
	/// The profile buffer to be decoded.
	const Profile& profile);


/// \brief Decodes a barcode from a line profile.

/// This function uses the standard configuration for the decoding process.
/// The built-in standard values suit for most barcodes,
/// however, these values can be overridden by the vl_cr1d.dec file!
//-----------------------------------------------------------------------------
void decodeBC(
	/// The BarcodeResults object.
	BarcodeResults& dest,
	/// The profile buffer to be decoded.
	const Profile& profile);


/// \brief Decodes a barcode from a line profile.

/// This function decodes an image using the decoder configuration given with
/// parameter \a barcodeConfiguration. \a barcodeConfiguration overrides the built-in defaults and the 
/// default values that may be given by the vl_cr1d.dec file.
//-----------------------------------------------------------------------------
BarcodeResults decodeBC(
	/// The profile buffer to be decoded.
	const Profile& profile,
	/// The barcode configuration to use.
	const BarcodeConfiguration& barcodeConfiguration);


/// \brief Decodes a barcode from a line profile.

/// This function decodes an image using the decoder configuration given with
/// parameter \a barcodeConfiguration. \a barcodeConfiguration overrides the built-in defaults and the 
/// default values that may be given by the vl_cr1d.dec file.
//-----------------------------------------------------------------------------
void decodeBC(
	/// The results buffer.
	BarcodeResults& dest, 
	/// The profile buffer to be decoded.
	const Profile& profile,
	/// The barcode configuration.
	const BarcodeConfiguration& barcodeConfiguration);


// \brief Decodes a barcode at a given location.

/// This function uses the standard configuration for the decoding process.
/// The built-in standard values suit for most barcodes,
/// however, these values can be overridden by the vl_cr1d.dec file!
//-----------------------------------------------------------------------------
BarcodeResults decodeBCAt(
	/// The source image buffer to be decoded.
	const Image& source,
	/// The location of the barcode.
	const Line& location);


/// \brief Decodes a barcode at a given location.

/// This function uses the standard configuration for the decoding process.
/// The built-in standard values suit for most barcodes,
/// however, these values can be overridden by the vl_cr1d.dec file!
//-----------------------------------------------------------------------------
void decodeBCAt(
	/// The BarcodeResults object.
	BarcodeResults& dest,
	/// The source image buffer to be decoded.
	const Image& source,
	/// The location of the barcode.
	const Line& location);


/// \brief Decodes a barcode at a given location.

/// This function decodes an image using the decoder configuration given with
/// parameter \a barcodeConfiguration. \a barcodeConfiguration overrides the built-in defaults and the 
/// default values that may be given by the vl_cr1d.dec file.
//-----------------------------------------------------------------------------
BarcodeResults decodeBCAt(
	/// The source image buffer to be decoded.
	const Image& source,
	/// The barcode configuration to use.
	const BarcodeConfiguration& barcodeConfiguration,
	/// The location of the barcode.
	const Line& location);


/// \brief Decodes a barcode at a given location.

/// This function decodes an image using the decoder configuration given with
/// parameter \a barcodeConfiguration. \a barcodeConfiguration overrides the built-in defaults and the 
/// default values that may be given by the vl_cr1d.dec file.
//-----------------------------------------------------------------------------
void decodeBCAt(
	/// The results buffer.
	BarcodeResults& dest, 
	/// The source image buffer to be decoded.
	const Image& source,
	/// The barcode configuration.
	const BarcodeConfiguration& barcodeConfiguration,
	/// The location of the barcode.
	const Line& location);


// \brief Decodes a barcode from distances stored in profile data.

/// Tries to decode 1D symbologies (barcodes) in a profile that consists of distances. 
/// This allows you to use your own locator and preprocessing to extract a symbols bar/space
/// thicknesses at a given location, and feed this data to the decoders only.
///
/// The profile no longer encodes pixel data, but bar/space width. The data can be
/// integer (in this case you've got to somehow normalize the width) or floating point
/// (in this case the function normalizes the data to the module width).
///
/// The distances must be given in strict monotonously increasing coordinates of the falling and
/// rising edges of the bars/spaces.

/// This function uses the standard configuration for the decoding process.
/// The built-in standard values suit for most barcodes,
/// however, these values can be overridden by the vl_cr1d.dec file!
//-----------------------------------------------------------------------------
BarcodeResults decodeBCFromDistances(
	/// The profile buffer containing distances to to be decoded.
	const Profile& distances);


/// \brief Decodes a barcode from distances stored in profile data.

/// Tries to decode 1D symbologies (barcodes) in a profile that consists of distances. 
/// This allows you to use your own locator and preprocessing to extract a symbols bar/space
/// thicknesses at a given location, and feed this data to the decoders only.
///
/// The profile no longer encodes pixel data, but bar/space width. The data can be
/// integer (in this case you've got to somehow normalize the width) or floating point
/// (in this case the function normalizes the data to the module width).
///
/// The distances must be given in strict monotonously increasing coordinates of the falling and
/// rising edges of the bars/spaces.

/// This function uses the standard configuration for the decoding process.
/// The built-in standard values suit for most barcodes,
/// however, these values can be overridden by the vl_cr1d.dec file!
//-----------------------------------------------------------------------------
void decodeBCFromDistances(
	/// The BarcodeResults object.
	BarcodeResults& dest,
	/// The profile buffer containing distances to to be decoded.
	const Profile& distances);


/// \brief Decodes a barcode from distances stored in profile data.

/// Tries to decode 1D symbologies (barcodes) in a profile that consists of distances. 
/// This allows you to use your own locator and preprocessing to extract a symbols bar/space
/// thicknesses at a given location, and feed this data to the decoders only.
///
/// The profile no longer encodes pixel data, but bar/space width. The data can be
/// integer (in this case you've got to somehow normalize the width) or floating point
/// (in this case the function normalizes the data to the module width).
///
/// The distances must be given in strict monotonously increasing coordinates of the falling and
/// rising edges of the bars/spaces.

/// This function decodes an image using the decoder configuration given with
/// parameter \a barcodeConfiguration. \a barcodeConfiguration overrides the built-in defaults and the 
/// default values that may be given by the vl_cr1d.dec file.
//-----------------------------------------------------------------------------
BarcodeResults decodeBCFromDistances(
	/// The profile buffer containing distances to to be decoded.
	const Profile& distances,
	/// The barcode configuration to use.
	const BarcodeConfiguration& barcodeConfiguration);


/// \brief Decodes a barcode from distances stored in profile data.

/// Tries to decode 1D symbologies (barcodes) in a profile that consists of distances. 
/// This allows you to use your own locator and preprocessing to extract a symbols bar/space
/// thicknesses at a given location, and feed this data to the decoders only.
///
/// The profile no longer encodes pixel data, but bar/space width. The data can be
/// integer (in this case you've got to somehow normalize the width) or floating point
/// (in this case the function normalizes the data to the module width).
///
/// The distances must be given in strict monotonously increasing coordinates of the falling and
/// rising edges of the bars/spaces.

/// This function decodes an image using the decoder configuration given with
/// parameter \a barcodeConfiguration. \a barcodeConfiguration overrides the built-in defaults and the 
/// default values that may be given by the vl_cr1d.dec file.
//-----------------------------------------------------------------------------
void decodeBCFromDistances(
	/// The results buffer.
	BarcodeResults& dest, 
	/// The profile buffer containing distances to to be decoded.
	const Profile& distances,
	/// The barcode configuration.
	const BarcodeConfiguration& barcodeConfiguration);


/// \brief Locates barcodes using standard parameters

///This function detects the barcodes, but does not decode them!
/// The location of the barcodes can be queried by calling mvIMPACT::barcode::BarcodeResults::getLocation.
/// The built-in standard values suit for most barcodes,
/// however, these values can be overridden by the vl_cr1d.dec file!
///
/// \see mvIMPACT::barcode::BarcodeResults::getLocation
//-----------------------------------------------------------------------------
BarcodeResults locateBC(
	/// The source image buffer where barcodes will be located.
	const Image& source);

#ifndef I386_LINUX
/// \brief Locates barcodes using standard parameters

///This function detects the barcodes, but does not decode them!
/// The location of the barcodes can be queried by calling mvIMPACT::barcode::BarcodeResults::getLocation.
/// The built-in standard values suit for most barcodes,
/// however, these values can be overridden by the vl_cr1d.dec file!
/// 
/// This overload can be used to visualize the runs used by the barcode
/// locator.
///
/// \see mvIMPACT::barcode::BarcodeResults::getLocation
//-----------------------------------------------------------------------------
BarcodeResults locateBC(
	/// The source image buffer where barcodes will be located.
	const Image& source,
	/// The window for the LineArrayOfInterest visualization.
	Window& window,
	/// The LineArrayOfInterest containing the runs of the barcode locator.
	LineArrayOfInterest& runs);
#endif // !I386_LINUX^M

/// \brief Locates barcodes using standard parameters

///This function detects the barcodes, but does not decode them!
/// The location of the barcodes can be queried by calling mvIMPACT::barcode::BarcodeResults::getLocation.
/// The built-in standard values suit for most barcodes,
/// however, these values can be overridden by the vl_cr1d.dec file!
/// 
/// \see mvIMPACT::barcode::BarcodeResults::getLocation
//-----------------------------------------------------------------------------
void locateBC(
	/// The results buffer.
	BarcodeResults& dest,
	/// The source image buffer where barcodes will be located.
	const Image& source);

#ifndef I386_LINUX
/// \brief Locates barcodes using standard parameters

///This function detects the barcodes, but does not decode them!
/// The location of the barcodes can be queried by calling mvIMPACT::barcode::BarcodeResults::getLocation.
/// The built-in standard values suit for most barcodes,
/// however, these values can be overridden by the vl_cr1d.dec file!
/// 
/// This overload can be used to visualize the runs used by the barcode
/// locator.
///
/// \see mvIMPACT::barcode::BarcodeResults::getLocation
//-----------------------------------------------------------------------------
void locateBC(
	/// The results buffer.
	BarcodeResults& dest,
	/// The source image buffer where barcodes will be located.
	const Image& source,
	/// The window for the LineArrayOfInterest visualization.
	Window& window,
	/// The LineArrayOfInterest containing the runs of the barcode locator.
	LineArrayOfInterest& runs);
#endif // !I386_LINUX^M

/// \brief Locates barcodes using a specific configuration.

/// This function detects the barcodes using configuration \a barcodeConfiguration, but does not decode them!
/// The location of the barcodes can be queried by calling mvIMPACT::barcode::BarcodeResults::getLocation.
/// 
/// \see mvIMPACT::barcode::BarcodeResults::getLocation
//-----------------------------------------------------------------------------
BarcodeResults locateBC(
	/// The source image buffer where barcodes will be located.
	const Image& source,
	/// The barcode configuration.
	const BarcodeConfiguration& barcodeConfiguration); 

#ifndef I386_LINUX
/// \brief Locates barcodes using a specific configuration.

/// This function detects the barcodes using configuration \a barcodeConfiguration, but does not decode them!
/// The location of the barcodes can be queried by calling mvIMPACT::barcode::BarcodeResults::getLocation.
/// 
/// This overload can be used to visualize the runs used by the barcode
/// locator.
/// 
/// \see mvIMPACT::barcode::BarcodeResults::getLocation
//-----------------------------------------------------------------------------
BarcodeResults locateBC(
	/// The source image buffer where barcodes will be located.
	const Image& source,
	/// The barcode configuration.
	const BarcodeConfiguration& barcodeConfiguration,
	/// The window for the LineArrayOfInterest visualization.
	Window& window,
	/// The LineArrayOfInterest containing the runs of the barcode locator.
	LineArrayOfInterest& runs);
#endif // !I386_LINUX^M

/// \brief Locates barcodes using a specific configuration.
///
///This function detects the barcodes using configuration \a barcodeConfiguration, but does not decode them!
/// The location of the barcodes can be queried by calling mvIMPACT::barcode::BarcodeResults::getLocation.
/// 
/// \see mvIMPACT::barcode::BarcodeResults::getLocation
//-----------------------------------------------------------------------------
void locateBC(
	/// The results buffer.
	BarcodeResults& dest, 
	/// The source image buffer where barcodes will be located.
	const Image& source,
	/// The barcode configuration.
	const BarcodeConfiguration& barcodeConfiguration);

#ifndef I386_LINUX
/// \brief Locates barcodes using a specific configuration.
///
///This function detects the barcodes using configuration \a barcodeConfiguration, but does not decode them!
/// The location of the barcodes can be queried by calling mvIMPACT::barcode::BarcodeResults::getLocation.
/// 
/// This overload can be used to visualize the runs used by the barcode
/// locator.
///
/// \see mvIMPACT::barcode::BarcodeResults::getLocation
//-----------------------------------------------------------------------------
void locateBC(
	/// The results buffer.
	BarcodeResults& dest, 
	/// The source image buffer where barcodes will be located.
	const Image& source,
	/// The barcode configuration.
	const BarcodeConfiguration& barcodeConfiguration,
	/// The window for the LineArrayOfInterest visualization.
	Window& window,
	/// The LineArrayOfInterest containing the runs of the barcode locator.
	LineArrayOfInterest& runs);
#endif // !I386_LINUX^M


/// \brief Calculates an edge map image.

///	The so called edge map image is the basis for barcode location. 
///	The contents of an edge map image is affected by the contrast setting.
//-----------------------------------------------------------------------------
Image edgeMapBC(
	/// The source image.
	const Image& source);

/// \brief Calculates an edge map image.

///	The so called edge map image is the basis for barcode location. 
///	The contents of an edge map image is affected by the contrast setting.
//-----------------------------------------------------------------------------
 void edgeMapBC(
	    /// The destination image.
	    Image& dest,
	    /// The source image.
	    const Image& source);

/// \brief Calculates an edge map image.

///	The so called edge map image is the basis for barcode location. 
///	The contents of an edge map image is affected by the contrast setting.
//-----------------------------------------------------------------------------
Image edgeMapBC(
	/// The source image.
	const Image& source,
	/// The barcode configuration.
	const BarcodeConfiguration& barcodeConfiguration); 

/// \brief Calculates an edge map image.

///	The so called edge map image is the basis for barcode location. 
///	The contents of an edge map image is affected by the contrast setting.
//-----------------------------------------------------------------------------
 void edgeMapBC(
	/// The destination image.
	Image& dest,
	/// The source image.
	const Image& source,
	/// The barcode configuration.
	const BarcodeConfiguration& barcodeConfiguration); 


/// \brief Add a barcode decoder.

///This function adds a decoder to the currently selected decoders. The parameter \a decoder
///specifies the decoder to be added.
///Under Windows the string passed in is used to build the name of a dynamic link library by
///adding ".dll" to the end. For example, if you pass "s1d_code39" in this string, the actual
///decoder dll that is used is named "s1d_code39.dll".
///Under Linux, the string passed in is used to build the name of a shared object by prefixing it
///with "lib" and by adding ".so" to the end. For example, if you pass "s1d_code39" in this
///string, the actual decoder dll that is used is named "libs1d_code39.so".
///The function tries to load the decoder and adds it to the list of decoders. If the decoder is
///already loaded an mvIMPACT::ImpactException is thrown.
///If the decoder could not be loaded because it does not exist or because it is not a valid
///decoder, the an mvIMPACT::ImpactException is thrown, too.
///The \a index parameter specifies where to add the decoder. Pass 0 to add the decoder to the
///begin of the list of decoders. Pass -1 to add the decoder to the end of the list of decoders. Pass
///a positive integer to insert the decoder at the specific position. If the Index parameter is out of
///range an mvIMPACT::ImpactException is thrown.
/// \see mvIMPACT::barcode::removeBCDecoder
//-----------------------------------------------------------------------------
void addBCDecoder(
	/// Specifies the decoder.
	const std::string& decoder,
	/// The index of the decoder.
	/// Set to 0 to insert the decoder at the beginning of the list.
	/// Set to -1 to insert the decoder at the end of the list.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="-1"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="getBCDecoderCount()"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			   
	int index=-1);






/// \brief Remove a barcode decoder from the decoder list

///This function removes a decoder from the currently selected decoders. The parameter
///Decoder specifies the decoder to be removed.
///Under Windows the string passed in is used to build the name of a dynamic link library by
///adding ".dll" to the end. For example, if you pass "s1d_code39" in this string, the actual
///decoder dll that is used is named "s1d_code39.dll".
///Under Linux, the string passed in is used to build the name of a shared object by prefixing it
///with "lib" and by adding ".so" to the end. For example, if you pass "s1d_code39" in this
///string, the actual decoder dll that is used is named "libs1d_code39.so".
///The function tries to remove the decoder from the list of decoders. If the decoder is not
///loaded, the function fails and an mvIMPACT::ImpactException is thrown.
//-----------------------------------------------------------------------------
void removeBCDecoder(
	/// Specifies the decoder.
	const std::string& decoder);






/// \brief Returns the number of loaded decoders.

///Returns the number of decoders, that were loaded via the vl_cr1d.dec file or
/// with several calls to mvIMPACT::barcode::addBCDecoder.
///\see mvIMPACT::barcode::addBCDecoder
///\see mvIMPACT::barcode::removeBCDecoder
//-----------------------------------------------------------------------------
int getBCDecoderCount();






/// \brief Returns the name of a specific decoder.

///Returns the name of a specific decoder, that was loaded via the vl_cr1d.dec file
/// or by calling mvIMPACT::barcode::addBCDecoder.
/// \note This is the name of the decoder, as it was passed to mvIMPACT::barcode::addBCDecoder and not the
/// actual filename of the decoder module.
///\see mvIMPACT::barcode::addBCDecoder
//-----------------------------------------------------------------------------
std::string getBCDecoderName(
	/// Specifies the decoder index. 
	///Valid range is from 0 to mvIMPACT::barcode::getBCDecoderCount -1							
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="getBCDecoderCount()-1"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			   
	int index);






/// \brief Returns the filename of a specific decoder.

///Returns the filename of a specific decoder module, that was loaded via the vl_cr1d.dev file
/// or by calling mvIMPACT::barcode::addBCDecoder.
/// \note This is the filename of the decoder, thus Windows will deliver other results than Linux!
///\see mvIMPACT::barcode::addBCDecoder
///\see mvIMPACT::barcode::getBCDecoderName
//-----------------------------------------------------------------------------
std::string getBCDecoderFileName(
	/// Specifies the decoder index.
	///Valid range is from 0 to mvIMPACT::barcode::getBCDecoderCount -1							
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="getBCDecoderCount()-1"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			   
	int index);






/// \brief Returns the flag that indicates if a specific decoder 
/// tries the next decoder even on successful decoding.

///Returns \c true if the decoder tries the next decoder even on success or \c false if not.
/// \see mvIMPACT::barcode::setBCTryNextOnSuccess
//-----------------------------------------------------------------------------
bool getBCTryNextOnSuccess(
	/// Specifies the decoder index.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="getBCDecoderCount()-1"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			   
	int index);








/// \brief Sets the flag that indicates if a specific decoder 
/// tries the next decoder even on successful decoding.
///
/// \see mvIMPACT::barcode::getBCTryNextOnSuccess
//-----------------------------------------------------------------------------
void setBCTryNextOnSuccess(
	/// Specifies the decoder index.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="getBCDecoderCount()-1"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			   
	int index,
	/// The new try next on success value
	/// Set to \c true to try next decoder even on success
	// or \c false if not.
	bool bCTryNextOnSuccess);






/// \brief Returns the decoder's checksum mode.

///\note The checksum will always be calculated. If you choose to ignore the checksum, 
/// it will be calculated anyway, but filtering of codes, whose checksum fail will be disabled!
/// \see mvIMPACT::barcode::setBCChecksumMode
//-----------------------------------------------------------------------------
ChecksummingMode getBCChecksumMode(
	/// Specifies the decoder index.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="getBCDecoderCount()-1"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			   
	int index);


/// \brief Set the decoder's checksum mode.

///\note The checksum will always be calculated. If you choose to ignore the checksum, 
/// it will be calculated anyway, but filtering of codes, whose checksum fail will be disabled!
/// \see mvIMPACT::barcode::getBCChecksumMode
//-----------------------------------------------------------------------------
void setBCChecksumMode(
	/// Specifies the decoder index.
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
	/// <mvParamMax><mvParamComplex Python="getBCDecoderCount()-1"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			   
	int index,
	/// The new checksum mode
	ChecksummingMode bCChecksumMode);


/// \brief Checks if the barcode license is valid.
bool checkLicenseBC();

/// \brief Return version number from barcode module.

/// This function returns a version string. The following format is used: 
/// "major.minor.point.build" where "major" specifies the major version number, "minor" specifies
/// the minor version number, "point" specifies a point version number and "build" specifies the build number. 
/// Example: 6.3.1.4529
//-----------------------------------------------------------------------------
std::string getVersionBC();

/// \brief Return version number from a decoder used by barcode module.

///Under Windows the string passed in is used to build the name of a dynamic link library by
///adding ".dll" to the end. For example, if you pass "s1d_code39" in this string, the actual
///decoder dll that is used is named "s1d_code39.dll".
///Under Linux, the string passed in is used to build the name of a shared object by prefixing it
///with "lib" and by adding ".so" to the end. For example, if you pass "s1d_code39" in this
///string, the actual decoder dll that is used is named "libs1d_code39.so".

/// This function returns a version string. The following format is used: 
/// "major.minor.point.build" where "major" specifies the major version number, "minor" specifies
/// the minor version number, "point" specifies a point version number and "build" specifies the build number. 
/// Example: 6.3.1.4529
//-----------------------------------------------------------------------------
std::string getVersionBCDecoder(
	/// Specifies the decoder.
	const std::string& decoder);


	} // namespace barcode
} // namespace mvIMPACT


#include "mvIMPACT_barcode_impl.h"

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_PYTHON

#ifndef MVIMPACT_USE_NAMESPACES
using namespace mvIMPACT::barcode;
#endif // #ifndef MVIMPACT_USE_NAMESPACES

#endif // #ifndef WRAP_PYTHON
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

#endif // #ifndef MVIMPACT_BARCODE_H_
