#ifndef MVIMPACT_BLOB_IMPL_H_
#define MVIMPACT_BLOB_IMPL_H_


#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY
namespace mvIMPACT {
	
	namespace blob {
	

inline RunLine::RunLine(int x0, int x1, int y)
{
	m_y = y; m_x0 = x0; m_x1 = x1;
}
	
inline int RunLine::getY() const
{
	return m_y;
}

inline int & RunLine::getY()
{
	return m_y;
}

inline RunLine& RunLine::setY(int y)
{
	m_y = y;
	return *this;
}
	
inline int RunLine::getX0() const
{
	return m_x0;
}

inline int & RunLine::getX0()
{
	return m_x0;
}

inline RunLine& RunLine::setX0(int x0)
{
	m_x0 = x0;
	return *this;
}
	
inline int RunLine::getX1() const
{
	return m_x1;
}

inline int & RunLine::getX1()
{
	return m_x1;
}

inline RunLine& RunLine::setX1(int x1)
{
	m_x1 = x1;
	return *this;
}


inline BlobResult::BlobResult(mvIMPACT_C::BL_FEATURE feature, mvIMPACT_C::IPL_ULONG subItemIndex)
{
	m_feature = feature;
	m_subItemIndex = subItemIndex;
}

inline mvIMPACT_C::BL_FEATURE BlobResult::getFeature() const
{
	return m_feature;
}

inline mvIMPACT_C::IPL_ULONG BlobResult::getSubItemIndex() const
{
	return m_subItemIndex;
}

inline BlobFeature::BlobFeature(
		BlobFeatureList* blobFeatureList,
		mvIMPACT_C::BL_FEATURE blFeature,
		void* param,
		int paramSize,
		int id
		)
{
	m_blobFeatureList = blobFeatureList;
	m_blFeature = blFeature;
	m_Id = id;
	if(NULL != param) { 
		m_Param.allocate((char*)param, paramSize, true);
	} 
	
	// if there is no entry for the feature yet, add new list for the feature:
	if(m_blobFeatureList->getFeatureMap().end() == m_blobFeatureList->getFeatureMap().find(m_blFeature) ) {
		(m_blobFeatureList->getFeatureMap())[m_blFeature] = std::list< BlobFeature >();
	} 
	
	// add feature to C-API's list
	if(!mvIMPACT_C::BL_FeatureSelect(m_blobFeatureList->getHandle(), m_blFeature, m_Param.getPointer(), mvIMPACT_C::BL_ADD))
	{ throwImpactError(); }
	
	// save a copy of this object in the feature map's list
	(m_blobFeatureList->getFeatureMap())[m_blFeature].push_back(*this);
	// beware: all changes made now to "this" will be discarded

	// save iterator in IdMap
	std::list<BlobFeature>::iterator tempIt = (m_blobFeatureList->getFeatureMap())[m_blFeature].end(); tempIt--;
	(m_blobFeatureList->getIdMap())[m_Id] = tempIt;
}


inline void BlobFeature::remove()
{
	BlobFeature save = *this;
	// copy current object, because below, the current object will
	// be destructed, so we must not access any member variables
	// of this !
	while(1) { // left with break
		// rotate the lists synchronously
		
		// pop first element
		if(!mvIMPACT_C::BL_FeatureSelect(save.m_blobFeatureList->getHandle(), save.m_blFeature, NULL, mvIMPACT_C::BL_REMOVE))
		{ throwImpactError(); }
		BlobFeature tempFeature = (save.m_blobFeatureList->getFeatureMap())[save.m_blFeature].front();
		(save.m_blobFeatureList->getFeatureMap())[save.m_blFeature].pop_front();
		// now it can be (it probably will be) that the current object has been 
		// destructed, so we are not allowed to access it any more

		if(tempFeature.m_Id == save.m_Id) { 
			// if the feature which was just removed is the feature we want to remove,
			// quit loop without pushing it again

			// erase from id map:
			(save.m_blobFeatureList->getIdMap()).erase(tempFeature.m_Id);

			break; 
		}
		
		// push element at the end
		if(!mvIMPACT_C::BL_FeatureSelect(
			tempFeature.m_blobFeatureList->getHandle(), 
			tempFeature.m_blFeature, 
			tempFeature.m_Param.getPointer(), 
			mvIMPACT_C::BL_ADD))
		{ throwImpactError(); }
		(save.m_blobFeatureList->getFeatureMap())[save.m_blFeature].push_back(tempFeature);
		
		// update id map
		std::list<BlobFeature>::iterator tempIt = 
			(save.m_blobFeatureList->getFeatureMap())[tempFeature.m_blFeature].end(); tempIt--;
		(save.m_blobFeatureList->getIdMap())[tempFeature.m_Id] = tempIt;
	}
}

inline mvIMPACT_C::BL_FEATURE BlobFeature::getBlFeature() const
{
	return m_blFeature;
}


inline BlobFeatureID BlobFeature::getId() const
{
	return m_Id;
}



inline void BlobFeatureList::dispose()
{
	(*m_counter)--;
	if (0 == *m_counter)
	{
		delete m_counter;
		if (!mvIMPACT_C::BL_BufFree(getHandle()))
		{ throwImpactError(); }
		delete m_FeatureMap;
		delete m_IdMap;
		delete m_idCounter;
	}
}



inline BlobFeatureList::BlobFeatureList() 
	{ 
		if (!mvIMPACT_C::BL_BufAlloc(
				&m_handle, 
				mvIMPACT_C::BL_FEATUREBUF))
		{ throwImpactError(); }
		m_counter = new int(1);
		m_idCounter = new int(0);
		m_FeatureMap = new std::map< mvIMPACT_C::BL_FEATURE, std::list< BlobFeature > >;
		m_IdMap = new std::map< BlobFeatureID, std::list<BlobFeature>::iterator >;
	}


inline BlobFeatureList::BlobFeatureList(const BlobFeatureList& r)
	{ 
		copy(r);
	}


inline BlobFeatureList& BlobFeatureList::operator=(const BlobFeatureList& r)
	{
		if(this != &r) 
		{
			dispose();
			copy(r);
		}
		return *this;
	}


inline BlobFeatureList::~BlobFeatureList()
	{ 
		// a destructor must never throw
		try
		{
			dispose();
		}
		catch(...)
		{
			mvIMPACT_C::IPL_Trace(" caught exception in destructor");
		}
	}


inline mvIMPACT_C::BL_BUFHANDLE BlobFeatureList::getHandle() const
{ return m_handle; }

inline std::map< mvIMPACT_C::BL_FEATURE, std::list< BlobFeature > > & BlobFeatureList::getFeatureMap()
{
	return *m_FeatureMap;
}

inline std::map< BlobFeatureID, std::list< BlobFeature >::iterator > & BlobFeatureList::getIdMap()
{
	return *m_IdMap;
}


inline BlobFeatureList& BlobFeatureList::setNeighborhoodMode(NeighborhoodMode neighborhoodMode)
	{
		mvIMPACT_C::IPL_LABEL_MODE labelMode;
		switch(neighborhoodMode) 
		{
			case nm4Connected: 
				labelMode = mvIMPACT_C::IPL_RECT4_3x3;
				break;
			case nm8Connected:
				labelMode = mvIMPACT_C::IPL_RECT8_3x3;
				break;				
			default:
				throwImpactException("Invalid value for NeighborhoodMode");
		}
		
		if (!mvIMPACT_C::BL_SetProcMode(
				getHandle(), 
				mvIMPACT_C::BLOC_CTLID_LATTICE_MODE, 
				labelMode))
			throwImpactError();
		return *this;
	}


inline NeighborhoodMode BlobFeatureList::getNeighborhoodMode() const
	{
		mvIMPACT_C::IPL_LABEL_MODE iplNeighborhoodMode;
		if (!mvIMPACT_C::BL_GetProcMode(
				getHandle(), 
				mvIMPACT_C::BLOC_CTLID_LATTICE_MODE, 
				(mvIMPACT_C::IPL_LONG*)&iplNeighborhoodMode))
			throwImpactError();
		switch (iplNeighborhoodMode) 
		{
		case mvIMPACT_C::IPL_RECT4_3x3: 
			return nm4Connected;

		case mvIMPACT_C::IPL_RECT8_3x3: 
			return nm8Connected;
			
		default:
			throwImpactException("Invalid value for NeighborhoodMode");
		}
	}

inline unsigned int BlobFeatureList::getStopAfterIndex() const
{
		mvIMPACT_C::IPL_ULONG iplStopIndex;
		if (!mvIMPACT_C::BL_GetProcMode(
				getHandle(), 
				mvIMPACT_C::BLOC_CTLID_STOP_AFTER_INDEX, 
				(mvIMPACT_C::IPL_LONG*)&iplStopIndex))
			throwImpactError();
		return (unsigned int)iplStopIndex;
}

inline BlobFeatureList& BlobFeatureList::setStopAfterIndex(unsigned int index)
{
		mvIMPACT_C::IPL_ULONG iplStopIndex = index;
	
		if (!mvIMPACT_C::BL_SetProcMode(
				getHandle(), 
				mvIMPACT_C::BLOC_CTLID_STOP_AFTER_INDEX, 
				iplStopIndex))
			throwImpactError();
			
		return *this;
}

inline bool BlobFeatureList::getKillBorderObjects() const
{
		mvIMPACT_C::IPL_BOOL iplKillBorderObjects;
		if (!mvIMPACT_C::BL_GetProcMode(
				getHandle(), 
				mvIMPACT_C::BLOC_CTLID_KILL_BORDER_OBJECTS, 
				(mvIMPACT_C::IPL_LONG*)&iplKillBorderObjects))
			throwImpactError();
		return (IPL_TRUE == iplKillBorderObjects) ? true : false;
}

inline BlobFeatureList& BlobFeatureList::setKillBorderObjects(bool kill)
{
		mvIMPACT_C::IPL_BOOL iplKillBorderObjects = kill ? IPL_TRUE : IPL_FALSE;
		if (!mvIMPACT_C::BL_SetProcMode(
				getHandle(), 
				mvIMPACT_C::BLOC_CTLID_KILL_BORDER_OBJECTS, 
				iplKillBorderObjects))
			throwImpactError();
		return *this;
}

inline unsigned int BlobFeatureList::getMinObjectSize() const
{
		mvIMPACT_C::IPL_ULONG iplMinObjectSize;
		if (!mvIMPACT_C::BL_GetProcMode(
				getHandle(), 
				mvIMPACT_C::BLOC_CTLID_MIN_SIZE, 
				(mvIMPACT_C::IPL_LONG*)&iplMinObjectSize))
			throwImpactError();
		return (unsigned int)iplMinObjectSize;
}

inline BlobFeatureList& BlobFeatureList::setMinObjectSize(unsigned int minSize)
{
		mvIMPACT_C::IPL_ULONG iplMinObjectSize = minSize;
		if (!mvIMPACT_C::BL_SetProcMode(
				getHandle(), 
				mvIMPACT_C::BLOC_CTLID_MIN_SIZE, 
				iplMinObjectSize))
			throwImpactError();
		return *this;
}

inline unsigned int BlobFeatureList::getMaxObjectSize() const
{
		mvIMPACT_C::IPL_ULONG iplMaxObjectSize;
		if (!mvIMPACT_C::BL_GetProcMode(
				getHandle(), 
				mvIMPACT_C::BLOC_CTLID_MAX_SIZE, 
				(mvIMPACT_C::IPL_LONG*)&iplMaxObjectSize))
			throwImpactError();
		return (unsigned int)iplMaxObjectSize;
}

inline BlobFeatureList& BlobFeatureList::setMaxObjectSize(unsigned int maxSize)
{
		mvIMPACT_C::IPL_ULONG iplMaxObjectSize = maxSize;
		if (!mvIMPACT_C::BL_SetProcMode(
				getHandle(), 
				mvIMPACT_C::BLOC_CTLID_MAX_SIZE, 
				iplMaxObjectSize))
			throwImpactError();
		return *this;
}

inline int BlobFeatureList::getThreshold() const
{
		mvIMPACT_C::IPL_LONG iplThreshold;
		if (!mvIMPACT_C::BL_GetProcMode(
				getHandle(), 
				mvIMPACT_C::BLOC_CTLID_THRESHOLD, 
				(mvIMPACT_C::IPL_LONG*)&iplThreshold))
			throwImpactError();
		return iplThreshold;
}
	
inline BlobFeatureList& BlobFeatureList::setThreshold(int threshold)
{
		mvIMPACT_C::IPL_LONG iplThreshold = threshold;
		if (!mvIMPACT_C::BL_SetProcMode(
				getHandle(), 
				mvIMPACT_C::BLOC_CTLID_THRESHOLD, 
				iplThreshold))
			throwImpactError();
		return *this;
}

inline PolarityMode BlobFeatureList::getPolarityMode() const
{
		mvIMPACT_C::IPL_POLARITY iplPolarityMode;
		if (!mvIMPACT_C::BL_GetProcMode(
				getHandle(), 
				mvIMPACT_C::BLOC_CTLID_OBJECTSTATE, 
				(mvIMPACT_C::IPL_LONG*)&iplPolarityMode))
			throwImpactError();
		return PolarityMode(iplPolarityMode);
}

inline BlobFeatureList& BlobFeatureList::setPolarityMode(PolarityMode polarityMode)
{
		mvIMPACT_C::IPL_POLARITY iplPolarityMode = 
			mvIMPACT_C::IPL_POLARITY(polarityMode);
		if (!mvIMPACT_C::BL_SetProcMode(
				getHandle(), 
				mvIMPACT_C::BLOC_CTLID_OBJECTSTATE, 
				iplPolarityMode))
			throwImpactError();
		return *this;
}



inline Position BlobFeatureList::getCoordinateSystem() const
{
	mvIMPACT_C::IPL_DOUBLE orgX = 0.;
	mvIMPACT_C::IPL_DOUBLE orgY = 0.;
	mvIMPACT_C::IPL_DOUBLE scaleFacX = 0.;
	mvIMPACT_C::IPL_DOUBLE scaleFacY = 0.;
	mvIMPACT_C::IPL_DOUBLE rotationAngle = 0.;

	if(!mvIMPACT_C::BL_GetCrdSystem(
		getHandle(),
		&orgX,
		&orgY,
		&scaleFacX,
		&scaleFacY,
		&rotationAngle)
		)
	{ throwImpactError(); }
	return Position((int)orgX, (int)orgY, scaleFacX, scaleFacY, rotationAngle);
}


inline BlobFeatureList& BlobFeatureList::setCoordinateSystem(const Position& pos) 
{
	if(!mvIMPACT_C::BL_SetCrdSystem (
		getHandle(),
		(mvIMPACT_C::IPL_DOUBLE)pos.getX(),
		(mvIMPACT_C::IPL_DOUBLE)pos.getY(),
		pos.getScaleX(),
		pos.getScaleY(),
		pos.getAngle())
		)
	{ throwImpactError(); }
	return *this;
}

inline void BlobFeatureList::transfer(BlobResultList& dest) const
{
	// recurse through the lists in the FeatureMap and assign
	// correct subItemIndex
	std::map< mvIMPACT_C::BL_FEATURE, std::list< BlobFeature > >::iterator it;
	for(it = (*m_FeatureMap).begin(); it != (*m_FeatureMap).end(); it++) {
		std::list< BlobFeature >::iterator jt;
		int j;
		for(jt = (*it).second.begin(), j = 0; jt != (*it).second.end(); jt++, j++) {
			assert(jt->getBlFeature() == (*it).first);
			dest.addFeature(jt->getId(), BlobResult(jt->getBlFeature(), j));
		}
	}
}



inline void BlobFeatureList::copy(const BlobFeatureList &src)
{
	if( this != &src) {
		m_counter = src.m_counter;
		m_handle = src.m_handle;
		m_FeatureMap = src.m_FeatureMap;
		m_IdMap = src.m_IdMap;
		m_idCounter = src.m_idCounter;

		(*m_counter)++;
	}
}

inline BlobFeatureID BlobFeatureList::addFeature(mvIMPACT_C::BL_FEATURE blFeature, void* param, int paramSize)
{
	(*m_idCounter)++;
	BlobFeature tempFeature(this, blFeature, param, paramSize, (*m_idCounter));
	// OK, this is a little bit strange: This constructor creates a copy of itself in the
	// appropriate list of m_FeatureMap. Afterwards tempFeature gets obsolete and can be
	// discarded
	
	return (*m_idCounter);
}

inline BlobFeatureList& BlobFeatureList::removeFeature(BlobFeatureID id)
{
	(*m_IdMap)[id]->remove();
	return *this;
}

	
inline BlobFeatureID BlobFeatureList::addArea()
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_AREA;
	return addFeature(blFeature, NULL, 0);
}

inline BlobFeatureID BlobFeatureList::addPerimeter()
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_PERIMETER;
	return addFeature(blFeature, NULL, 0);
}

	
inline BlobFeatureID BlobFeatureList::addRoundness()
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_ROUNDNESS;
	return addFeature(blFeature, NULL, 0);
}

	
inline BlobFeatureID BlobFeatureList::addExtremeBox()
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_EXTREME_BOX;
	return addFeature(blFeature, NULL, 0);
}


inline BlobFeatureID BlobFeatureList::addFirstPoint()
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_FIRST_POINT;
	return addFeature(blFeature, NULL, 0);
}


inline BlobFeatureID BlobFeatureList::addEulerNumber()
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_EULER_NUMBER;
	return addFeature(blFeature, NULL, 0);
}


inline BlobFeatureID BlobFeatureList::addNumberHoles()
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_NUMBER_HOLES;
	return addFeature(blFeature, NULL, 0);
}

	
inline BlobFeatureID BlobFeatureList::addMajorAxis(int steps, bool grayscaleImage)
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_MAJOR_AXIS;

	if (grayscaleImage) { blFeature = (mvIMPACT_C::BL_FEATURE) (blFeature | mvIMPACT_C::BL_GRAYSCALE); }

	mvIMPACT_C::IPL_DOUBLE iplSteps = steps;
	return addFeature(blFeature, &iplSteps, sizeof(iplSteps));
}

	
inline BlobFeatureID BlobFeatureList::addMajorAxisAngle(int steps, bool grayscaleImage)
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_MAJOR_AXIS_ANGLE;

	if (grayscaleImage) { blFeature = (mvIMPACT_C::BL_FEATURE) (blFeature | mvIMPACT_C::BL_GRAYSCALE); }

	mvIMPACT_C::IPL_DOUBLE iplSteps = steps;
	return addFeature(blFeature, &iplSteps, sizeof(iplSteps));
}

	
inline BlobFeatureID BlobFeatureList::addMajorAxisLength(int steps, bool grayscaleImage)
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_MAJOR_AXIS_LENGTH;

	if (grayscaleImage) { blFeature = (mvIMPACT_C::BL_FEATURE) (blFeature | mvIMPACT_C::BL_GRAYSCALE); }

	mvIMPACT_C::IPL_DOUBLE iplSteps = steps;
	return addFeature(blFeature, &iplSteps, sizeof(iplSteps));
}

	
inline BlobFeatureID BlobFeatureList::addMinorAxis(int steps, bool grayscaleImage)
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_MINOR_AXIS;

	if (grayscaleImage) { blFeature = (mvIMPACT_C::BL_FEATURE) (blFeature | mvIMPACT_C::BL_GRAYSCALE); }

	mvIMPACT_C::IPL_DOUBLE iplSteps = steps;
	return addFeature(blFeature, &iplSteps, sizeof(iplSteps));
}

	
inline BlobFeatureID BlobFeatureList::addMinorAxisAngle(
		int steps, 
		bool grayscaleImage)
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_MINOR_AXIS_ANGLE;

	if (grayscaleImage) { blFeature = (mvIMPACT_C::BL_FEATURE) (blFeature | mvIMPACT_C::BL_GRAYSCALE); }

	mvIMPACT_C::IPL_DOUBLE iplSteps = steps;
	return addFeature(blFeature, &iplSteps, sizeof(iplSteps));
}


inline BlobFeatureID BlobFeatureList::addMinorAxisLength(
		int steps, 
		bool grayscaleImage)
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_MINOR_AXIS_LENGTH;

	if (grayscaleImage) { blFeature = (mvIMPACT_C::BL_FEATURE) (blFeature | mvIMPACT_C::BL_GRAYSCALE); }

	mvIMPACT_C::IPL_DOUBLE iplSteps = steps;
	return addFeature(blFeature, &iplSteps, sizeof(iplSteps));
}


inline BlobFeatureID BlobFeatureList::addFeretMax(
		int steps)
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_FERET_MAX;


	mvIMPACT_C::IPL_DOUBLE iplSteps = steps;
	return addFeature(blFeature, &iplSteps, sizeof(iplSteps));
}

	
inline BlobFeatureID BlobFeatureList::addFeretMaxAngle(
		int steps)
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_FERET_MAX_ANGLE;

	mvIMPACT_C::IPL_DOUBLE iplSteps = steps;
	return addFeature(blFeature, &iplSteps, sizeof(iplSteps));
}

	
inline BlobFeatureID BlobFeatureList::addFeretMin(
		int steps)
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_FERET_MIN;

	mvIMPACT_C::IPL_DOUBLE iplSteps = steps;
	return addFeature(blFeature, &iplSteps, sizeof(iplSteps));
}


inline BlobFeatureID BlobFeatureList::addFeretMinAngle(
		int steps)
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_FERET_MIN_ANGLE;

	mvIMPACT_C::IPL_DOUBLE iplSteps = steps;
	return addFeature(blFeature, &iplSteps, sizeof(iplSteps));
}

	
inline BlobFeatureID BlobFeatureList::addFeretDiameter(double angle)
{
	if(0.0 == angle) {
		return addFeature(mvIMPACT_C::BL_AR_FERET_0, NULL, 0);
	} else if(90.0 == angle) {
		return addFeature(mvIMPACT_C::BL_AR_FERET_90, NULL, 0);
	} else {
		mvIMPACT_C::IPL_DOUBLE iplAngle = angle;
		return addFeature(mvIMPACT_C::BL_AR_FERET_GEN, &iplAngle, sizeof(iplAngle));;
	}
}

	
inline BlobFeatureID BlobFeatureList::addFeretRatio(
		int steps)
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_FERET_RATIO;

	mvIMPACT_C::IPL_DOUBLE iplSteps = steps;
	return addFeature(blFeature, &iplSteps, sizeof(iplSteps));
}

	
inline BlobFeatureID BlobFeatureList::addPerimeterConvex(
		int steps)
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_PERIMETER_CONVEX;

	mvIMPACT_C::IPL_DOUBLE iplSteps = steps;
	return addFeature(blFeature, &iplSteps, sizeof(iplSteps));
}


inline BlobFeatureID BlobFeatureList::addRoughness(
		int steps)
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_ROUGHNESS;

	mvIMPACT_C::IPL_DOUBLE iplSteps = steps;
	return addFeature(blFeature, &iplSteps, sizeof(iplSteps));
}

	
inline BlobFeatureID BlobFeatureList::addAreaConvex(
		int steps)
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_AREA_CONVEX;

	mvIMPACT_C::IPL_DOUBLE iplSteps = steps;
	return addFeature(blFeature, &iplSteps, sizeof(iplSteps));
}

	
inline BlobFeatureID BlobFeatureList::addChordSum()
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_AREA_CONVEX;

	return addFeature(blFeature, NULL, 0);
}


inline BlobFeatureID BlobFeatureList::addAreaHoles()
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_AREA_HOLES;

	return addFeature(blFeature, NULL, 0);
}

	
inline BlobFeatureID BlobFeatureList::addAreaPerforation()
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_AREA_PERFORATION;

	return addFeature(blFeature, NULL, 0);
}

	
inline BlobFeatureID BlobFeatureList::addAreaHolesMean()
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_AREA_HOLES_MEAN;

	return addFeature(blFeature, NULL, 0);
}

	
inline BlobFeatureID BlobFeatureList::addFiberLength()
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_FIBER_LENGTH;

	return addFeature(blFeature, NULL, 0);
}

	
inline BlobFeatureID BlobFeatureList::addFiberWidth()
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_FIBER_WIDTH;

	return addFeature(blFeature, NULL, 0);
}

	
inline BlobFeatureID BlobFeatureList::addConvexhullPoints(
		int steps)
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_CONVEXHULL_POINTS;

	mvIMPACT_C::IPL_DOUBLE iplSteps = steps;
	return addFeature(blFeature, &iplSteps, sizeof(iplSteps));
}

	
inline BlobFeatureID BlobFeatureList::addMin()
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_MIN;

	return addFeature(blFeature, NULL, 0);
}

	
inline BlobFeatureID BlobFeatureList::addMax()
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_MAX;

	return addFeature(blFeature, NULL, 0);
}

	
inline BlobFeatureID BlobFeatureList::addSum()
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_SUM;

	return addFeature(blFeature, NULL, 0);
}

	
inline BlobFeatureID BlobFeatureList::addSquaredSum()
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_SQUARED_SUM;

	return addFeature(blFeature, NULL, 0);
}

	
inline BlobFeatureID BlobFeatureList::addMean()
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_MEAN;

	return addFeature(blFeature, NULL, 0);
}


inline BlobFeatureID BlobFeatureList::addStddev()
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_STDDEV;

	return addFeature(blFeature, NULL, 0);
}

	
inline BlobFeatureID BlobFeatureList::addSkewness()
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_SKEWNESS;

	return addFeature(blFeature, NULL, 0);
}

	
inline BlobFeatureID BlobFeatureList::addKurtosis()
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_KURTOSIS;

	return addFeature(blFeature, NULL, 0);
}

	
inline BlobFeatureID BlobFeatureList::addSkewnessFac()
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_SKEWNESS_FAC;

	return addFeature(blFeature, NULL, 0);
}

	
inline BlobFeatureID BlobFeatureList::addKurtosisFac()
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_KURTOSIS_FAC;

	return addFeature(blFeature, NULL, 0);
}


inline BlobFeatureID BlobFeatureList::addEntropy()
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_ENTROPY;

	return addFeature(blFeature, NULL, 0);
}

	
inline BlobFeatureID BlobFeatureList::addVariationCoefficient()
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_VARCOEF;

	return addFeature(blFeature, NULL, 0);
}


inline BlobFeatureID BlobFeatureList::addHistogram()
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_HISTOGRAM;

	return addFeature(blFeature, NULL, 0);
}

	
inline BlobFeatureID BlobFeatureList::addCooccurrenceMatrix(
		int dx,
		int dy,
		int size)
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_CCMTX;

	mvIMPACT_C::BL_CCMTX ccmtx;
	ccmtx.dx = dx;
	ccmtx.dy = dy;
	ccmtx.Size = size;
	
	return addFeature(blFeature, &ccmtx, sizeof(ccmtx));
}

	
inline BlobFeatureID BlobFeatureList::addEnergy(
		int dx,
		int dy,
		int size)
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_ENERGY;

	mvIMPACT_C::BL_CCMTX ccmtx;
	ccmtx.dx = dx;
	ccmtx.dy = dy;
	ccmtx.Size = size;
	
	return addFeature(blFeature, &ccmtx, sizeof(ccmtx));
}

	
inline BlobFeatureID BlobFeatureList::addContrast(
		int dx,
		int dy,
		int size)
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_CONTRAST;

	mvIMPACT_C::BL_CCMTX ccmtx;
	ccmtx.dx = dx;
	ccmtx.dy = dy;
	ccmtx.Size = size;
	
	return addFeature(blFeature, &ccmtx, sizeof(ccmtx));
}

	
inline BlobFeatureID BlobFeatureList::addEntropyCooccurrenceMatrix(
		int dx,
		int dy,
		int size)
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_ENTROPY_CCMTX;

	mvIMPACT_C::BL_CCMTX ccmtx;
	ccmtx.dx = dx;
	ccmtx.dy = dy;
	ccmtx.Size = size;
	
	return addFeature(blFeature, &ccmtx, sizeof(ccmtx));
}

	
inline BlobFeatureID BlobFeatureList::addCenterGravityX(
		bool grayscaleImage)
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_CENTER_GRAVITY_X;

	if (grayscaleImage) { blFeature = (mvIMPACT_C::BL_FEATURE) (blFeature | mvIMPACT_C::BL_GRAYSCALE); }

	return addFeature(blFeature, NULL, 0);
}

	
inline BlobFeatureID BlobFeatureList::addCenterGravityY(
		bool grayscaleImage)
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_CENTER_GRAVITY_Y;

	if (grayscaleImage) { blFeature = (mvIMPACT_C::BL_FEATURE) (blFeature | mvIMPACT_C::BL_GRAYSCALE); }

	return addFeature(blFeature, NULL, 0);
}

	
inline BlobFeatureID BlobFeatureList::addCenterGravity(
		bool grayscaleImage)
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_CENTER_GRAVITY;

	if (grayscaleImage) { blFeature = (mvIMPACT_C::BL_FEATURE) (blFeature | mvIMPACT_C::BL_GRAYSCALE); }

	return addFeature(blFeature, NULL, 0);
}


inline BlobFeatureID BlobFeatureList::addMomentCentral(
		int x, 
		int y, 
		bool grayscaleImage)
{
	mvIMPACT_C::BL_FEATURE blFeature;
	if(0 == x && 1 == y) {
		blFeature = mvIMPACT_C::BL_AR_MOMENT_CENTRAL_X0_Y1;
	} else if(1 == x && 0 == y) {
		blFeature = mvIMPACT_C::BL_AR_MOMENT_CENTRAL_X1_Y0;
	} else if(2 == x && 0 == y) {
		blFeature = mvIMPACT_C::BL_AR_MOMENT_CENTRAL_X2_Y0;
	} else if(1 == x && 1 == y) {
		blFeature = mvIMPACT_C::BL_AR_MOMENT_CENTRAL_X1_Y1;
	} else if(0 == x && 2 == y) {
		blFeature = mvIMPACT_C::BL_AR_MOMENT_CENTRAL_X0_Y2;
	} else if(3 == x && 0 == y) {
		blFeature = mvIMPACT_C::BL_AR_MOMENT_CENTRAL_X3_Y0;
	} else if(2 == x && 1 == y) {
		blFeature = mvIMPACT_C::BL_AR_MOMENT_CENTRAL_X2_Y1;
	} else if(1 == x && 2 == y) {
		blFeature = mvIMPACT_C::BL_AR_MOMENT_CENTRAL_X1_Y2;
	} else if(0 == x && 3 == y) {
		blFeature = mvIMPACT_C::BL_AR_MOMENT_CENTRAL_X0_Y3;
	} else if(2 == x && 2 == y) {
		blFeature = mvIMPACT_C::BL_AR_MOMENT_CENTRAL_X2_Y2;
	} else {
		throwImpactException("Invalid combination of x and/or y");
	}
	if (grayscaleImage) { blFeature = (mvIMPACT_C::BL_FEATURE) (blFeature | mvIMPACT_C::BL_GRAYSCALE); }

	return addFeature(blFeature, NULL, 0);
}

    
inline BlobFeatureID BlobFeatureList::addAxisSymmetryAngle(
		bool grayscaleImage)
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_AXIS_SYMMETRY_ANGLE;

	if (grayscaleImage) { blFeature = (mvIMPACT_C::BL_FEATURE) (blFeature | mvIMPACT_C::BL_GRAYSCALE); }

	return addFeature(blFeature, NULL, 0);
}

	
inline BlobFeatureID BlobFeatureList::addAxisSymmetryAngle90(
		bool grayscaleImage)
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_AXIS_SYMMETRY_ANGLE_90;

	if (grayscaleImage) { blFeature = (mvIMPACT_C::BL_FEATURE) (blFeature | mvIMPACT_C::BL_GRAYSCALE); }

	return addFeature(blFeature, NULL, 0);
}

	
inline BlobFeatureID BlobFeatureList::addPolarDistMax(
		bool grayscaleImage)
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_POLAR_DIST_MAX;

	if (grayscaleImage) { blFeature = (mvIMPACT_C::BL_FEATURE) (blFeature | mvIMPACT_C::BL_GRAYSCALE); }

	return addFeature(blFeature, NULL, 0);
}

	
inline BlobFeatureID BlobFeatureList::addPolarDistMin(
		bool grayscaleImage)
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_POLAR_DIST_MIN;

	if (grayscaleImage) { blFeature = (mvIMPACT_C::BL_FEATURE) (blFeature | mvIMPACT_C::BL_GRAYSCALE); }

	return addFeature(blFeature, NULL, 0);
}

	
inline BlobFeatureID BlobFeatureList::addPolarDist(
		int steps,
		bool grayscaleImage)
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_POLAR_DIST;

	if (grayscaleImage) { blFeature = (mvIMPACT_C::BL_FEATURE) (blFeature | mvIMPACT_C::BL_GRAYSCALE); }
	

	mvIMPACT_C::IPL_DOUBLE iplSteps = steps;
	return addFeature(blFeature, &iplSteps, sizeof(iplSteps));
}

inline BlobFeatureID BlobFeatureList::addNbOutlinePoints()
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_NB_OUTLINE_POINTS;

	return addFeature(blFeature, NULL, 0);
}

inline BlobFeatureID BlobFeatureList::addOutlinePoints()
{
	//automatically add feature "number of outline points"
	this->addNbOutlinePoints();

	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_OUTLINE_POINTS;

	return addFeature(blFeature, NULL, 0);
}

inline BlobFeatureID BlobFeatureList::addNbScanlines()
{
	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_NB_SCANLINES;

	return addFeature(blFeature, NULL, 0);
}

inline BlobFeatureID BlobFeatureList::addScanlines()
{
	//automatically add feature "number of scanlines"
	this->addNbScanlines();

	mvIMPACT_C::BL_FEATURE blFeature = mvIMPACT_C::BL_AR_SCANLINES;

	return addFeature(blFeature, NULL, 0);
}

inline BlobResultList::BlobResultList()
	{ 
		if (!mvIMPACT_C::BL_BufAlloc(
				&m_handle, 
				mvIMPACT_C::BL_RESULTBUF))
		{ throwImpactError(); }
		m_counter = new int(1);
		m_map = new std::map< BlobFeatureID, BlobResult >;
	}


inline BlobResultList::BlobResultList(const BlobResultList& r)
{
	copy(r);
}

inline int BlobResultList::getBands() const
	{
		double count=0.0;
		if (!mvIMPACT_C::BL_ResultBufInquire(
				getHandle(), 
				0, 0, 0, (mvIMPACT_C::BL_FEATURE)-1, 0, 
				mvIMPACT_C::BL_GET_NB_BANDS,
				&count))
			throwImpactError();
		return static_cast<int>(count);
	}


inline int BlobResultList::getSlices() const
	{
		double count=0.0;
		if (!mvIMPACT_C::BL_ResultBufInquire(
				getHandle(), 
				0, 0, 0, (mvIMPACT_C::BL_FEATURE)-1, 0, 
				mvIMPACT_C::BL_GET_NB_Z_SLICES,
				&count))
			throwImpactError();
		return static_cast<int>(count);
	}


inline int BlobResultList::getFrames() const
	{
		double count=0.0;
		if (!mvIMPACT_C::BL_ResultBufInquire(
				getHandle(), 
				0, 0, 0, (mvIMPACT_C::BL_FEATURE)-1, 0, 
				mvIMPACT_C::BL_GET_NB_TIME_SLICES,
				&count))
			throwImpactError();
		return static_cast<int>(count);
	}

inline int BlobResultList::getNumberOfBlobs(int band, int slice, int frame) const
	{
		double count=0.0;
		if (!mvIMPACT_C::BL_ResultBufInquire(
				getHandle(), 
				band, slice, frame, (mvIMPACT_C::BL_FEATURE)-1, 0, 
				mvIMPACT_C::BL_GET_NB_BLOBS,
				&count))
			throwImpactError();
		return static_cast<int>(count);
	}

inline int BlobResultList::getDataType(BlobFeatureID id) const
{
	BlobResult blobResult = (*m_map)[id];

	double datatype=0.0;
	if (!mvIMPACT_C::BL_ResultBufInquire(
			getHandle(), 
			0, 0, 0, 
			blobResult.getFeature(),
			0,
			mvIMPACT_C::BL_GET_DATATYPE,
			&datatype))
			throwImpactError();
		return static_cast<int>(datatype);

}


inline int BlobResultList::getNumberOfFeatures() const
	{
		double count=0.0;
		if (!mvIMPACT_C::BL_ResultBufInquire(
				getHandle(), 
				0, 0, 0, (mvIMPACT_C::BL_FEATURE)-1, 0, 
				mvIMPACT_C::BL_GET_NB_FEATURES,
				&count))
			throwImpactError();
		return static_cast<int>(count);
	}

inline BlobResultList& BlobResultList::operator=(const BlobResultList & src)
{
	if(this != &src)
	{
		dispose();
		copy(src);
	}
	return *this;
}

inline mvIMPACT_C::BL_BUFHANDLE BlobResultList::getHandle() const
{ return m_handle; }

inline void BlobResultList::dispose()
{
	(*m_counter)--;
	if( 0 == *m_counter ) {
		delete m_counter;
		if(!mvIMPACT_C::BL_BufFree(m_handle))
		{ throwImpactError(); }
		delete m_map;
	}
}
	
inline void BlobResultList::copy(const BlobResultList &src)
{
	m_counter = src.m_counter;
	m_handle = src.m_handle;
	m_map = src.m_map;

	(*m_counter)++;
}

inline BlobResultList::~BlobResultList()
	{ 
		// a destructor must never throw
		try
		{
			dispose();
		}
		catch(...)
		{
			mvIMPACT_C::IPL_Trace(" caught exception in destructor");
		}
	}
	
inline int BlobResultList::getNumberOfResultItems(BlobFeatureID id) const
{
	BlobResult blobResult = (*m_map)[id];

	double count=0.0;
	if (!mvIMPACT_C::BL_ResultBufInquire(
			getHandle(), 
			0, 0, 0, 
			blobResult.getFeature(),
			0,
			mvIMPACT_C::BL_GET_NB_ITEMS,
			&count))
			throwImpactError();
		return static_cast<int>(count);

}
	

inline std::vector<double> BlobResultList::getSingleResult(BlobFeatureID id, int index, int band, int slice, int frame) const
{
	BlobResult blobResult = (*m_map)[id];

	int nbitems = getNumberOfResultItems(id);
	if (blobResult.getFeature() == mvIMPACT_C::BL_AR_OUTLINE_POINTS ||
		blobResult.getFeature() == mvIMPACT_C::BL_AR_SCANLINES)
	{
		mvIMPACT_C::BL_FEATURE feature = blobResult.getFeature() == mvIMPACT_C::BL_AR_OUTLINE_POINTS ? 
										mvIMPACT_C::BL_AR_NB_OUTLINE_POINTS : mvIMPACT_C::BL_AR_NB_SCANLINES; 
		mvIMPACT_C::IPL_ULONG items = 0;
		if (!mvIMPACT_C::BL_GetSingleResultBufData(
							getHandle(), 
							band, slice, frame, 
							feature,
							0, 
							index,
							&items))
							throwImpactError();
		if (feature == mvIMPACT_C::BL_AR_NB_OUTLINE_POINTS)
			nbitems = items * 2; // x,y
		else
			nbitems = items * 3; // y, xl, xr
	}
	DynArray<mvIMPACT_C::IPL_DOUBLE, double> data(nbitems);

	if (getDataType(id) == mvIMPACT_C::IPL_DTULONG)
	{
		//int nbitems = getNumberOfResultItems(id);
		mvIMPACT_C::IPL_ULONG *res = new mvIMPACT_C::IPL_ULONG [nbitems];
		if(!mvIMPACT_C::BL_GetSingleResultBufData(
					getHandle(), 
					band, slice, frame, 
					blobResult.getFeature(),
					blobResult.getSubItemIndex(), 
					index,
					res))
			{ 
				if (res) delete res;
				throwImpactError();
			}

		for(int i = 0; i < nbitems; i++) 
			data.getPointer()[i] = res[i];

		if (res)
			delete res;
	}
	else
	{
		if(!mvIMPACT_C::BL_GetSingleResultBufData(
					getHandle(), 
					band, slice, frame, 
					blobResult.getFeature(),
					blobResult.getSubItemIndex(), 
					index,
					data.getPointer()))
			{ throwImpactError(); }
	}

	return data.asVector();
}


inline std::vector<double> BlobResultList::getResultItem(
		BlobFeatureID id, int itemIndex, int band, int slice, int frame) const
{
	BlobResult blobResult = (*m_map)[id];

	int blobCount = getNumberOfBlobs(band, slice, frame);
	int itemCount = getNumberOfResultItems(id);
	
	DynArray<mvIMPACT_C::IPL_DOUBLE, double> data(blobCount*itemCount);

	std::vector<double> result;
	if (getDataType(id) == mvIMPACT_C::IPL_DTULONG)
	{
		mvIMPACT_C::IPL_ULONG *res = new mvIMPACT_C::IPL_ULONG [itemCount * blobCount];
		if(!mvIMPACT_C::BL_GetResultBufData (
				getHandle(),
				band, slice, frame,
				blobResult.getFeature(),
				blobResult.getSubItemIndex(), 
				res))
			{ 
				if (res) delete res;
				throwImpactError();
			}

		for(int i = itemIndex; i < itemCount*blobCount; i+=itemCount) 
			result.push_back(res[i]);

		if (res)
			delete res;
	}
	else
	{
		if(!mvIMPACT_C::BL_GetResultBufData (
				getHandle(),
				band, slice, frame,
				blobResult.getFeature(),
				blobResult.getSubItemIndex(), 
				data.getPointer()))
			{ throwImpactError(); }

		for(int i = itemIndex; i < itemCount*blobCount; i+=itemCount) {
			result.push_back(data.getPointer()[i]);
		}
	}

		
	return result;
}

inline double BlobResultList::getSingleResultItem(
					BlobFeatureID id, int index, 
					int itemIndex, int band, int slice, int frame) const
{
	return getSingleResult(id, index, band, slice, frame).at(itemIndex);
}


inline void BlobResultList::addFeature(BlobFeatureID id, BlobResult result) const
{
	(*m_map)[id] = result;
}

inline BlobLocator::BlobLocator()
{
	m_neighborhoodMode = nm8Connected;
	m_stopAfterIndex = 0;
	m_killBorderObjects = false;
	m_minObjectSize = 0;
	m_maxObjectSize = -1;
	m_polarityMode = pmWhiteOnBlack;
}


inline NeighborhoodMode BlobLocator::getNeighborhoodMode() const
{
	return m_neighborhoodMode;
}

inline BlobLocator& BlobLocator::setNeighborhoodMode(NeighborhoodMode mode)
{
	m_neighborhoodMode = mode;
	return *this;
}

inline int BlobLocator::getStopAfterIndex() const
{
	return m_stopAfterIndex;
}

inline BlobLocator& BlobLocator::setStopAfterIndex(int index)
{
	m_stopAfterIndex = index;
	return *this;
}

inline bool BlobLocator::getKillBorderObjects() const
{
	return m_killBorderObjects;
}


inline BlobLocator& BlobLocator::setKillBorderObjects(bool kill)
{
	m_killBorderObjects = kill;
	return *this;
}

inline int BlobLocator::getMinObjectSize() const
{
	return m_minObjectSize;
}

inline BlobLocator& BlobLocator::setMinObjectSize(int minSize)
{
	m_minObjectSize = minSize;
	return *this;
}

inline int BlobLocator::getMaxObjectSize() const
{
	return m_maxObjectSize;
}

inline int BlobLocator::getThreshold() const
{
	return m_threshold;
}
	
inline BlobLocator& BlobLocator::setThreshold(int threshold)
{
	m_threshold = threshold;
	return *this;
}

inline BlobLocator& BlobLocator::setMaxObjectSize(int maxSize)
{
	m_maxObjectSize = maxSize;
	return *this;
}

inline PolarityMode BlobLocator::getPolarityMode() const
{
	return m_polarityMode;
}

inline BlobLocator& BlobLocator::setPolarityMode(PolarityMode polarityMode)
{
	m_polarityMode = polarityMode;;
	return *this;
}

inline void BlobLocator::transferConfiguration() const
{
	mvIMPACT_C::IPL_LABEL_MODE iplLabelMode;
	switch(m_neighborhoodMode) 
	{
	case nm4Connected: 
		iplLabelMode = mvIMPACT_C::IPL_RECT4_3x3;
		break;
	case nm8Connected:
		iplLabelMode = mvIMPACT_C::IPL_RECT8_3x3;
		break;				
	default:
		throwImpactException("Invalid value for NeighborhoodMode");
	}
	if(!mvIMPACT_C::BLOC_SetProcMode(mvIMPACT_C::BLOC_CTLID_LATTICE_MODE, (mvIMPACT_C::IPL_LONG)iplLabelMode)) 
	{ throwImpactError(); }

	if(!mvIMPACT_C::BLOC_SetProcMode(mvIMPACT_C::BLOC_CTLID_STOP_AFTER_INDEX, (mvIMPACT_C::IPL_LONG)m_stopAfterIndex)) 
	{ throwImpactError(); }

	if(!mvIMPACT_C::BLOC_SetProcMode(mvIMPACT_C::BLOC_CTLID_KILL_BORDER_OBJECTS, (mvIMPACT_C::IPL_LONG)m_killBorderObjects)) 
	{ throwImpactError(); }

	if(!mvIMPACT_C::BLOC_SetProcMode(mvIMPACT_C::BLOC_CTLID_MIN_SIZE, (mvIMPACT_C::IPL_LONG)m_minObjectSize)) 
	{ throwImpactError(); }

	if(!mvIMPACT_C::BLOC_SetProcMode(mvIMPACT_C::BLOC_CTLID_MAX_SIZE, (mvIMPACT_C::IPL_LONG)m_maxObjectSize)) 
	{ throwImpactError(); }
	
	if(!mvIMPACT_C::BLOC_SetProcMode(mvIMPACT_C::BLOC_CTLID_THRESHOLD, (mvIMPACT_C::IPL_LONG)m_threshold)) 
	{ throwImpactError(); }

	if(!mvIMPACT_C::BLOC_SetProcMode(mvIMPACT_C::BLOC_CTLID_OBJECTSTATE, (mvIMPACT_C::IPL_LONG)m_polarityMode)) 
	{ throwImpactError(); }
	
}

inline BlobLocatorResultList::BlobLocatorResultList()
		: m_counter(new int(1)), m_handle(0)
{
}
	
inline BlobLocatorResultList::BlobLocatorResultList(mvIMPACT_C::BLOC_BUFHANDLE handle)
		: m_counter(new int(1)), m_handle(handle)
{
}

inline mvIMPACT_C::BLOC_BUFHANDLE BlobLocatorResultList::getHandle() const
{
	return m_handle;
}


inline BlobLocatorResultList::BlobLocatorResultList(const BlobLocatorResultList& src)
{
	copy(src);
}


inline BlobLocatorResultList::~BlobLocatorResultList()
{
	dispose();
}

inline BlobLocatorResultList& BlobLocatorResultList::operator=(const BlobLocatorResultList& src)
{
	if(this != &src) 
	{
		dispose();
		copy(src);
	}
	return *this;
}


inline int BlobLocatorResultList::getBands() const
{
	mvIMPACT_C::IPL_LONG dummy;
	mvIMPACT_C::IPL_LONG bands;
	
	if(!mvIMPACT_C::BLOC_InquirePlanes(getHandle(), &bands, &dummy, &dummy))
	{ throwImpactError(); }
	
	return bands;
}

inline int BlobLocatorResultList::getSlices() const
{
	mvIMPACT_C::IPL_LONG dummy;
	mvIMPACT_C::IPL_LONG slices;
	
	if(!mvIMPACT_C::BLOC_InquirePlanes(getHandle(), &dummy, &slices, &dummy))
	{ throwImpactError(); }
	
	return slices;
}

inline int BlobLocatorResultList::getFrames() const
{
	mvIMPACT_C::IPL_LONG dummy;
	mvIMPACT_C::IPL_LONG frames;
	
	if(!mvIMPACT_C::BLOC_InquirePlanes(getHandle(), &dummy, &dummy, &frames))
	{ throwImpactError(); }
	
	return frames;
}

inline int BlobLocatorResultList::getNumberOfBlobs(int band, int slice, int frame) const
{
	mvIMPACT_C::IPL_LONG numberOfBlobs;
	
	if(!mvIMPACT_C::BLOC_InquireObjects(getHandle(), band, slice, frame, &numberOfBlobs))
	{ throwImpactError(); }
	
	return numberOfBlobs;
}


inline SingleBlob BlobLocatorResultList::getSingleBlob(int index, int band, int slice, int frame) const
{
	mvIMPACT_C::BLOC_BUFHANDLE hD;
	
	if(!mvIMPACT_C::BLOC_GetResultHandle(
		getHandle(),
		band, slice, frame,
		index,
		&hD))
	{ throwImpactError(); }
	
	return SingleBlob(hD);
}


inline double BlobLocatorResultList::getArea(int index, int band, int slice, int frame) const
{
	mvIMPACT_C::IPL_DOUBLE area;
	
	if(!mvIMPACT_C::BLOC_GetResultBufData(
		getHandle(), 
		band, slice, frame, 
		index, 
		mvIMPACT_C::BLOC_AREA,
		&area))
	{ throwImpactError(); }

	return area;		
}

inline Rectangle BlobLocatorResultList::getExtremeBox(int index, int band, int slice, int frame) const
{
	DynArray<mvIMPACT_C::IPL_INT> rectArray(4);
	
	if(!mvIMPACT_C::BLOC_GetResultBufData(
		getHandle(), 
		band, slice, frame, 
		index, 
		mvIMPACT_C::BLOC_BOUNDING_BOX,
		rectArray.getPointer()))
	{ throwImpactError(); }
	
	return Rectangle(
			rectArray.getPointer()[0], rectArray.getPointer()[1],
			rectArray.getPointer()[0]+rectArray.getPointer()[2]-1, rectArray.getPointer()[1]+rectArray.getPointer()[3]-1);
}

inline std::vector<Point> BlobLocatorResultList::getOutlinePoints(int index, int band, int slice, int frame) const
{
	mvIMPACT_C::IPL_INT nbOutlinePoints;
	
	if(!mvIMPACT_C::BLOC_GetResultBufData(
		getHandle(), 
		band, slice, frame, 
		index, 
		mvIMPACT_C::BLOC_NB_OUTLINE_POINTS,
		&nbOutlinePoints))
	{ throwImpactError(); }
	
	DynArray<mvIMPACT_C::IPL_INT> outlinePointsArray(nbOutlinePoints*2);
	if(!mvIMPACT_C::BLOC_GetResultBufData(
		getHandle(), 
		band, slice, frame, 
		index, 
		mvIMPACT_C::BLOC_OUTLINE_POINTS,
		outlinePointsArray.getPointer()))
	{ throwImpactError(); }

	
	std::vector<Point> result;

	for(int i = 0; i < nbOutlinePoints*2; i+=2) {
		result.push_back(Point(outlinePointsArray.getPointer()[i], outlinePointsArray.getPointer()[i+1]));
	}
	
	return result;
}

inline std::vector<RunLine> BlobLocatorResultList::getRunLines(int index, int band, int slice, int frame) const
{
	mvIMPACT_C::IPL_INT nbScanlines;
	
	if(!mvIMPACT_C::BLOC_GetResultBufData(
		getHandle(), 
		band, slice, frame, 
		index, 
		mvIMPACT_C::BLOC_NB_SCANLINES,
		&nbScanlines))
	{ throwImpactError(); }
	
	DynArray<mvIMPACT_C::IPL_INT> scanlinePointsArray(nbScanlines*3);
	if(!mvIMPACT_C::BLOC_GetResultBufData(
		getHandle(), 
		band, slice, frame, 
		index, 
		mvIMPACT_C::BLOC_SCANLINES,
		scanlinePointsArray.getPointer()))
	{ throwImpactError(); }
	
	std::vector<RunLine> result;

	for(int i = 0; i < nbScanlines*3; i+=3) {
		result.push_back(RunLine(scanlinePointsArray.getPointer()[i+1], scanlinePointsArray.getPointer()[i+2], scanlinePointsArray.getPointer()[i]));
	}
	
	return result;
}

inline void BlobLocatorResultList::copy(const BlobLocatorResultList& src)
{
	m_counter = src.m_counter;
	m_handle = src.m_handle;
	
	(*m_counter)++;
}
	
inline void BlobLocatorResultList::dispose()
{
	(*m_counter)--;
	if(0 == *m_counter) {
		delete m_counter;
		if(0 != m_handle) {
			if(!mvIMPACT_C::BLOC_BufFree(m_handle)) 
			{ throwImpactError(); }
		}
	}
}


inline SingleBlob::SingleBlob()
		: m_counter(new int(1)), m_handle(0)
{
}

inline SingleBlob::SingleBlob(mvIMPACT_C::BLOC_BUFHANDLE handle)
		: m_counter(new int(1)), m_handle(handle)
{
}

inline mvIMPACT_C::BLOC_BUFHANDLE SingleBlob::getHandle() const
{
	return m_handle;
}

inline SingleBlob::SingleBlob(const SingleBlob& src)
{
	copy(src);
}

inline SingleBlob::~SingleBlob()
{
	dispose();
}
	
inline SingleBlob& SingleBlob::operator=(const SingleBlob& src)
{
	if(this != &src)
	{
		dispose();
		copy(src);
	}
	return *this;
}

inline void SingleBlob::copy(const SingleBlob& src)
{
	m_counter = src.m_counter;
	m_handle = src.m_handle;
	
	(*m_counter)++;
}
	
inline void SingleBlob::dispose()
{
	(*m_counter)--;
	if(0 == *m_counter) {
		delete m_counter;
		if(!mvIMPACT_C::BLOC_BufFree(m_handle)) 
		{ throwImpactError(); }
	}
}


inline BlobLocatorResultList locateBlobs(Image& S, const BlobLocator &blobLocator)
{
	blobLocator.transferConfiguration();
	
	mvIMPACT_C::IPL_BUFHANDLE hS = S.getHandle();
	mvIMPACT_C::BLOC_BUFHANDLE hD;

	if(!mvIMPACT_C::BLOC_LocateObjects(
		hS,
		-1,
		&hS,
		&hD))
	{ throwImpactError(); }

	return BlobLocatorResultList(hD);
}

inline BlobLocatorResultList locateBlobs(
		const Image& S, 
		Image& L,
		const BlobLocator &blobLocator)
{
	blobLocator.transferConfiguration();
	
	mvIMPACT_C::IPL_BUFHANDLE hS = S.getHandle();
	mvIMPACT_C::IPL_BUFHANDLE hL = L.getHandle();
	mvIMPACT_C::BLOC_BUFHANDLE hD;

	if(!mvIMPACT_C::BLOC_LocateObjects(
		hS,
		IPL_DONT_CARE,
		&hL,
		&hD))
	{ throwImpactError(); }

	if(L.getHandle() != hL) { L = Image::imageFactory(hL); }

	return BlobLocatorResultList(hD);
}

inline BlobLocatorResultList locateBlobsGray(
		Image& G,
		const BlobLocator &blobLocator
		)
{
	blobLocator.transferConfiguration();
	
	mvIMPACT_C::IPL_BUFHANDLE hG = G.getHandle();
	mvIMPACT_C::BLOC_BUFHANDLE hD;

	if(!mvIMPACT_C::BLOC_LocateObjects(
		IPL_DONT_CARE,
		hG,
		&hG,
		&hD))
	{ throwImpactError(); }

	return BlobLocatorResultList(hD);
}

inline BlobLocatorResultList locateBlobsGray(
		const Image& G,
		Image& L,		
		const BlobLocator &blobLocator
		)
{
	blobLocator.transferConfiguration();
	
	mvIMPACT_C::IPL_BUFHANDLE hG = G.getHandle();
	mvIMPACT_C::IPL_BUFHANDLE hL = L.getHandle();
	mvIMPACT_C::BLOC_BUFHANDLE hD;

	if(!mvIMPACT_C::BLOC_LocateObjects(
		IPL_DONT_CARE,
		hG,
		&hL,
		&hD))
	{ throwImpactError(); }

	if(L.getHandle() != hL) { L = Image::imageFactory(hL); }

	return BlobLocatorResultList(hD);
	
}


inline BlobResultList analyze(Image& S, const BlobFeatureList& F)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hS = S.getHandle();
	BlobResultList D;
	
	// do the analyzation:
	if (!mvIMPACT_C::BL_Analyze(
			hS,
			-1,
			&hS,
			F.getHandle(),
			D.getHandle()))
		throwImpactError();
		
	// transfer the features
	F.transfer(D);
	
	return D;
}

inline BlobResultList analyze(const Image& S, const Image& G, Image& L, const BlobFeatureList& F)
{
	mvIMPACT_C::IPL_BUFHANDLE hS = S.getHandle();
	mvIMPACT_C::IPL_BUFHANDLE hG = G.getHandle();
	mvIMPACT_C::IPL_BUFHANDLE hL = L.getHandle();

	BlobResultList D;

	// do the analyzation:
	if (!mvIMPACT_C::BL_Analyze(
			hS,
			hG,
			&hL,
			F.getHandle(),
			D.getHandle()))
		throwImpactError();

	if(L.getHandle() != hL) { L = Image::imageFactory(hL); }
	
	// transfer the features
	F.transfer(D);
	
	return D;
}

inline BlobResultList analyzeGray(
		       const Image& G, 
		       Image& L, 
		       const BlobFeatureList& F)
{
	mvIMPACT_C::IPL_BUFHANDLE hG = G.getHandle();
	mvIMPACT_C::IPL_BUFHANDLE hL = L.getHandle();

	BlobResultList D;

	// do the analyzation:
	if (!mvIMPACT_C::BL_Analyze(
			IPL_DONT_CARE,
			hG,
			&hL,
			F.getHandle(),
			D.getHandle()))
		throwImpactError();

	if(L.getHandle() != hL) { L = Image::imageFactory(hL); }
	
	// transfer the features
	F.transfer(D);
	
	return D;
}		       

inline BlobResultList analyzeGray(
		       Image& G, 
		       const BlobFeatureList& F)
{
	mvIMPACT_C::IPL_BUFHANDLE hG = G.getHandle();

	BlobResultList D;

	// do the analyzation:
	if (!mvIMPACT_C::BL_Analyze(
			IPL_DONT_CARE,
			hG,
			&hG,
			F.getHandle(),
			D.getHandle()))
		throwImpactError();

	if(G.getHandle() != hG) { G = Image::imageFactory(hG); }
	
	// transfer the features
	F.transfer(D);
	
	return D;
}		       


inline BlobResultList analyzeSingleBlob(const SingleBlob& B, const BlobFeatureList& F)
{
	BlobResultList D;

	if(!mvIMPACT_C::BL_AnalyzeSingle(B.getHandle(), F.getHandle(), D.getHandle()))
	{ throwImpactError(); }

	// transfer the features
	F.transfer(D);
	
	return D;
}


inline PointDouble transform(Position trans, const PointDouble& source)
{
	mvIMPACT_C::IPL_DOUBLE destX = 0.;
	mvIMPACT_C::IPL_DOUBLE destY = 0.;
	if(!mvIMPACT_C::BL_GetCrdPoint (
		source.getX(),
		source.getY(),
		&destX,
		&destY,
		trans.getX(),
		trans.getY(),
		trans.getScaleX(),
		trans.getScaleY(),
		trans.getAngle())
	)
	{ throwImpactError(); }
	return PointDouble(destX, destY);
}


inline bool checkLicenseBlob()
//-----------------------------------------------------------------------------
{
	return (IPL_TRUE == mvIMPACT_C::BL_CheckLic());
}

inline std::string getVersionBlob()
{
	int size = 12;
	std::string sv;
	sv.resize(size);
	while (mvIMPACT_C::BL_GetVersion(sv.c_str(), size) == IPL_FALSE)
	{
		size += 1;
		sv.resize(size);
	}
	sv.resize(strlen(sv.c_str()));

	return sv;
}

inline std::string getVersionBlobLocator()
{
	int size = 12;
	std::string sv;
	sv.resize(size);
	while (mvIMPACT_C::BLOC_GetVersion(sv.c_str(), size) == IPL_FALSE)
	{
		size += 1;
		sv.resize(size);
	}
	sv.resize(strlen(sv.c_str()));

	return sv;
}

	} // namespace blob
} // namespace mvIMPACT
#endif // #ifndef WRAP_ANY
#endif// DOXYGEN_SHOULD_SKIP_THIS


#endif // MVIMPACT_BLOB_IMPL_H_
