#ifndef MVIMPACT_FOCUS_H_
#define MVIMPACT_FOCUS_H_

/// \file
///

#ifndef WRAP_PYTHON
#include "mvIMPACT.h"
#endif // #ifndef WRAP_PYTHON

#ifndef  DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_PYTHON


#include <vl_autofocus.h>


#endif // #ifndef WRAP_PYTHON
#endif // DOXYGEN_SHOULD_SKIP_THIS

namespace mvIMPACT {

        /// \namespace mvIMPACT::focus This namespace contains all classes and functions from mvIMPACT Focus.
	namespace focus {

/// \brief Specifies the filter type to use when using edge 
/// information to find the in-focus image.
enum FilterType 
{
	/// 3x3 Sobel filter kernel.
	ftSobel = AF_SOBEL, 
	/// 2x2 Canny filter kernel.
	ftCanny = AF_CANNY, 
	/// 3x3 horizontal edge filter kernel.
	ftHoredge = AF_HOREDGE
};

/// \brief Reflects the status of the focus finding.
enum ZDriveMode
{
	/// The in-focus position is not reached yet.
	zdmChange = mvIMPACT_C::AF_CHANGE,
	/// The in-focus position has been reached.
	zdmFocus  = mvIMPACT_C::AF_FOCUS
};


/// \brief Focussing method.

/// Determines which method to use.
enum AutoFocusMode
{
	/// The algorithm uses intensity information to find the in-focus position. It is not recommended to use
	/// this method, because it works only in certain circumstances. Kept for compatibility only.
	afmMaxIntensity = mvIMPACT_C::AF_MAX_INTENSITY,
	/// The algorithm uses edge information to find the in-focus position.
	afmMaxEdge = mvIMPACT_C::AF_MAX_EDGE,
	/// The algorithm uses variance information to find the in-focus position.
	afmMaxVariance = mvIMPACT_C::AF_MAX_VARIANCE,
};



/// \brief Focussing method for mvIMPACT::focus::computeMultiFocusImage

enum MultiFocusMode
{
	/// Builds an in-focus image based on edges in images.
        mfmImageEdge = mvIMPACT_C::AF_MULTIFOCUS_IMAGE_EDGE///,
	/// Not implemented. Do not use.
	///	mfmImageIntensity = mvIMPACT_C::AF_MULTIFOCUS_IMAGE_INTENSITY
};


/// \brief Special starting position value 
///
enum FocusPositionMode
{
	/// Special value for the starting position to let the focus finder first use a rough estimate
	/// for the in-focus position.
	/// \see mvIMPACT::focus::FocusFinder::getStartPosition
	/// \see mvIMPACT::focus::FocusFinder::setStartPosition
	fpmUnknown = (int)AF_UNKNOWN
};


PYTHON_ONLY(%feature("director") FocusFinder);
///\brief This class includes all functions and properties, that are needed to
///\brief get an in-focus image.
///
/// To use this class you need to derive from it and overwrite at least the
/// function onGrabImage() which is called when a new image is to be aquired and
/// you need to overwrite onAdjustFocus() which allows you to set the parameters
/// of your imaging device to adjust the focus.
///
/// \note This is an abstract class!
//-----------------------------------------------------------------------------
class FocusFinder
//-----------------------------------------------------------------------------
{
public:
	///\brief Construct a FocusFinder object with default values.
	///
	/// The default values are:
	/// samplesAfterMax = 2;
	/// stepDecrement = 50;
	/// filterType = ftSobel;
	/// iterations = 1;
	/// filterStepWidth = 1;
	/// samplesDirection = 2;
	/// minPosition = -1e+008;
	/// maxPosition = 1e+008;
	/// startPosition = 0;
	/// stepIncrementRough = 10;
	/// stepIncrement = 1;
	///
	/// If you derive your own class from FocusFinder and do not call the parent constructor
	/// from its constructor, then these values will be in an undefined state, i.e. you must
	/// call the parent constructor.
	FocusFinder();

	///\brief Computes an in-focus image
	///
	/// This function computes an in-focus image. Whenever the algorithm needs a new image,
	/// onGrabImage is called with \a dest as its argument. After grabbing an image, onAdjustFocus 
	/// is called.
	///
	/// After computation has been finished, the result is available in \a dest.
	void findFocus
	(
		/// The destination image. Also used for aquiring images during the algorithm
        Image&            dest,
        /// A child image (of \a dest, for example) to specify a region from which the focus information
        /// is calculated. Pass an unassigned image to use the whole buffer
        const Image&       regionImage = Image(),
        /// The algorithm used for calculating the in-focus image.
        AutoFocusMode     mode = afmMaxEdge
	);	



public: //properties


	virtual ~FocusFinder();

	/// \brief Get resulting focus position
	///
	/// Returns the focus position of the last call to findFocus() or 0.0 if
	/// findFocus() has not been called yet.
	double getFocusPosition() const;


#ifndef WRAP_DOTNET
	/**
	\brief Method that is called by the callback mechanism.

	This method allows you to implement your own
	in-focus algorithm. 
	Compute a number of merit for beeing in focus. The higher the value
	is the better it the focus.
	
	If you overwrite this function, you must call useOnCustomFocus() in the
	the constructor of your derived class. Otherwise, this function will never be called
	and the default implementation is used. This is for technical reasons. We apologize
	for the inconvenience.
		
	Return \c true to continue the focus finding process.
	Returning \c false terminates the focus finder with an exception.
	 
    \xmlonly
    <mvWrapperDoc><htmlinclude ref="python_doc_start.html"/></mvWrapperDoc>
	\endxmlonly 
	\htmlinclude python_doc_start.html
	In Python, this function is used differently:
	The user's implementation has to return a tuple consisting
	of the return value and the changed value of \a score, i.e.
	\code
	def onCustomFocus(source, score) :
		## assign calculation result to score here
		return (True, score)
	\endcode
	\htmlinclude specific_doc_end.html
	\xmlonly
	<mvWrapperDoc><htmlinclude ref="specific_doc_end.html"/></mvWrapperDoc>
	\endxmlonly
	*/
	virtual bool onCustomFocus(///the image for which the focus score must be calculated
				   const Image& source, 
				   ///the focus score
				   double& PYTHON_CPP_SWITCH(DOUBLEREF_DIRECTOR,score));


	///\brief Function that is called by the callback mechanism.

	///Implement this method to grab the image.
	/// It is not "allowed" though possible to assign a different object to \a dest.
	/// The data must be stored in the buffer \a dest. Use e.g. mvIMPACT::Image::setPixel 
	/// for that purpose.
	///
	/// Return \c true if grabbing image was succcessful.
	///
	virtual bool onGrabImage (///the aquired image
				Image& dest) = 0;


	/// \brief Method that is called by the callback mechanism before an image is grabbed.

	///Implement this method to adjust the focus. 
	///It allows you to adjust your focus-device to the position specified with the \a position
	///parameter in the callback function.
	///Return \c true if adjusting focus was succcessful.
	///Returning \c false terminates the focus finder with an exception.
	virtual bool onAdjustFocus (///the status of the focussing
				  ZDriveMode mode, 
				  ///position
				  double position) = 0;
#endif // #ifndef WRAP_DOTNET


	/// \brief Specify the number of images to acquire after the maximum has been reached.

	///
	/// \see mvIMPACT::focus::FocusFinder::getSamplesAfterMax
	FocusFinder& setSamplesAfterMax (
				/// the number of images 
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamConst value="50"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int samplesAfterMax);

	/// \brief Retrieve the number of images to acquire after the maximum has been reached.	

	///
	/// \see mvIMPACT::focus::FocusFinder::setSamplesAfterMax
	int getSamplesAfterMax () const;

	///\brief Specify the step decrement.

	/// The value is given as a percentage [1..100]. It is applied after each iteration.

	///
	/// \see mvIMPACT::focus::FocusFinder::getStepDecrement
	FocusFinder& setStepDecrement (
				///the value in percentage 
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
				/// <mvParamMax><mvParamConst value="100"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int stepDecrement);

	/// \brief Retrieve the step decrement

	/// The value is given as a percentage [0..100]. It is applied after each iteration.
	/// \see mvIMPACT::focus::FocusFinder::setStepDecrement
	int getStepDecrement () const;

	/// \brief Specifies the filter type to use when using edge information to find the in-focus image.

	/// Valid filtertypes are: AF_SOBEL, AF_CANNY, AF_HOREDGE
	///\see mvIMPACT::focus::FocusFinder::getFilterType
	FocusFinder& setFilterType (///the filter type to use 
			    FilterType filterType);


	/// \brief Retrieve the filter type which is used to find the in-focus image.

	///
	///\see mvIMPACT::focus::FocusFinder::setFilterType
	FilterType getFilterType () const;


	/// \brief Specify the number of iterations to be applied.

	///
	/// \see mvIMPACT::focus::FocusFinder::getIterations
	FocusFinder& setIterations ( 
				///the number of iterations
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
				/// </mvParamRange>
				/// \endxmlonly			   			   
				int iterations);


	///\brief Retrieve  the number of iterations to be applied.

	///
	/// \see mvIMPACT::focus::FocusFinder::getIterations
	int getIterations () const;



	/// \brief Specifies the internal filter function step width.

	///A value of 1 indicates, that every pixel from the image will be used. A value of 2 means, that
	///every second pixel of the image will be used, etc. The greater the step width, the faster the
	///process will be. However, the chance to fail the in-focus position increases.
	///\see mvIMPACT::focus::FocusFinder::getFilterStepWidth
	FocusFinder& setFilterStepWidth (
				///the actual step width 
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
				/// <mvParamMax><mvParamConst value="50"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int filterStepWidth );

	/// \brief Retrieve the step width of the internal filter function .

	///A value of 1 indicates, that every pixel from the image will be used. A value of 2 means, that
	///every second pixel of the image will be used, etc. The greater the step width, the faster the
	///process will be. However, the chance to fail the in-focus position increases.
	///\see mvIMPACT::focus::FocusFinder::setFilterStepWidth
	int getFilterStepWidth () const;



	///\brief Specifies the number of images to acquire to determine the right direction.

	///The more images are acquired, the better is the chance to find the tight direction. However,
	///the processing time increases.
	///\see mvIMPACT::focus::FocusFinder::getSamplesDirection
	FocusFinder& setSamplesDirection (
				///the number of images to be acquired 
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
				/// <mvParamMax><mvParamConst value="50"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int samplesDirection );


	///\brief Retrieve the number of images to be acquired to determine the right direction.

	///The more images are acquired, the better is the chance to find the tight direction. However,
	///the processing time increases.
	///\see mvIMPACT::focus::FocusFinder::setSamplesDirection
	int getSamplesDirection () const;



	///\brief Specifies the minimum z-position where to start if startPosition property is set to
	///fpmUnknown.

	///This value must be smaller than the value from mvIMPACT::focus::FocusFinder::getMaxPosition.
	/// \see mvIMPACT::focus::FocusFinder::getMinPosition
	/// \see mvIMPACT::focus::FocusFinder::setStartPosition
	FocusFinder& setMinPosition (
				///the minimum z-position 
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.maxPosition-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int minPosition );

	///\brief Returns the minimum z-position where to start if startPosition property is set to
	///fpmUnknown.

	/// \see mvIMPACT::focus::FocusFinder::setMinPosition
	/// \see mvIMPACT::focus::FocusFinder::setStartPosition
	int getMinPosition () const;



	///\brief Specifies the maximum z-position where to start if startPosition property is set to
	///fpmUnknown.

	/// \see mvIMPACT::focus::FocusFinder::getMaxPosition
	/// \see mvIMPACT::focus::FocusFinder::setStartPosition
	FocusFinder& setMaxPosition (
				///the maximum z-position 
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamComplex Python="%(instance)s.minPosition+1"/></mvParamMin>
				/// </mvParamRange>
				/// \endxmlonly			   
				int maxPosition);

	///\brief Returns the maximum z-position where to start if startPosition property is set to
	///fpmUnknown.

	/// \see mvIMPACT::focus::FocusFinder::setMaxPosition
	/// \see mvIMPACT::focus::FocusFinder::setStartPosition
	int getMaxPosition () const;


	/// \brief Set initial z-position where to start to search for the in-focus position.

	///If value is set to fpmUnknown, the algorithm first tries to find a rough estimated in-focus
	///position in a given range as specified with mvIMPACT::focus::FocusFinder::setMinPosition and mvIMPACT::focus::FocusFinder::setMaxPosition.
	///The increment between each step can be specified with mvIMPACT::focus::FocusFinder::setStepIncrementRough.
	///\see mvIMPACT::focus::FocusFinder::setStepIncrementRough
	///\see mvIMPACT::focus::FocusFinder::setMinPosition 
	///\see mvIMPACT::focus::FocusFinder::setMaxPosition
	///\see mvIMPACT::focus::FocusFinder::getStartPosition
	FocusFinder& setStartPosition ( 
		/// the new start position
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMax><mvParamConst value="100000000"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   	
		int startPosition );



	/// \brief Retrieve initial z-position where to start to search for the in-focus position.

	///If Value is fpmUnknown, the algorithm first tries to find a rough estimated in-focus
	///position in a given range between mvIMPACT::focus::FocusFinder::getMinPosition and mvIMPACT::focus::FocusFinder::getMaxPosition.
	///The increment between each step can be retrieved with mvIMPACT::focus::FocusFinder::getStepIncrementRough.
	///\see mvIMPACT::focus::FocusFinder::getStepIncrementRough
	///\see mvIMPACT::focus::FocusFinder::getMinPosition 
	///\see mvIMPACT::focus::FocusFinder::getMaxPosition
	///\see mvIMPACT::focus::FocusFinder::setStartPosition
	int getStartPosition () const;

	///\brief Specifies the step width used during the initial search, if startPosition property is set to
	///fpmUnknown.

	///\see mvIMPACT::focus::FocusFinder::setStartPosition
	///\see mvIMPACT::focus::FocusFinder::setMinPosition 
	///\see mvIMPACT::focus::FocusFinder::setMaxPosition
	///\see mvIMPACT::focus::FocusFinder::getStepIncrementRough 
	FocusFinder& setStepIncrementRough (
					///the rough step width 
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamComplex Python="%(instance)s.stepIncrement"/></mvParamMin>
					/// <mvParamMax><mvParamConst value="10"/></mvParamMax>
					/// <mvParamStep>0.1</mvParamStep>
					/// </mvParamRange>
					/// \endxmlonly			   	
					double stepIncrementRough );

	///\brief Retrieve the step width used during the initial search, if startPosition property is set to
	///fpmUnknown.
	///
	///\see mvIMPACT::focus::FocusFinder::setStartPosition
	///\see mvIMPACT::focus::FocusFinder::setMinPosition 
	///\see mvIMPACT::focus::FocusFinder::setMaxPosition
	///\see mvIMPACT::focus::FocusFinder::getStepIncrementRough 
	double getStepIncrementRough () const;


	///\brief Specifies the step width used during the search of the in-focus position.

	///This value should be smaller than mvIMPACT::focus::FocusFinder::getStepIncrementRough
	///\see mvIMPACT::focus::FocusFinder::getStepIncrement
	FocusFinder& setStepIncrement (
					///the step width 
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
					/// <mvParamMax><mvParamComplex Python="%(instance)s.stepIncrementRough"/></mvParamMax>
					/// <mvParamStep>0.1</mvParamStep>
					/// </mvParamRange>
					/// \endxmlonly			   	
					double stepIncrement );

	///\brief Retrieve the step width used during the search of the in-focus position.

	///This value should be smaller than mvIMPACT::focus::FocusFinder::getStepIncrementRough
	///\see mvIMPACT::focus::FocusFinder::getStepIncrement
	double getStepIncrement () const;

#ifndef WRAP_ANY

/// \brief Sets the registry path used for saving parameters.

/// Sets the actual registry key used to serialize parameters. If no path is supplied, the internal
/// settings will be saved to an ini-file in the current module directory.
/// \note This is not useable from Python.
//-----------------------------------------------------------------------------
void setRegPath 
(
	/// Specifies the registry key (HKEY_CURRENT_USER,
	/// HKEY_LOCAL_MACHINE, etc.)
	mvIMPACT_C::IPL_HKEY key ,
	/// Registry path (including the trailing backslash).
	const std::string& subkey
	);

#endif // #ifndef WRAP_ANY


	/// \brief Writes values into the registry or into an ini-file.

	/// Writes actual property set to registry. Note that if you do not use
	/// setRegPath() to set a user defined registry key, only one property set
	/// can be saved in the registry, i.e. every call to writeInitializationValues()
	/// overwrites the previous values
	//-----------------------------------------------------------------------------
	void writeInitializationValues ();


	/// \brief Reads values from the registry or from an ini-file.

	/// Reads actual property set from registry. 
	void readInitializationValues ();


	void useOnCustomFocus();

protected:

private:	
	#ifndef WRAP_ANY
	static mvIMPACT_C::IPL_FRESULT IPL_C_CALLTYPE adjustFocusCallback
	(
		mvIMPACT_C::AF_Z_DRIVE_MODE Status,            
		mvIMPACT_C::IPL_DOUBLE      Position,
		mvIMPACT_C::IPL_CONST_VOID_PTR pCookie
	);

	static mvIMPACT_C::IPL_FRESULT IPL_C_CALLTYPE grabImageCallback
	(
        mvIMPACT_C::IPL_BUFHANDLE*  BufHandle, 
        mvIMPACT_C::IPL_CONST_VOID_PTR pCookie 
	);

	static mvIMPACT_C::IPL_FRESULT IPL_C_CALLTYPE customFocusCallback
	(
	    mvIMPACT_C::IPL_BUFHANDLE   BufHandle, 
		mvIMPACT_C::IPL_DOUBLE*     FocusParam, 
		mvIMPACT_C::IPL_CONST_VOID_PTR pCookie  
	);
	#endif

	void transferConfiguration() const;
	void inquireConfiguration();

private:
	double         m_focusPosition;

	int m_samplesAfterMax;
	int m_stepDecrement;
	FilterType m_filterType;
	int m_iterations;
	int m_filterStepWidth;
	int m_samplesDirection;
	int m_minPosition;
	int m_maxPosition;
	int m_startPosition;
	double m_stepIncrementRough;
	double m_stepIncrement;
	
	bool m_useOnCustomFocus;

};



#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY
/// \brief This class is for internal use only
class CustomFocusCookie
{
public:
	CustomFocusCookie(FocusFinder* finder, Image* image);

	FocusFinder* finder;
	Image* image;
};
#endif // #ifndef WRAP_ANY
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS


///\brief Compute one in-focus image from various images

/// \a sources must at least contain two mvIMPACT::Image objects
//-----------------------------------------------------------------------------
Image computeMultiFocusImage
(
        ///the vector containing the images
	const std::vector<Image>& PYTHON_CPP_SWITCH(SEQUENCE_TO_VECTOR,sources),
	///the focussing mode to use
	MultiFocusMode mode = mfmImageEdge
	);

///\brief Compute one in-focus image from various images

/// \a sources must at least contaion tw mvIMPACT::Image objects
//-----------------------------------------------------------------------------
void computeMultiFocusImage
(
        ///the destination mvIMPACT::Image
	Image& dest,
        ///the vector containing the images
	const std::vector<Image>& PYTHON_CPP_SWITCH(SEQUENCE_TO_VECTOR,sources),
	///the focussing mode to use
	MultiFocusMode mode = mfmImageEdge
	);

///\brief Returns a focus value of an image
///
/// This function allows you to get a focus value of an image. It can help you if you have to adjust the focus manually.
/// The value itself depends on the method used (please see \a AutoFocusMode). An image is in-focus, if the value reaches
/// a maximum.
double findFocusValue
(
	/// The source image. It is used to find the focus value.
    Image&            src,
    /// A child image (of \a src, for example) to specify a region from which the focus information
    /// is calculated. Pass an unassigned image to use the whole buffer.
    const Image&       regionImage = Image(),
    /// The algorithm used for calculating the in-focus image.
    AutoFocusMode     mode = afmMaxEdge,
	/// If \a mode is set to afmMaxEdge, this parameter selects the filter type to use. Otherwise it will be ignored.
	FilterType type = ftSobel,
	/// The step width to apply. Example: if this value is set to 2, every second pixel and every second line is included
	/// in the calculation process of the focus value. The higher the value the faster the calculation will be. 
	/// \xmlonly
	/// <mvParamRange>
	/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
	/// <mvParamMax><mvParamConst value="20"/></mvParamMax>
	/// </mvParamRange>
	/// \endxmlonly			   	
	int step = 1
	);	


	/// \brief Checks if the focus license is valid.
	bool checkLicenseFocus();

	/// \brief Return version number from vl_autofocus module.

	/// This function returns a version string. The following format is used: 
	/// "major.minor.point.build" where "major" specifies the major version number, "minor" specifies
	/// the minor version number, "point" specifies a point version number and "build" specifies the build number. 
	/// Example: 6.3.1.4529
	//-----------------------------------------------------------------------------
	std::string getVersionFocus();


	} // namespace focus
} // namespace mvIMPACT

#include "mvIMPACT_focus_impl.h"

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_PYTHON

#ifndef MVIMPACT_USE_NAMESPACES
using namespace mvIMPACT::focus;
#endif // #ifndef MVIMPACT_USE_NAMESPACES

#endif // #ifndef WRAP_PYTHON
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS


#endif // MVIMPACT_FOCUS_H_
