#ifndef MVIMPACT_GMM_H_
#define MVIMPACT_GMM_H_


/// \file
///

#ifndef WRAP_PYTHON
#include "mvIMPACT.h"
#endif // #ifndef WRAP_PYTHON




#ifndef  DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_PYTHON


#include <vl_gmm.h>


#endif // #ifndef WRAP_PYTHON
#endif // DOXYGEN_SHOULD_SKIP_THIS

namespace mvIMPACT {

        /// \namespace mvIMPACT::gmm This namespace contains all classes and functions from mvIMPACT GMM.
	namespace gmm {


#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY

/// "ONLY USED INTERNALLY"
typedef int* GeoFeatureID;

class ArcPiece;

/// "ONLY USED INTERNALLY"
/// Corner data is shared between copies of Corner objects.
/// Thus it is encapsulated into a separate struct.
struct CornerData
{
	PointDouble cornerPoint;
	double openingAngle;
	double bisector;
	LineDouble line1;
	LineDouble line2;
};

/// "ONLY USED INTERNALLY"
/// Arc data is shared between copies of Arc objects.
/// Thus it is encapsulated into a separate struct.
struct ArcData
{
	PointDouble center;
	double radius;
	std::vector<ArcPiece> vPieces;
};


#endif // #ifndef WRAP_ANY
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

/// \brief Special value for maximum number of matches
///
/// \see mvIMPACT::gmm::GeoMatcher::setMaxNumberOfMatches
/// \see mvIMPACT::gmm::GeoMatcher::getMaxNumberOfMatches
enum MatcherMode {
        /// Deliver all matches. Use this as parameter for mvIMPACT::gmm::GeoMatcher::setMaxNumberOfMatches
        /// to get all matches.
	mmMatcherFindAll     = mvIMPACT_C::GMM_MATCHER_FIND_ALL
};


/// \brief special values for number of features selected from a model
///
/// \see mvIMPACT::gmm::GeoModel::GeoModel
enum FindFeatureMode {
        ///Select all features from the model. This
        ///options should only be used after you have tried #ffmModelFindAllStable
		ffmModelFindAll       = mvIMPACT_C::GMM_MODEL_FIND_ALL,

		///Select all stable features from the
		///model. Stable feature can be found in different resolution levels of the model and are
		///well suited for pattern matching. If you are not satisfied with the automatic feature
		///selection, this mode is usefully. You should deselect all unwanted features from the
		///model.
		ffmModelFindAllStable = mvIMPACT_C::GMM_MODEL_FIND_ALL_STABLE
	};



/// \brief Defines the scope of model inquire functions.
///
/// These enumeration values can be used to specify whether the inquire functions
/// work on enabled, disable or all features.
enum FeatureInquireScope {
		///The inquire function considers enabled features only
		fisOnlyEnabled		= mvIMPACT_C::INQUIRE_SCOPE_ENABLED,

		/// The inquire function considers disabled features only
		fisOnlyDisabled		= mvIMPACT_C::INQUIRE_SCOPE_DISABLED,

		/// The inquire function considers  all features, 
		/// no matter if they are enabled or disabled
		fisAll		= mvIMPACT_C::INQUIRE_SCOPE_ALL,
};

/// \brief Defines the scope of model add functions.
///
/// These enumeration values can be used to specify whether the add functions
/// add the features as enabled or disabled
enum FeatureAddScope {
		///The feature will be added to the set of enabled features
		fasEnabled		= mvIMPACT_C::ADD_SCOPE_ENABLED,

		///The feature will be added to the set of disabled features		
		fasDisabled		= mvIMPACT_C::ADD_SCOPE_DISABLED,
};




/// \brief Piece of an arc. 

/// A piece is defined with two angles. A offset angle and an increment angle 
/// that is applied on this offset. All angle values are in radian.
/// The rotation direction is counter clockwise.
/// The following image shows two arc pieces (in blue):
/// \image html images/mvi_gmm/gmmArcPiece.png
/// Now the corresponding angle values for these pieces. <br>
/// <ul>
/// <li>Piece 1: <br>
/// &nbsp;&nbsp;&nbsp;Offset 0 and increment PI/2 (90 degrees).
/// </li>
/// <li>Piece 2: <br>
/// &nbsp;&nbsp;&nbsp;Offset PI and increment PI/2 (90 degrees).
/// </li>
/// </ul>
//-----------------------------------------------------------------------------
class ArcPiece
//-----------------------------------------------------------------------------
{
public:
	/// \brief Construct an empty arc piece.
	ArcPiece();

	/// \brief Construct an arc piece.

	///Construct an mvIMPACT::gmm::ArcPiece by specifying an offset angle 
	/// and an increment angle. 
	///\note Both values are given in radian values.
	ArcPiece
	(
		/// The offset angle (radian value) that starts the arc piece.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="6.2831853071795862"/></mvParamMax>
		/// <mvParamStep>0.01</mvParamStep>
		/// </mvParamRange>
		/// \endxmlonly			   	
		double startAngle, 
		/// The increment angle (radian value) that is applied to the start angle. 
		///This defines the length of the mvIMPACT::gmm::ArcPiece.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="6.2831853071795862"/></mvParamMax>
		/// <mvParamStep>0.01</mvParamStep>
		/// </mvParamRange>
		/// \endxmlonly			   	
		double incAngle
		);

	double getStartAngle() const;

	double getIncAngle() const;


private:
	/// \brief The offset angle that starts the arc piece.
	double m_startAngle;
	/// \brief The increment angle that is applied to the start angle. 
	/// This is the length of the arc.
	double m_incAngle;
};


/// \brief Represents an arc. 

///Arcs are one of the geometric primitives the GMM 
/// module can search for.
/// An arc has the following properties that are used by the GMM:
/// <ul>
/// <li>Center point</li>
/// <li>Radius</li>
/// </ul>
/// Additionaly information about the single pieces this arc consists of can be
/// retrieved. This information is not used internally.<br>
/// The following image illustrates the arc properties:
/// \image html images/mvi_gmm/gmmArc.png
/// The illustration shows an arc that consists of two pieces.

//-----------------------------------------------------------------------------
class Arc
//-----------------------------------------------------------------------------
{
private:
	int *m_counter;
	ArcData* m_data;
	
public:
	/// \brief create a default Arc object.
	///
	/// The default arc is a unit circle, i.e. the values are: 
	/// x = 0; y = 0; radius = 1; pieces = std::vector<ArcPiece>()
	Arc();

	/// \brief create an Arc
	Arc(
		/// the arc center.
		PointDouble center,
		/// Radius of the arc.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0.1"/></mvParamMin>
		/// <mvParamStep>0.1</mvParamStep>
		/// </mvParamRange>
		/// \endxmlonly			   	
		double radius,
		/// Any number of arc pieces. If set to zero a single piece describing a 
		/// full circle is created.
		const std::vector<ArcPiece>& PYTHON_CPP_SWITCH(SEQUENCE_TO_VECTOR,pieces)
	);

	/// \brief Copy construct an Arc object
	Arc(const Arc& src);	

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY
	Arc(mvIMPACT_C::GMM_ARC iplArc);
#endif // #ifndef WRAP_ANY
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

	/// \brief Destructor.
	~Arc();

#ifndef WRAP_ANY
	/// \brief Assign \a src to current object.
	Arc& operator=(const Arc& src);
#endif // #ifndef WRAP_ANY

	/// \brief Retrieve the center point.
	///
	PointDouble getCenter() const;                        


	/// \brief Retrieve the radius of the arc.
	double getRadius() const;                   

	/// \brief Retrieve the number of arc pieces.
	int  getNumberOfArcPieces() const;


	/// \brief Retrieve a single arc piece.
	ArcPiece getArcPiece
	(
		/// Index of the piece. Valid in the range [0,mvIMPACT::gmm::Arc::getNumberOfArcPieces()-1]
		int index
	) const;
	
#ifndef WRAP_DOTNET
	const std::vector<ArcPiece>& getArcPieces() const;
#endif // #ifndef WRAP_DOTNET
	
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY
	GeoFeatureID getID() const;
#endif // #ifndef WRAP_ANY
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

private:
	void dispose();
	
	void copy(const Arc& src);	

};


/// \brief Represents a corner. 

/// Corners are one of the geometric primitives the GMM 
/// module can search for.
/// A corner has the following properties that are used by the GMM:
/// <ul>
/// <li>Corner point</li>
/// <li>Opening angle</li>
/// <li>Bisector</li>
/// </ul>
/// Additionaly the two edges that formed the corner are available. The edges
/// are not used internally. <br>
/// The following image illustrates the corner properties:
/// \image html images/mvi_gmm/gmmCorner.png
/// The opening angle and bisector are specified in radian.
/// The rotation direction is counter clockwise.
/// The corner displayed on the above image has an opening angle of PI/2 (90 degrees)
/// and a bisector of 7/4 PI (315 degrees).
//-----------------------------------------------------------------------------
class Corner 
//-----------------------------------------------------------------------------
{
private:
	int *m_counter;
	CornerData* m_data;

public:
	/// \brief Create a default Corner object.
	///
	/// The default is the corner in (0,0) with the unit vectors as edges:
	/// x = 0; y = 0; openingAngle = M_PI/2; bisector = M_PI/4;  
	/// edge1Length = 1.0; edge2Length = 1.0;
	Corner();

	/// \brief create a Corner
	Corner(
		/// the point in the corner
		const PointDouble & cornerPoint,
		/// Opening angle of the corner in radian.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="6.2831853071795862"/></mvParamMax>
		/// <mvParamStep>0.01</mvParamStep>
		/// </mvParamRange>
		/// \endxmlonly			   	
		double openingAngle,
		/// Bisector of the corner in radian.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="6.2831853071795862"/></mvParamMax>
		/// <mvParamStep>0.01</mvParamStep>
		/// </mvParamRange>
		/// \endxmlonly			   	
		double bisector,
		/// Length of first edge
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0.1"/></mvParamMin>
		/// <mvParamStep>0.1</mvParamStep>
		/// </mvParamRange>
		/// \endxmlonly			   	
		double edge1Length = 1.0,
		/// Length of second edge
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0.1"/></mvParamMin>
		/// <mvParamStep>0.1</mvParamStep>
		/// </mvParamRange>
		/// \endxmlonly			   	
		double edge2Length = 1.0
		);

	/// \brief create a Corner
	Corner(
		/// The point in the corner.
		const PointDouble& cornerPoint,
		/// The end point of the first edge. The edge goes from \a cornerPoint to \a edge1EndPoint.
		const PointDouble& edge1EndPoint,
		/// The end point of the second edge. The edge goes from \a cornerPoint to \a edge2EndPoint.
		const PointDouble& edge2EndPoint
		);

	/// \brief Copy construct a Corner object
	Corner(const Corner& src);

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY
	Corner(mvIMPACT_C::GMM_CORNER iplCorner);
#endif // #ifndef WRAP_ANY
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

	/// \brief Destructor.
	~Corner();

#ifndef WRAP_ANY
	/// \brief Assign \a src to current object.
	Corner& operator=(const Corner& src);
#endif // #ifndef WRAP_ANY


	/// \brief Retrieve the corner point.
	///
	PointDouble getCornerPoint() const;
	

	/// \brief Retrieve the opening angle of the corner in radian.
	///
	/// \note The angle is measured counterclockwise
	double     getOpeningAngle() const;

	/// \brief Retrieve the bisector of the corner in radian.
	///
	/// \note The angle is measured counterclockwise
	double     getBisector()  const;


	/// \brief Retrieve the first edge that forms this corner.
	///
	/// \see mvIMPACT::gmm::Corner::getLine2
	LineDouble getLine1() const;


	/// \brief Retrieve the second edge that forms this corner.
	///
	/// \see mvIMPACT::gmm:Corner::getLine1
	LineDouble getLine2() const;

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY
	GeoFeatureID getID() const;
#endif // #ifndef WRAP_ANY
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

private:
	void dispose();
	
	void copy(const Corner& src);	
};


/**
\brief This class represents a search model for the GMM module.

The class that represents a search model is GeoModel. You have three possiblities to create a search model:

    - From an existing image
    - From an description file
    - Completely manual

To create a model based on an existing image the following code can be used:

\xmlonly
<mvWrapperDoc><htmlinclude ref="code_fragment_start_cpp.html"/></mvWrapperDoc>
\endxmlonly
\htmlinclude code_fragment_start_cpp.html
\code
   GeoModel model(loadImage("model1.bmp"), 12, 30); 
\endcode
\htmlinclude code_fragment_end.html
\xmlonly
<mvWrapperDoc><htmlinclude ref="code_fragment_end.html"/></mvWrapperDoc>
\endxmlonly
\xmlonly
<mvWrapperDoc>
<htmlinclude ref="code_fragment_start_python.html"/>
\code
   model(mvIMPACT.loadImage("model1.bmp"), 12, 30)
\endcode
<htmlinclude ref="code_fragment_end.html"/>
</mvWrapperDoc>
\endxmlonly

Using this code mvIMPACT GMM tries to detect 12 features (corner or arcs) from the model image. 
Additionally to these features an edge map of the model image is generated. This edge map 
contains edge pixels from the model image. The threshold value mentioned above is used to 
select suitable edge pixels.
The features of mvIMPACT GMM are represented with the classes mvIMPACT::gmm::Corner and mvIMPACT::gmm::Arc. You can query the model 
features with mvIMPACT::gmm::GeoModel::getCorners and mvIMPACT::gmm::GeoModel::getArcs.
\see mvIMPACT::gmm::Corner
\see mvIMPACT::gmm::Arc
*/
//-----------------------------------------------------------------------------
class GeoModel
//-----------------------------------------------------------------------------
{
private:
	/// The buffer handle.
	mvIMPACT_C::GMM_MODELHANDLE m_handle;
	/// Unassigned buffer constant.
	enum { unassigned = -1 };
	/// Pointer to the reference counter.
	mutable int* m_counter;

	/// Pointer to model image or NULL
	Image* m_pModelImage;

	std::map<mvIMPACT_C::GMM_FEATUREHANDLE, Arc> *m_ArcHandleMap;
	std::map<GeoFeatureID, mvIMPACT_C::GMM_FEATUREHANDLE> *m_ArcIDMap;
	
	std::map<mvIMPACT_C::GMM_FEATUREHANDLE, Corner> *m_CornerHandleMap;
	std::map<GeoFeatureID, mvIMPACT_C::GMM_FEATUREHANDLE> *m_CornerIDMap;
		
public:
	/// \brief Allocates an unassigned GeoModel object.
	///
	/// \note This is only a dummy object, you can do nothing sensible with it
	/// apart from assigning it an assigned GeoModel object
	GeoModel();

	/// \brief This function creates a model with a user defined size. 

	///This is necessary if 
	/// there is no image that shows the model. An example is a system that is based 
	/// on CAD descriptions.
	GeoModel 
	(
		/// Width of the new model.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			   	
		int width, 
		/// Height of the new model.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			   	
		int height
		);


	/// \brief This function creates a model description using the specified image. 

	/// The image is set as the model image for further calls.
	/// \see #FindFeatureMode
	GeoModel 
	(
		/// The source image the model is based on.
		const Image& source, 
		/// Number of model features that should be extracted automatically. 
		/// This must be a number larger than 2 or one of the following constants:
		/// <ul>
		/// <li>ffmModelFindAll: Use all found features from the model.</li>
		/// <li>ffmModelFindAllStable: Use all stable features from the model.</li>
		/// </ul>
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="3"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			   	
		int numberOfFeatures, 
		/// This threshold is used to choose edge pixels for model verification.
		/// Valid in the range [0,255].
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="255"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   	
		int threshold
		);

	/// \brief Create a model from a file based description. 

	///The file format is specific to the mvIMPACT GMM. A model can be saved by calling
	///mvIMPACT::gmm::GeoModel::save
	/// \note If the model file is not found or corrupt an exception is raised. A model references three image files. 
	/// \note if one of them cannot be found the model is considered corrupt.
	///\see mvIMPACT::gmm::GeoModel::save
	GeoModel 
	(
		/// File name of the model description.
		const std::string& fileName
		);


	///\brief Construct a mvIMPACT::gmm::GeoModel from \a src.
	GeoModel(const GeoModel& src);

#ifndef WRAP_ANY
	/// \brief Overwrite current object with \a src.
	GeoModel& operator= (const GeoModel& src);
#endif // #ifndef WRAP_ANY

	///\brief Destructor
	~GeoModel();

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY
	int getUseCount() const; 
	bool getIsUnique   () const; 
	mvIMPACT_C::GMM_MODELHANDLE getHandle  () const; 
#endif // #ifndef WRAP_ANY
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS





	/// \brief Marks a specific corner as "enabled" in the model.
	
	/// This method enables the given corner in the model. All enabled
	/// features are used to calculate the match.
	/// \see mvIMPACT::gmm::GeoModel::enableArc
	/// \see mvIMPACT::gmm::GeoModel::disableCorner
	void enableCorner(
			const Corner& corner);
	


	/// \brief Marks a specific arc as "enabled" in the model.
	
	/// This method enables the given arc in the model. All enabled
	/// features are used to calculate the match.
	/// \see mvIMPACT::gmm::GeoModel::disableArc
	/// \see mvIMPACT::gmm::GeoModel::enableCorner
	void enableArc(
			const Arc& arc);


	/// \brief Marks a specific corner as "disabled" in the model.
	
	/// This method disables the given corner in the model. Disabled
	/// features are contained in the model, but not used to calculate
	/// the match.
	/// \see mvIMPACT::gmm::GeoModel::disableArc
	/// \see mvIMPACT::gmm::GeoModel::enableCorner
	void disableCorner(
			const Corner& corner);
	


	/// \brief Marks a specific arc as "disabled" in the model.
	
	/// This method disables the given arc in the model. Disabled
	/// features are contained in the model, but not used to calculate
	/// the match.
	/// \see mvIMPACT::gmm::GeoModel::enableArc
	/// \see mvIMPACT::gmm::GeoModel::disableCorner
	void disableArc(
			const Arc& arc);



	/// \brief Marks a rectangular area in the model as "don't care".

	/// This function can only be used if a model image is currently set.
	/// The areas marked by this function are respected when calling
	/// mvIMPACT::gmm::GeoModel::selectEdgePoints and mvIMPACT::gmm::GeoModel::findFeatures.
	/// \note If the rectangular area is outside the model image the function 
	/// \note raises an exception.
	/// \see mvIMPACT::gmm::GeoModel::selectEdgePoints
	/// \see mvIMPACT::gmm::GeoModel::findFeatures
	/// \see mvIMPACT::gmm::GeoModel::useRect
	void ignoreRect 
	(
		/// A rectanglar area that will be ignored.
		const Rectangle& rect
		);
	

	/// \brief Marks a rectangular area in the model to be used.

	/// This function can only be used if a model image is currently set.
	/// By default the whole model image is marked as used. The areas marked by this function are
	/// respected when calling mvIMPACT::gmm::GeoModel::selectEdgePoints and mvIMPACT::gmm::GeoModel::findFeatures.
	/// \note If the rectangular area is outside the model image the function
	/// \note raises an exception.
	/// \see mvIMPACT::gmm::GeoModel::selectEdgePoints
	/// \see mvIMPACT::gmm::GeoModel::findFeatures
	/// \see mvIMPACT::gmm::GeoModel::ignoreRect
	void useRect 
	(
		/// A rectanglar area that will be used.
		const Rectangle& rect
		);
	

	/// \brief Selects edge points used for verification.

	/// This function can only be used if a model image is currently set.
	/// The edge points are chosen by applying a threshold.
	/// \see mvIMPACT::gmm::GeoModel::ignoreRect
	/// \see mvIMPACT::gmm::GeoModel::useRect
	void selectEdgePoints 
	(
		/// Threshold used to choose edge pixels for model verification.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="255"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   	
		int threshold
		);
	

	/// \brief Finds model features.

	/// This function can only be used if a model image is currently set.
	/// Call this function to detect features in an image.
	/// \see mvIMPACT::gmm::GeoModel::ignoreRect
	/// \see mvIMPACT::gmm::GeoModel::useRect
	void findFeatures 
	(
		/// Number of model features that should be extracted automatically. 
		/// This must be a number larger than 2 or one of the following constants:
		/// <ul>
		/// <li>ffmModelFindAll: Use all found features from the model.</li>
		/// <li>ffmModelFindAllStable: Use all stable features from the model.</li>
		/// </ul>
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="3"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			   	
		int numberOfFeatures
		);
	

	/// \brief Counts model corners.

	///Returns the number of corners in the model.
	/// If \c scope is set to \c fisOnlyEnabled the method will return the number 
	/// of all enabled corners in the model. If \c scope is set to \c fisOnlyDisabled
	/// the method will return the number of all disabled corners in the model. If
	/// it is set to \c fisAll the number of all corners in the model will be returned, 
	/// no matter if they are enabled or disabled.
	/// \see mvIMPACT::gmm::GeoModel::countCorners
	/// \see mvIMPACT::gmm::GeoModel::countEnabledCorners
	/// \see mvIMPACT::gmm::GeoModel::countDisabledCorners
	/// \see mvIMPACT::gmm::GeoModel::countAllCorners
	/// \see mvIMPACT::gmm::GeoModel::countArcs
	/// \see mvIMPACT::gmm::GeoModel::countEnabledArcs
	/// \see mvIMPACT::gmm::GeoModel::countDisabledArcs
	/// \see mvIMPACT::gmm::GeoModel::countAllArcs
	/// \see #FeatureInquireScope
	int countCorners(
			/// scope of the function
			FeatureInquireScope scope = fisOnlyEnabled) const;
	
	/// \brief Counts enabled model corners.

	/// Returns the number of enabled corners in the model.

	/// \see mvIMPACT::gmm::GeoModel::countCorners
	/// \see mvIMPACT::gmm::GeoModel::countDisabledCorners
	/// \see mvIMPACT::gmm::GeoModel::countAllCorners
	/// \see mvIMPACT::gmm::GeoModel::countArcs
	/// \see mvIMPACT::gmm::GeoModel::countEnabledArcs
	/// \see mvIMPACT::gmm::GeoModel::countDisabledArcs
	/// \see mvIMPACT::gmm::GeoModel::countAllArcs
	/// \see #FeatureInquireScope
	int countEnabledCorners() const;

	/// \brief Counts disabled model corners.

	/// Returns the number of disabled corners in the model.

	/// \see mvIMPACT::gmm::GeoModel::countCorners
	/// \see mvIMPACT::gmm::GeoModel::countEnabledCorners
	/// \see mvIMPACT::gmm::GeoModel::countAllCorners
	/// \see mvIMPACT::gmm::GeoModel::countArcs
	/// \see mvIMPACT::gmm::GeoModel::countEnabledArcs
	/// \see mvIMPACT::gmm::GeoModel::countDisabledArcs
	/// \see mvIMPACT::gmm::GeoModel::countAllArcs
	/// \see #FeatureInquireScope
	int countDisabledCorners() const;

	/// \brief Counts all model corners.

	/// Returns the number of all corners in the model.

	/// \see mvIMPACT::gmm::GeoModel::countCorners
	/// \see mvIMPACT::gmm::GeoModel::countEnabledCorners
	/// \see mvIMPACT::gmm::GeoModel::countDisabledCorners
	/// \see mvIMPACT::gmm::GeoModel::countArcs
	/// \see mvIMPACT::gmm::GeoModel::countEnabledArcs
	/// \see mvIMPACT::gmm::GeoModel::countDisabledArcs
	/// \see mvIMPACT::gmm::GeoModel::countAllArcs
	/// \see #FeatureInquireScope
	int countAllCorners() const;

	/// \brief Counts model arcs.

	/// Returns the number of arcs in the model.

	/// If \c scope is set to \c fisOnlyEnabled the method will return the number 
	/// of all enabled arcs in the model. If \c scope is set to \c fisOnlyDisabled
	/// the method will return the number of all disabled arcs in the model. If
	/// it is set to \c fisAll the number of all arcs in the model will be returned, 
	/// no matter if they are enabled or disabled.
	/// \see mvIMPACT::gmm::GeoModel::countCorners
	/// \see mvIMPACT::gmm::GeoModel::countEnabledCorners
	/// \see mvIMPACT::gmm::GeoModel::countDisabledCorners
	/// \see mvIMPACT::gmm::GeoModel::countAllCorners
	/// \see mvIMPACT::gmm::GeoModel::countEnabledArcs
	/// \see mvIMPACT::gmm::GeoModel::countDisabledArcs
	/// \see mvIMPACT::gmm::GeoModel::countAllCorners
	/// \see #FeatureInquireScope
	int countArcs(
			/// scope of the function
			FeatureInquireScope scope = fisOnlyEnabled) const;
	
	/// \brief Counts model arcs.

	/// Returns the number of arcs in the model.

	/// If \c scope is set to \c fisOnlyEnabled the method will return the number 
	/// of all enabled arcs in the model. If \c scope is set to \c fisOnlyDisabled
	/// the method will return the number of all disabled arcs in the model. If
	/// it is set to \c fisAll the number of all arcs in the model will be returned, 
	/// no matter if they are enabled or disabled.
	/// \see mvIMPACT::gmm::GeoModel::countCorners
	/// \see mvIMPACT::gmm::GeoModel::countEnabledCorners
	/// \see mvIMPACT::gmm::GeoModel::countDisabledCorners
	/// \see mvIMPACT::gmm::GeoModel::countAllCorners
	/// \see mvIMPACT::gmm::GeoModel::countArcs
	/// \see mvIMPACT::gmm::GeoModel::countDisabledArcs
	/// \see mvIMPACT::gmm::GeoModel::countAllArcs
	/// \see #FeatureInquireScope
	int countEnabledArcs() const;

	/// \brief Counts model arcs.

	/// Returns the number of arcs in the model.

	/// If \c scope is set to \c fisOnlyEnabled the method will return the number 
	/// of all enabled arcs in the model. If \c scope is set to \c fisOnlyDisabled
	/// the method will return the number of all disabled arcs in the model. If
	/// it is set to \c fisAll the number of all arcs in the model will be returned, 
	/// no matter if they are enabled or disabled.
	/// \see mvIMPACT::gmm::GeoModel::countCorners
	/// \see mvIMPACT::gmm::GeoModel::countEnabledCorners
	/// \see mvIMPACT::gmm::GeoModel::countDisabledCorners
	/// \see mvIMPACT::gmm::GeoModel::countAllCorners
	/// \see mvIMPACT::gmm::GeoModel::countArcs
	/// \see mvIMPACT::gmm::GeoModel::countEnabledArcs
	/// \see mvIMPACT::gmm::GeoModel::countAllArcs
	/// \see #FeatureInquireScope
	int countDisabledArcs() const;

	/// \brief Counts model arcs.

	/// Returns the number of arcs in the model.

	/// If \c scope is set to \c fisOnlyEnabled the method will return the number 
	/// of all enabled arcs in the model. If \c scope is set to \c fisOnlyDisabled
	/// the method will return the number of all disabled arcs in the model. If
	/// it is set to \c fisAll the number of all arcs in the model will be returned, 
	/// no matter if they are enabled or disabled.
	/// \see mvIMPACT::gmm::GeoModel::countCorners
	/// \see mvIMPACT::gmm::GeoModel::countEnabledCorners
	/// \see mvIMPACT::gmm::GeoModel::countDisabledCorners
	/// \see mvIMPACT::gmm::GeoModel::countAllCorners
	/// \see mvIMPACT::gmm::GeoModel::countArcs
	/// \see mvIMPACT::gmm::GeoModel::countEnabledArcs
	/// \see mvIMPACT::gmm::GeoModel::countDisabledArcs
	/// \see #FeatureInquireScope
	int countAllArcs() const;

	/// \brief Saves model data in a file.
	///The function saves all data that is needed to do pattern matching in a text based file format.
	///This includes:
	/// - Model width and height
	/// - Corners
	/// - Arcs
	/// - Edge points as an image. The file name of the edge image is stored in the file. The mask is not saved!
	void save 
	(
		/// File name where model data will be stored.
		const std::string& fileName
	) const;
	

	/// \brief Checks if a model has valid features.

	/// For models that are created without using the automatic feature selection
	/// it is neccessary to	call this function in order to check if the selected
	/// features are useable. A feature is not useable if another similar feature
	/// is too close to it. In this case one of the features must be deleted. <br>
	/// Returns \c True if the model is valid. \c False for an invalid model.
	/// \note If a model has less then 3 features defined this check always reports an 
	/// \note invalid model.
	bool checkValidity () const;
	
	
	/// \brief Remove a feature from the model.
	void removeCorner
	(
		/// The feature to remove.
		const Corner& corner
		);
	
	///\brief Add a corner to the model.

	/// Adds a corner to the model. If \c scope is set to \c fasEnabled the corner
	/// is added to the set of enabled corners. If \c scope is set to \c fasDisabled
	/// the corner is added to the set of disabled corners.
	/// \see #FeatureAddScope
	void addCorner(
		/// The corner to add
		const Corner& corner,
		/// defines to which set the corner will be added to
		FeatureAddScope scope = fasEnabled
		);
	
	/// \brief Remove a feature from the model.
	void removeArc
	(
		/// The feature to remove.
		const Arc& arc
		);
	
	///\brief Add an arc to the model.

	/// Adds an arc to the model. If \c scope is set to \c fasEnabled the arc
	/// is added to the set of enabled arcs. If \c scope is set to \c fasDisabled
	/// the arc is added to the set of disabled arcs.
	/// \see #FeatureAddScope
	void addArc(
		/// The arc to add
		const Arc& arc,
		/// defines to which set the arc will be added to
		FeatureAddScope scope = fasEnabled

		);

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY
	void removeFromList(mvIMPACT_C::GMM_FEATUREHANDLE handle);
#endif // #ifndef WRAP_ANY
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS


	/// \brief Sets the used and not used areas.
	///
	/// This function can only be used if a model image is currently set.
	/// The image in \c mask is used to define the used areas. This image must
	/// have the same size as the model image. Every pixel in it with a value 
	/// unequal zero indicates a used pixel.
	///\see mvIMPACT::gmm::GeoModel::useRect
	///\see mvIMPACT::gmm::GeoModel::ignoreRect
	///\see mvIMPACT::gmm::GeoModel::getMask
	GeoModel& setMask 
	(
		/// A image that contains the used and not used areas.
		const Image& mask
		);
	

	/// \brief Returns an image that contains the used and not used areas.

	/// This function can only be used if a model image is currently set.
	/// The image has the same size as the model image. Every pixel in the 
	/// returned image with a value unequal zero indicates a used pixel.
	///\see mvIMPACT::gmm::GeoModel::useRect
	///\see mvIMPACT::gmm::GeoModel::ignoreRect
	///\see mvIMPACT::gmm::GeoModel::setMask
	Image getMask () const;
	

#ifndef WRAP_ANY
	/// \brief Returns an image that contains the used and not used areas.

	/// This function can only be used if a model image is currently set.
	/// The image has the same size as the model image. Every pixel in the 
	/// returned image with a value unequal zero indicates a used pixel.
	///\see mvIMPACT::gmm::GeoModel::useRect
	///\see mvIMPACT::gmm::GeoModel::ignoreRect
	///\see mvIMPACT::gmm::GeoModel::setMask
	void getMask 
	(
		/// The destination image. It must have the same as the model image.
		Image& dest
		) const;
#endif	


	/// \brief Sets the edge points for verification.

	/// This function can only be used if a model image is currently set.
	/// The edge points are read from an image that has the same size as the 
	/// model. Every pixel in this image with a value unequal zero is used as 
	/// an edge point.
	GeoModel& setEdgePoints 
	(
		/// A image that contains the edges points.
		const Image& source
		);
	

	/// \brief Returns selected edge points for model verification.

	/// The edge points are returned in an image. All pixels in the image with 
	/// a value unequal zero indicate edge points. The image has the same size
	/// as the model.
	///\see mvIMPACT::gmm::GeoModel::getEdgePoints
	Image getEdgePoints () const;
	

#ifndef WRAP_ANY
	/// \brief Returns selected edge points for model verification.

	/// The edge points are returned in an image. All pixels in the image with 
	/// a value unequal zero indicate edge points. The image has the same size
	/// as the model.
	///\see mvIMPACT::gmm::GeoModel::setEdgePoints
	void getEdgePoints 
	(
		/// The destination image. If must have the same as the model.
		Image& dest
	) const;
#endif	

#ifndef  DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY

	Corner getCornerFromHandle (mvIMPACT_C::GMM_FEATUREHANDLE featureHandle) const;
	

	Arc getArcFromHandle (mvIMPACT_C::GMM_FEATUREHANDLE featureHandle) const;
	
#endif // #ifndef WRAP_ANY
#endif // DOXYGEN_SHOULD_SKIP_THIS

	
public: //properties

	/// \brief Set or change the model image. 
	///
	///Only the image is set. 
	/// You have to select edge points and search or add features to have 
	/// a useful model. This function can be useful if you want to create
	/// a user defined model based on an image but want to set all 
	/// features manually.
	/// \note By calling this function all edge points and model features are 
	/// \note deleted.
	GeoModel& setImage 
	(
		/// The new model image.
		const Image& source
		);
	
	/// \brief Return the model image if any.
	///
	/// Return the model image if there is any. If there is none set, an exception
	/// is thrown.
	Image getImage() const;
	
	/// \brief Return a single model corner
	///
	/// Return model corner with index \a index. The indices start counting at 0.
	Corner getCorner(
		///index of the corner
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.countCorners(fisAll)-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   	
		int index) const;

	/// \brief Return model corners.
	///
	/// If \c scope is set to \c fisOnlyEnabled the method will return 
	/// all enabled corners in the model. If \c scope is set to \c fisOnlyDisabled
	/// the method will return all disabled corners in the model. If
	/// it is set to \c fisAll all corners in the model will be returned, 
	/// no matter if they are enabled or disabled.
	///
	/// \see mvIMPACT::gmm::Corner
	/// \see #FeatureInquireScope
	/// \see mvIMPACT::gmm::GeoModel::getEnabledCorners
	/// \see mvIMPACT::gmm::GeoModel::getDisabledCorners
	/// \see mvIMPACT::gmm::GeoModel::getAllCorners
	std::vector<Corner> getCorners(
			///defines which set of corners to return
			FeatureInquireScope scope = fisOnlyEnabled) const;
	

	/// \brief Return enabled model corners.
	///
	/// \see mvIMPACT::gmm::Corner
	/// \see #FeatureInquireScope
	/// \see mvIMPACT::gmm::GeoModel::getCorners
	/// \see mvIMPACT::gmm::GeoModel::getDisabledCorners
	/// \see mvIMPACT::gmm::GeoModel::getAllCorners
	std::vector<Corner> getEnabledCorners() const;


	/// \brief Return disabled model corners.
	///
	/// \see mvIMPACT::gmm::Corner
	/// \see #FeatureInquireScope
	/// \see mvIMPACT::gmm::GeoModel::getEnabledCorners
	/// \see mvIMPACT::gmm::GeoModel::getCorners
	/// \see mvIMPACT::gmm::GeoModel::getAllCorners
	std::vector<Corner> getDisabledCorners() const;


	/// \brief Return all model corners.
	///
	/// \see mvIMPACT::gmm::Corner
	/// \see #FeatureInquireScope
	/// \see mvIMPACT::gmm::GeoModel::getEnabledCorners
	/// \see mvIMPACT::gmm::GeoModel::getDisabledCorners
	/// \see mvIMPACT::gmm::GeoModel::getCorners
	std::vector<Corner> getAllCorners() const;


	/// \brief Return a single model arc
	///
	/// Return model arc with index \a index. The indices start counting at 0.
	Arc getArc(
		///index of the arc
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.countArcs(fisAll)-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   	
		int index) const;

	/// \brief Return model arcs.
	///
	/// If \c scope is set to \c fisOnlyEnabled the method will return 
	/// all enabled arcs in the model. If \c scope is set to \c fisOnlyDisabled
	/// the method will return all disabled arcs in the model. If
	/// it is set to \c fisAll all arcs in the model will be returned, 
	/// no matter if they are enabled or disabled.
	///\see mvIMPACT::gmm::Arc
	///\see #FeatureInquireScope
	/// \see mvIMPACT::gmm::GeoModel::getEnabledArcs
	/// \see mvIMPACT::gmm::GeoModel::getDisabledArcs
	/// \see mvIMPACT::gmm::GeoModel::getAllArcs
	std::vector<Arc> getArcs(
		///defines which set of arcs to return
		FeatureInquireScope scope = fisOnlyEnabled) const;


	/// \brief Return enabled model arcs.
	///
	///\see mvIMPACT::gmm::Arc
	///\see #FeatureInquireScope
	/// \see mvIMPACT::gmm::GeoModel::getArcs
	/// \see mvIMPACT::gmm::GeoModel::getDisabledArcs
	/// \see mvIMPACT::gmm::GeoModel::getAllArcs
	std::vector<Arc> getEnabledArcs() const;


	/// \brief Return disabled model arcs.
	///
	///\see mvIMPACT::gmm::Arc
	///\see #FeatureInquireScope
	/// \see mvIMPACT::gmm::GeoModel::getEnabledArcs
	/// \see mvIMPACT::gmm::GeoModel::getArcs
	/// \see mvIMPACT::gmm::GeoModel::getAllArcs
	std::vector<Arc> getDisabledArcs() const;


	/// \brief Return all model arcs.
	///
	///\see mvIMPACT::gmm::Arc
	///\see #FeatureInquireScope
	/// \see mvIMPACT::gmm::GeoModel::getEnabledArcs
	/// \see mvIMPACT::gmm::GeoModel::getDisabledArcs
	/// \see mvIMPACT::gmm::GeoModel::getArcs
	std::vector<Arc> getAllArcs() const;


	/// \brief Retrieve the model width.
	int getWidth() const;
	

	/// \brief Retrieve the model height.
	int getHeight() const;
	

	/// \brief Set the scaling steps for stable feature extraction

	/// A feature is considered stable if it occurs also in scaled versions of the model image.
	/// Set this value to a lower value if you want to use very small features. <br>
	/// Scaling starts with a factor of 0.9 (scalingSteps=1) and can go down until 0.5(scalingSteps=5).
	/// The default value is 5.
	/// The valid range is 1 <= scalingSteps <= 5
	///\see mvIMPACT::gmm::GeoModel::getScalingSteps
	GeoModel& setScalingSteps (
				///scaling steps to perform during stable feature detection
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
				/// <mvParamMax><mvParamConst value="5"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   	
				int scalingSteps);
	

	/// \brief Retrieve the scaling steps for stable feature detection

	/// A feature is considered stable if it occurs also in scaled versions of the model image.
	/// Set this value to a lower value if you want to use very small features. <br>
	/// Scaling starts with a factor of 0.9 (scalingSteps=1) and can go down until 0.5(scalingSteps=5).
	///\see mvIMPACT::gmm::GeoModel::setScalingSteps
	int getScalingSteps () const;




	/// \brief Set the x-coordinate of the rotation center. 

	/// The rotation center influences the returned transformations.
	/// Set this value to the x-center of your model to get results more obvious for a human. <br>
	/// The default value is 0.
	///\see mvIMPACT::gmm::GeoModel::getRotationCenterX
	///\see mvIMPACT::gmm::GeoModel::setRotationCenterY
	GeoModel& setRotationCenterX (
				///x-coordinate of the rotation center
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.width-1"/></mvParamMax>
				/// <mvParamStep>0.1</mvParamStep>
				/// </mvParamRange>
				/// \endxmlonly			   	
				double rotationCenterX);
	

	/// \brief Retrieve the x-coordinate of the rotation center.

	/// The rotation center influences the returned transformations.
	/// This value should be in the x-center of your model to get results more obvious for a human. <br>
	/// The default value is 0.
	///\see mvIMPACT::gmm::GeoModel::setRotationCenterX
	///\see mvIMPACT::gmm::GeoModel::getRotationCenterY
	double getRotationCenterX () const;
	

	/// \brief Set the y-coordinate of the rotation center. 

	/// The rotation center influences the returned transformations.
	/// Set this value to the y-center of your model to get results more obvious for a human. <br>
	/// The default value is 0. 
	///\see mvIMPACT::gmm::GeoModel::getRotationCenterY
	///\see mvIMPACT::gmm::GeoModel::setRotationCenterX
	GeoModel& setRotationCenterY (
				/// y-coordinate of the rotation center
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.height-1"/></mvParamMax>
				/// <mvParamStep>0.1</mvParamStep>
				/// </mvParamRange>
				/// \endxmlonly			   	
				double rotationCenterY );
	
	
	/// Retrieve the y-coordinate of the rotation center.

	/// The rotation center influences the returned transformations.
	/// This value should be in the y-center of your model to get results more obvious for a human. <br>
	/// The default value is 0.
	///\see mvIMPACT::gmm::GeoModel::setRotationCenterY
	///\see mvIMPACT::gmm::GeoModel::getRotationCenterX
	double getRotationCenterY () const;
	

	/// \brief Set edge detector threshold in the model image.

	/// Pixels brighter than this threshold are considered as edge pixels.
	/// For images with good contrast this option can be set to a higher value. <br>
	/// The default value is 10.
	/// \see mvIMPACT::gmm::GeoModel::getEdgeDetectorThreshold
	GeoModel& setEdgeDetectorThreshold (
					///the new threshold value
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
					/// <mvParamMax><mvParamConst value="255"/></mvParamMax>
					/// </mvParamRange>
					/// \endxmlonly			   	
					int edgeDetectorThreshold);
	

	/// \brief Retrieve the edge detector threshold.

	/// Pixels brighter than this threshold are considered as edge pixels.
	/// The default value is 10.
	/// \see mvIMPACT::gmm::GeoModel::setEdgeDetectorThreshold
	int getEdgeDetectorThreshold () const;
	
		
	/// \brief Set edge extractor threshold in the model image.

	/// Start to extract edges with pixels that are brighter than this value.
	/// If there is a lot of noise in the image, set this parameter to a higher value. <br>
	/// The default value is 10.
	///\see mvIMPACT::gmm::GeoModel::getUpperThreshold
	GeoModel& setUpperThreshold (
				///the new threshold
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamConst value="255"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   	
				int upperThreshold );
	
	
	/// \brief Retrieve the edge extractor threshold in the model image.

	/// Edges with pixels that are brighter than this value are extracted.
	///\see mvIMPACT::gmm::GeoModel::setUpperThreshold
	int getUpperThreshold () const;
	
	
	/// \brief Set the maximum vectorization error in the model image.

	/// Maximum error for the vectorization algorithm.
	/// For objects without small details this option can be set to a higher value to
	/// reduce influence of noise. <br>
	/// The default value is 0.17.
	///\see mvIMPACT::gmm::GeoModel::getVectorizationError
	GeoModel& setVectorizationError (
					///the maximal error
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
					/// <mvParamStep>0.01</mvParamStep>
					/// </mvParamRange>
					/// \endxmlonly			   	
					double vectorizationError );
	

	/// \brief Retrieve the vectorization error.

	/// Returns the maximum error for the vectorization algorithm.
	/// For objects without small details this option can be set to a higher value to
	/// reduce influence of noise. <br>
	/// The default value is 0.17.
	///\see mvIMPACT::gmm::GeoModel::setVectorizationError
	double getVectorizationError () const;
	

	/// \brief Set minimum ratio value for an arc.

	/// Pixel chain parts are evaluated and yield a curvature score. Pieces 
	/// with a score higher than \c ratio are interpreted as arcs.
	/// All other pieces are interpreted as lines. <br>
	/// The default value is 0.3.
	///\see mvIMPACT::gmm::GeoModel::getMinRatioArc
	GeoModel& setMinRatioArc (
					///the new ratio
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
					/// <mvParamMax><mvParamConst value="1.0"/></mvParamMax>
					/// <mvParamStep>0.1</mvParamStep>
					/// </mvParamRange>
					/// \endxmlonly			   	
					double minRatioArc);
	

	/// \brief Retrieve the minimum ratio value for an arc.

	/// Pixel chain parts are evaluated and yield a curvature score. Pieces 
	/// with a score higher than \c ratio are interpreted as arcs.
	/// All other pieces are interpreted as lines. <br>
	/// The default value is 0.3.
	///\see mvIMPACT::gmm::GeoModel::setMinRatioArc
	double getMinRatioArc() const;
	

private:

	mvIMPACT_C::GMM_FEATUREHANDLE addCornerRaw(const Corner& corner);
	mvIMPACT_C::GMM_FEATUREHANDLE addArcRaw(const Arc& arc);


	/// Dispose the model. If the reference count drops to zero
	/// the model is freed.
	void dispose();
	
	void copy(const GeoModel& src);

	
};



/// \brief This class stores a single match.

/// This includes a translation,rotation,scaling and a matching score.
//-----------------------------------------------------------------------------
class GeoMatchData
//-----------------------------------------------------------------------------
{
public:
	/// \brief get translation in x direction.
	double getTransX() const;
	/// \brief set translation in x direction.
	GeoMatchData& setTransX(double transX);

	/// \brief get Translation in y direction.
	double getTransY() const;
	/// \brief set Translation in y direction.
	GeoMatchData& setTransY(double transY);

	/// \brief get Rotation in radian. 
	
	/// This is always a positive value. It is in the range
	/// [0,2PI[.
	double getRotation() const;
	
	
	/// \brief set Rotation in radian. 
	
	/// This is always a positive value. It is in the range
	/// [0,2PI[.
	GeoMatchData& setRotation(
		/// the rotation value in radian
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="6.2831853071795862"/></mvParamMax>
		/// <mvParamStep>0.01</mvParamStep>
		/// </mvParamRange>
		/// \endxmlonly			   	
		double rotation);

	/// \brief get Scaling factor. 

    ///A value of 1.0 means that the found model instance is
	/// not scaled.
	double getScale() const;

	/// \brief set Scaling factor. 

    ///A value of 1.0 means that the found model instance is
	/// not scaled.
	GeoMatchData& setScale(
		/// scaling factor
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0.1"/></mvParamMin>
		/// <mvParamStep>0.1</mvParamStep>
		/// </mvParamRange>
		/// \endxmlonly			   	
		double scale);
	
	/// \brief get Matching score. 

        ///This value is in the range [minScore,1]. Where \c minScore
	/// is the minimum allowed matching score.
	double getScore() const;

	/// \brief set Matching score. 

    ///This value is in the range [minScore,1]. Where \c minScore
	/// is the minimum allowed matching score.
	GeoMatchData& setScore(
		/// matching score
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="1"/></mvParamMax>
		/// <mvParamStep>0.01</mvParamStep>
		/// </mvParamRange>
		/// \endxmlonly			   	
		double score);
	
	/// \brief get number of arcs confirmed in the scene.
	int getFoundArcs() const;

	/// \brief set number of arcs confirmed in the scene.
	GeoMatchData& setFoundArcs(
		/// number of arcs
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			   	
		int foundArcs);

	/// \brief get number of corners confirmed in the scene.
	int getFoundCorners() const;

	/// \brief set number of corners confirmed in the scene.
	GeoMatchData& setFoundCorners(
		///number of corners
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			   	
		int foundCorners);
	
private:
	double m_transX;
	double m_transY;
	double m_rotation;
	double m_scale;
	double m_score;
	int m_foundArcs;
	int m_foundCorners;
};


class GeoMatchResult;




/// \brief The GeoMatcher class allows to search for a GeoModel in an image.

/// All parameters of the search algorithm can be fine tuned here.
/// The most important parameters of the matcher are:
/// <ul>
/// <li>Maximum rotation that an instance of the model may have.</li>
/// <li>Minimum and maximum scaling of a model %(instance)s.</li>
/// <li>Minimum score</li>
/// </ul>
//-----------------------------------------------------------------------------
class GeoMatcher
//-----------------------------------------------------------------------------
{
public:
	/// \brief Allocates a GeoMatcher object.
	GeoMatcher();

	/// \brief Construct new GeoMatcher object from \a src.
	GeoMatcher(const GeoMatcher& src);
#ifndef WRAP_ANY
	/// \brief Overwrite current object with \a src.
	GeoMatcher& operator= (const GeoMatcher& src);
#endif // #ifndef WRAP_ANY
	
	///\brief Destructor.
	~GeoMatcher();

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY
	int getUseCount() const; 
	bool getIsUnique   () const; 
	mvIMPACT_C::GMM_MATCHERHANDLE getHandle  () const; 
#endif // #ifndef WRAP_ANY
#endif

	/// \brief Search for a GeoModel. 

	///The result is returned in a mvIMPACT::gmm::GeoMatchResult object.
	/// Even if there is nothing found an GeoMatchResult (with 0 entries) is returned.
	///\see mvIMPACT::gmm::GeoMatchResult
	GeoMatchResult findGeoModel 
	(
		/// Model to search for.
		const GeoModel& geoModel,
		/// Source image to search in.
		const Image& source
	);

public: //properties

	/// \brief Set the maximum allowed rotation in both directions.

	/// This value is given in radian. The default is PI which means that full rotation 
	/// is allowed. Only matches where the model rotation does not exceed this value 
	/// (in both directions) are accepted.
	///\see mvIMPACT::gmm::GeoMatcher::getMaxRotation
	GeoMatcher& setMaxRotation (
				///the maximum rotation
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
				/// <mvParamMax><mvParamConst value="3.1415926535897931"/></mvParamMax>
				/// <mvParamStep>0.1</mvParamStep>
				/// </mvParamRange>
				/// \endxmlonly			   	
				double maxRotation);
	

	/// \brief Retrieve the maximum allowed rotation.

	/// This value is given in radian. The default is PI which means that full rotation 
	/// is allowed. Only matches where the model rotation does not exceed this value 
	/// (in both directions) are accepted.
	///\see mvIMPACT::gmm::GeoMatcher::setMaxRotation
	double getMaxRotation () const;
	

	/// \brief Set the minimum allowed scaling. 

	///Only matches where the model scaling does
	/// not fall below this value are accepted.
	/// The default value is 0.5.
	///\see mvIMPACT::gmm::GeoMatcher::getLowerScalingBound
	GeoMatcher& setLowerScalingBound (
					///the new minimum scaling factor
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0.1"/></mvParamMin>
					/// <mvParamMax><mvParamComplex Python="%(instance)s.upperScalingBound"/></mvParamMax>
					/// <mvParamStep>0.1</mvParamStep>
					/// </mvParamRange>
					/// \endxmlonly			   	
					double lowerScalingBound);
	

	/// \brief Retrieve the minimum allowed scaling.

	///Only matches where the model scaling does
	/// not fall below this value are accepted.
	/// The default value is 0.5.
	///\see mvIMPACT::gmm::GeoMatcher::setLowerScalingBound
	double getLowerScalingBound () const;
	

	/// \brief Set the maximum allowed scaling. 

	///Only matches where the model scaling does
	/// not exceed this value are accepted.
	/// The default value is 2.0.
	///\see mvIMPACT::gmm::GeoMatcher::getUpperScalingBound
	GeoMatcher& setUpperScalingBound (
					///the new maximum scaling factor
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamComplex Python="%(instance)s.lowerScalingBound"/></mvParamMin>
					/// <mvParamStep>0.1</mvParamStep>
					/// </mvParamRange>
					/// \endxmlonly			   	
					double upperScalingBound);
	
	
	/// \brief Retrieve the maximum allowed scaling.

	///Only matches where the model scaling does
	/// not exceed this value are accepted.
	/// The default value is 2.0.
	///\see mvIMPACT::gmm::GeoMatcher::setUpperScalingBound
	double getUpperScalingBound () const;
	

	/// \brief Set the minimum score. 

	///The default value is 0.7. Only matches with a 
	/// score higher than this value are accepted. The allowed range
	/// for this parameter is [0,1].
	/// If occluded objects have to be found, reduce this value.
	///\see mvIMPACT::gmm::GeoMatcher::getMinScore
	GeoMatcher& setMinScore (
				///the new minimum score
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
				/// <mvParamMax><mvParamConst value="1.0"/></mvParamMax>
				/// <mvParamStep>0.1</mvParamStep>
				/// </mvParamRange>
				/// \endxmlonly			   	
				double minScore);
	

	/// \brief Retrieve the minium score.

	///The default value is 0.7. Only matches with a 
	/// score higher than this value are accepted. 
	///\see mvIMPACT::gmm::GeoMatcher::setMinScore
	double getMinScore () const;
	

	/// \brief Set number of matches to return.

	/// The matches are sorted descending by score. To retrieve all matches call
	/// this function with \c mmMatcherFindAll. <br>
	/// The default is to return only the very best match.
	///\see mvIMPACT::gmm::GeoMatcher::getMaxNumberOfMatches
	GeoMatcher& setMaxNumberOfMatches (
					///the number of matches to return
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
					/// </mvParamRange>
					/// \endxmlonly			   	
					int numberOfMatches 
					);
	

	/// \brief Retrieve the number of matches to return.

	/// The matches are sorted descending by score.
	/// The default is to return only the very best match.
	///\see mvIMPACT::gmm::GeoMatcher::setMaxNumberOfMatches
	int getMaxNumberOfMatches () const;
	

	/// \brief Minimum gradient value for pixels to be accepted as edge pixels.

	/// This is used while verifying matches. For images where the searched object
	/// has a good contrast this value can be increased to improve reliability. <br>
	/// The default value is 15.
	///\see mvIMPACT::gmm::GeoMatcher::getVerificationThreshold
	GeoMatcher& setVerificationThreshold (
					///the new threshold
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
					/// <mvParamMax><mvParamConst value="255"/></mvParamMax>
					/// </mvParamRange>
					/// \endxmlonly			   	
					int verificationThreshold );
	

	/// Retrieve the model verification threshold.

	///
	/// The default value is 15.
	///\see mvIMPACT::gmm::GeoMatcher::setVerificationThreshold
	int getVerificationThreshold () const;
	

	/// \brief Set a timeout value in milli seconds used when calling mvIMPACT::gmm::GeoMatcher::findModel. 

	/// If the time elapses mvIMPACT::gmm::GeoMatcher::findModel raises an exception. All matches found 
	/// so far can be queried. The quality of these matches may be reduced 
	/// compared to a full run. <br>
	/// The default value is 5000 ms.
	/// \see mvIMPACT::gmm::GeoMatcher::getMaxAllowedTime
	GeoMatcher& setMaxAllowedTime (
					///allowed time in ms
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="1"/></mvParamMin>					
					/// </mvParamRange>
					/// \endxmlonly			   	
					int maxAllowedTime);
	
	
	/// \brief Retrieve the timeout value used for pattern matching in milli seconds.

	/// If the time elapses mvIMPACT::gmm::GeoMatcher::findModel raises an exception. All matches found 
	/// so far can be queried. The quality of these matches may be reduced 
	/// compared to a full run. <br>
	/// The default value is 5000 ms.
	/// \see mvIMPACT::gmm::GeoMatcher::setMaxAllowedTime
	int getMaxAllowedTime () const;
	
	
	/// \brief Set the tolerance used for comparing angles. 

	///This value must be given in radian.
	/// If you specify too high values the matcher will get confused. <br>
	/// The default value is ~0.09 (5 degrees).
	/// \see mvIMPACT::gmm::GeoMatcher::getAngleTolerance
	GeoMatcher& setAngleTolerance (
					///the new angle tolerance
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0.01"/></mvParamMin>
					/// <mvParamMax><mvParamConst value="2.00"/></mvParamMax>
					/// <mvParamStep>0.01</mvParamStep>
					/// </mvParamRange>
					/// \endxmlonly			   	
					double angleTolerance);
	
	
	/// \brief Retrieve the angle tolerance used for comparing angles.

	/// The default value is ~0.09 (5 degrees).
	/// \see mvIMPACT::gmm::GeoMatcher::setAngleTolerance
	double getAngleTolerance () const;
	
	
	/// \brief Set the tolerance for comparing radii.

	/// If you specify too high values the
	/// matcher will get confused. <br>
	/// The default value is 5.
	///\see mvIMPACT::gmm::GeoMatcher::getRadiusTolerance
	GeoMatcher& setRadiusTolerance (
					//new radius tolerance
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0.1"/></mvParamMin>
					/// <mvParamMax><mvParamConst value="10.0"/></mvParamMax>
					/// <mvParamStep>0.1</mvParamStep>
					/// </mvParamRange>
					/// \endxmlonly			   	
					double radiusTolerance);
	
	
	/// \brief Retrieve the tolerance used for comparing radii.

	/// The default value is 5.
	///\see mvIMPACT::gmm::GeoMatcher::setRadiusTolerance
	double getRadiusTolerance () const;
	

	/// \brief Set the tolerance used for comparing distances. 

	///If you specify too high values the
	/// matcher will get confused. <br>
	/// The default value is 5.
	///\see mvIMPACT::gmm::GeoMatcher::getDistanceTolerance
	GeoMatcher&  setDistanceTolerance (
					///the new tolerance
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0.01"/></mvParamMin>
					/// <mvParamMax><mvParamConst value="10.00"/></mvParamMax>
					/// <mvParamStep>0.1</mvParamStep>
					/// </mvParamRange>
					/// \endxmlonly			   	
					double distanceTolerance);
	
	
	/// \brief Retrieve the tolerance used for comparing distances.

	/// The default value is 5.
	///\see mvIMPACT::gmm::GeoMatcher::setDistanceTolerance
	double getDistanceTolerance () const;
	

	/// \brief Set the edge detector threshold in the model image.

	/// Pixels brighter than this threshold are considered as edge pixels.
	/// For images with good contrast this option can be set to a higher value. <br>
	/// The default value is 10.
	///\see mvIMPACT::gmm::GeoMatcher::getEdgeDetectorThreshold
	GeoMatcher& setEdgeDetectorThreshold (
					///the new threshold value
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
					/// <mvParamMax><mvParamConst value="255"/></mvParamMax>
					/// </mvParamRange>
					/// \endxmlonly			   	
					int edgeDetectorThreshold);
	
	
	/// \brief Retrieve the edge detector threshold.

	/// Pixels brighter than this threshold are considered as edge pixels.
	/// The default value is 10.
	///\see mvIMPACT::gmm::GeoMatcher::setEdgeDetectorThreshold
	int getEdgeDetectorThreshold () const;
	

	/// \brief Set the edge extractor threshold in the scene image.

	/// Start to extract edges with pixels that are brighter than this value.
	/// If there is a lot of noise in the image, set this parameter to a higher value. <br>
	/// The default value is 10.
	///\see mvIMPACT::gmm::GeoMatcher::getUpperThreshold
	GeoMatcher& setUpperThreshold (
					///the new threshold value
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
					/// <mvParamMax><mvParamConst value="255"/></mvParamMax>
					/// </mvParamRange>
					/// \endxmlonly			   	
					int upperThreshold );
	
	
	/// Retrieve the edge extractor threshold in the model image.

	/// Pixels that are brighter than this value are extracted to edges.
	/// The default value is 10.
	///\see mvIMPACT::gmm::GeoMatcher::setUpperThreshold
	int getUpperThreshold () const;
	

	/// \brief Set the maximum vectorization error in the scene image.

	/// Maximum error for the vectorization algorithm.
	/// For objects without small details this option can be set to a higher value to
	/// reduce influence of noise. <br>
	/// The default value is 0.17.
	///\see mvIMPACT::gmm::GeoMatcher::getVectorizationError
	GeoMatcher& setVectorizationError (
					///the new maximum error
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
					/// <mvParamStep>0.01</mvParamStep>
					/// </mvParamRange>
					/// \endxmlonly			   	
					double vectorizationError);
	

	/// \brief Retrieve the maximum vectorization error.

	/// Maximum error for the vectorization algorithm.
	/// The default value is 0.17.
	///\see mvIMPACT::gmm::GeoMatcher::setVectorizationError
	double getVectorizationError () const;
	

	/// \brief Minimum ratio value for an arc.

	/// Pixel chain parts are evaluated and yield a curvature score. Pieces 
	/// with a score higher than \c ratio are interpreted as arcs.
	/// All other pieces are interpreted as lines. <br>
	/// The default value is 0.3.
	///\see mvIMPACT::gmm::GeoMatcher::getMinRatioArc
	GeoMatcher& setMinRatioArc (
					///the new ratio
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
					/// <mvParamMax><mvParamConst value="1.0"/></mvParamMax>
					/// <mvParamStep>0.1</mvParamStep>
					/// </mvParamRange>
					/// \endxmlonly			   	
					double minRatioArc);
	
	
	/// \brief Retrieve the minimum ratio value for an arc.

	/// Pixel chain parts are evaluated and yield a curvature score. Pieces 
	/// with a score higher than \c ratio are interpreted as arcs.
	/// All other pieces are interpreted as lines. <br>
	/// The default value is 0.3.
	///\see mvIMPACT::gmm::GeoMatcher::setMinRatioArc
	double getMinRatioArc () const;
	

private:
	/// The buffer handle.
	mvIMPACT_C::GMM_MATCHERHANDLE m_handle;
	/// Unassigned buffer constant.
	enum { unassigned = -1 };
	/// Pointer to the reference counter.
	mutable int* m_counter;

	/// Dispose the matcher. If the reference count drops to zero
	/// and the matcher is freed.
	void dispose();
	
	void copy(const GeoMatcher& src);
	
};



/// \brief This class represents a matching result. 

///An object of this class is returned
/// when calling mvIMPACT::gmm::GeoMatcher::findGeoModel. 
/// It contains several mvIMPACT::gmm::GeoMatchData objects.
/// \note This class is not meant to be instantiated 
/// \note directly.
//-----------------------------------------------------------------------------
class GeoMatchResult
//-----------------------------------------------------------------------------
{
	friend class GeoMatcher;
	
private:
#ifndef DOXYGEN_SHOULD_SKIP_THIS
	GeoMatchResult (const GeoMatcher& matcher, const GeoModel& model);
#endif
	
public:
// only needed by the wrapper code 
	/// \brief Allocates an unassigned GeoMatchResult object.
	GeoMatchResult();
	
	/// \brief Destructor
	~GeoMatchResult();

	///\brief Construct a GeoMatchResult object from \a src.
	GeoMatchResult(const GeoMatchResult& src);
#ifndef WRAP_ANY
	/// \brief Overwrite current object with \a src.
	GeoMatchResult& operator= (const GeoMatchResult& src);
#endif // #ifndef WRAP_PYTHON


	/// \brief Retrieve the number of matches found.

	///Use this method to retriev tha valid range of indices for mvIMPACT::gmm::GeoMatchResult::getMatch.
	///\see mvIMPACT::gmm::GeoMatchResult::getMatch
	int getNumberOfMatches() const;
	

	/// \brief Retrieve the data of a single match.
	
	///
	///\see mvIMPACT::gmm::GeoMatchData
	GeoMatchData getMatch 
	(
		/// Index of the match. This value must be in the range [0,getNumberOfMatches()-1].
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfMatches-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   	
		int index
	) const;
	

	/// \brief This functions returns the corners that correspond to the searched corners
	/// \brief in the model that were actually found in the scene.

	///
	/// \see mvIMPACT::gmm::Corner
	std::vector<Corner> getFoundCorners 
	(
		/// The index of the match where you want to query the found corners.
		/// This value must be in the range [0,getNumberOfMatches()-1].
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfMatches-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   	
		int index
	) const;
	

	/// \brief This functions returns the arcs that correspond to the searched arcs
	/// \brief in the model that were actually found in the scene.
	///
	/// \see mvIMPACT::gmm::Arc
	std::vector<Arc> getFoundArcs 
	(
		/// The index of the match where you want to query the found arcs.
		/// This value must be in the range [0,getNumberOfMatches()-1].
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfMatches-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   	
		int index
	) const;
	

	/// \brief Get the matched corners from the scene
	/// \see mvIMPACT::gmm::Corner
	std::vector<Corner> getSceneCorners
	(
		/// The index of the match where you want to query the found scene corners.
		/// This value must be in the range [0,getNumberOfMatches()-1].
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfMatches-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   	
		int index
	) const;


	/// \brief Get the matched arcs from the scene
	/// \see mvIMPACT::gmm::Arc
	std::vector<Arc> getSceneArcs
	(
		/// The index of the match where you want to query the found scene corners.
		/// This value must be in the range [0,getNumberOfMatches()-1].
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfMatches-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   	
		int index
	) const;


	/// \brief Get the transformed edge points of the model
	/// \see mvIMPACT::Point
	std::vector<Point> getTransformedEdgePoints
	(
		/// The index of the match where you want to query the found scene corners.
		/// This value must be in the range [0,getNumberOfMatches()-1].
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.numberOfMatches-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   	
		int index
	)const;




private:
	/// Interal pointer to the results.
	mvIMPACT_C::GMM_MATCH* m_pMatches;
	/// Number of results.
	mvIMPACT_C::IPL_ULONG m_count;
	/// The matcher that produced this result. This member is needed to quarantee that 
	/// the matcher lives as long as the match result.
	GeoMatcher m_matcher;
	/// The model used for matching. It is needed to be able to query the arcs and corners
	/// of the matches.
	GeoModel m_model;
	/// Pointer to the reference counter.
	mutable int* m_counter;

	/// Dispose the model. If the reference count drops to zero
	/// and the mode is frejed.
	void dispose();
	
	void copy(const GeoMatchResult& src);
};


/// \brief Checks if the GMM license is valid.
bool checkLicenseGMM();

/// \brief Return version number from vl_gmm module.

/// This function returns a version string. The following format is used: 
/// "major.minor.point.build" where "major" specifies the major version number, "minor" specifies
/// the minor version number, "point" specifies a point version number and "build" specifies the build number. 
/// Example: 6.3.1.4529
//-----------------------------------------------------------------------------
std::string getVersionGMM();



	} // namespace gmm
} // namespace mvIMPACT

#include "mvIMPACT_gmm_impl.h"


#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_PYTHON

#ifndef MVIMPACT_USE_NAMESPACES
using namespace mvIMPACT::gmm;
#endif // #ifndef MVIMPACT_USE_NAMESPACES

#endif // #ifndef WRAP_PYTHON
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS


#endif // MVIMPACT_GMM_H_
