#ifndef MVIMPACT_MEASURE_H_
#define MVIMPACT_MEASURE_H_


/// \file
///

#ifndef WRAP_PYTHON
#include "mvIMPACT.h"
#endif // #ifndef WRAP_PYTHON


#ifndef  DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_PYTHON


#include <vl_gauge.h>


#endif // #ifndef WRAP_PYTHON
#endif // DOXYGEN_SHOULD_SKIP_THIS


namespace mvIMPACT {

        /// \namespace mvIMPACT::measure This namespace contains all classes and functions from mvIMPACT Measure.
	namespace measure {

	 
///Polarity modes for single edges.
///For an explanation see \ref mvi_measure_subsubsection_edges_classes_polarity
///\see \ref mvi_measure_section_terms
/// \see MeasureEdges::setPolarity
/// \see MeasureEdges::getPolarity
/// \see MeasureCircles::setPolarity
/// \see MeasureCircles::getPolarity
enum EdgePolarityMode
{
	/// A negative edge is a transition from light to dark.
	epmNegative = mvIMPACT_C::GAU_NEGATIVE,
	/// An edge can have any polarity.
	epmAny = mvIMPACT_C::GAU_ANY,
	/// A positive edge is a transition from dark to light.
	epmPositive = mvIMPACT_C::GAU_POSITIVE
};


/// Specifies the search direction of the analysis module when searching
/// for edge pixel candidates for circles.
/// \see \ref mvi_measure_circles_measurement_direction
/// \see MeasureBase::setMeasurementDirection
/// \image html images/mvi_measure/mvi_measure_circle_window_place.png
enum CircleMeasurementDirection
{
	/// Combines the two methods specified with \c cmdHorizontal and \c cmdVertical. 
	/// You should use this method if you expect horizontal and vertical
	/// oriented edge pixel candidates 
        ///(see measurement window 5 in the figure above).
        ///\image html images/mvi_measure/mvi_measure_circles_dir_any.png
	cmdAny = mvIMPACT_C::GAU_ANY_EDGE,
	/// Edge pixel candidates will be searched from the top and bottom
	/// side until a candidate is found. The procedure goes from left to right. 
	/// You should use this method if you expect more horizontal oriented edge 
	/// pixel candidates (see measurement window 1 in the figure above).
	///\image html images/mvi_measure/mvi_measure_circles_dir_vert.png
	cmdVertical = mvIMPACT_C::GAU_VERTICAL,
	/// Edge pixel candidates will be searched from the left and right
	/// side until a candidate is found. The procedure goes from top to bottom.
	/// You should use this method if you expect more vertical oriented edge 
	/// pixel candidates (see measurement window 3 in the figure above).
	///\image html images/mvi_measure/mvi_measure_circles_dir_hor.png
	cmdHorizontal = mvIMPACT_C::GAU_HORIZONTAL,
	/// Edge pixel candidates will be searched from the left side only until a 
	/// candidate is found. The procedure goes from top to bottom. You should 
	/// use this method if you expect more vertical oriented edge pixel 
	/// candidates (see measurement window 3 in the figure above).
	///\image html images/mvi_measure/mvi_measure_circles_dir_hor_lr.png
	cmdHorizontalLeftToRight = mvIMPACT_C::GAU_HORIZONTAL_LEFT_TO_RIGHT,
	/// Edge pixel candidates will be searched from the right side only until a 
	/// candidate is found. The procedure goes from top to bottom. You should 
	/// use this method if you expect more vertical oriented edge pixel
	/// candidates (see measurement window 2 in the figure above).
	///\image html images/mvi_measure/mvi_measure_circles_dir_hor_rl.png
	cmdHorizontalRightToLeft = mvIMPACT_C::GAU_HORIZONTAL_RIGHT_TO_LEFT,
	/// Edge pixel candidates will be searched from the top side only until a
	/// candidate is found. The procedure goes from left to right. You should 
	/// use this method if you expect more horizontal oriented edge pixel 
	/// candidates (see measurement window 1 in the figure above).
	///\image html images/mvi_measure/mvi_measure_circles_dir_vert_tb.png
	cmdVerticalTopToBottom = mvIMPACT_C::GAU_VERTICAL_TOP_TO_BOTTOM,
	/// Edge pixel candidates will be searched from the bottom side only until a
	/// candidate is found. The procedure goes from left to right. You should 
	/// use this method if you expect more horizontal oriented edge pixel 
	/// candidates (see measurement window 1 and 4 in the figure above. When applying 
	/// to window 4, it forces the measurement module to select only edge pixel 
	/// candidates at the bottom side of the circular object).
	///\image html images/mvi_measure/mvi_measure_circles_dir_vert_bt.png
	cmdVerticalBottomToTop = mvIMPACT_C::GAU_VERTICAL_BOTTOM_TO_TOP
};

/// Specifies the interpolation mode used when placing a measurement window in oblique direction.
/// \see MeasureEdgesBase::setInterpolationMode
/// \see MeasureEdgesBase::getInterpolationMode
enum InterpolationMode
{
	/// Nearest neighbour interpolation
	imNearestNeighbor = mvIMPACT_C::GAU_NEAREST_NEIGHBOUR,
	/// Bilinear interpolation
	imBilinear = mvIMPACT_C::GAU_BILINEAR
};

/// Possible polarity modes when searching for stripes (using class mvIMPACT::measure::MeasureEdgePairs) or toruses
/// (using mvIMPACT::measure::MeasureCirclePair)
/// \see \ref mvi_measure_section_terms
/// \see MeasureEdgePairs::setPairMode
/// \see MeasureCirclePair::setPairMode
enum PairMode
{
	/// Specifies an pair with positive to negative polarity.
	/// This means a bright stripe on a dark background when searching for edge pairs
	/// and respectively a bright torus on a dark background when searching for
	/// a circle pair.
	pmPosNeg = mvIMPACT_C::GAU_PAIR_POS_NEG,
	/// Specifies an pair with negative to positive polarity.
	/// This means a dark stripe on a bright background when searching for edge pairs
	/// and respectively a dark torus on a bright background when searching for
	/// a circle pair.
	pmNegPos = mvIMPACT_C::GAU_PAIR_NEG_POS
};


///Possible kernel types internally used to find extrema.
/// \see MeasureEdgesBase::setKernelType
/// \see MeasureEdgesBase::getKernelType
enum KernelType
{
	/// 5x1 gradient filter.
	ktGradient5x1 = mvIMPACT_C::GAU_5x1,
	/// 3x1 gradient filter.
	ktGradient3x1 = mvIMPACT_C::GAU_3x1,
	/// 2x1 gradient filter.
	ktGradient2x1 = mvIMPACT_C::GAU_2x1
};


/// Available features for single edges.
/// See also \ref mvi_measure_subsection_edges_features.
/// \note Currently some of the features are only available if the 'bestMatchOnly' flag has been set to \c true !
/// \see MeasureEdges::setBestMatchOnly
/// \see MeasureEdges::addFeature
/// \see MeasureEdges::getFeature
enum EdgeFeature
{
	/// The x-position of an edge will be calculated.
	efXPos = mvIMPACT_C::GAU_EDGE_XPOS,
	/// The y-position of an edge will be calculated.
	efYPos = mvIMPACT_C::GAU_EDGE_YPOS,
	/// The polarity of an edge will be calculated.
	efPolarity = mvIMPACT_C::GAU_EDGE_POLARITY,
	/// The contrast of an edge will be calculated.
	efContrast = mvIMPACT_C::GAU_EDGE_CONTRAST,
	/// The steepness of an edge will be calculated.
	efSteepness = mvIMPACT_C::GAU_EDGE_STEEPNESS,
	/// The match score of an edge will be calculated.
	efScore = mvIMPACT_C::GAU_EDGE_SCORE,
	/// The slope of an edge will be calculated.
	efSlope = mvIMPACT_C::GAU_EDGE_SLOPE,
	/// The intersection point with the y-axis will be calculated.
	///\note Currently this feature is only available if the 'bestMatchOnly' flag has been set to \c true.
	efYIntercept = mvIMPACT_C::GAU_EDGE_Y_INTERCEPT,
	/// The angle between the horizontal axis and the edge will be calculated.
	///\note Currently this feature is only available if the 'bestMatchOnly' flag has been set to \c true.
	efHorizontalAngle = mvIMPACT_C::GAU_EDGE_HOR_ANGLE,
	/// The x-coordinate of the first intersection point with the image border will be calculated.
	///\note Currently this feature is only available if the 'bestMatchOnly' flag has been set to \c true.
	efX0Intersect = mvIMPACT_C::GAU_EDGE_X0_INTERCEPT,
	/// The y-coordinate of the first intersection point with the image border will be calculated.
	///\note Currently this feature is only available if the 'bestMatchOnly' flag has been set to \c true.
	efY0Intersect = mvIMPACT_C::GAU_EDGE_Y0_INTERCEPT,
	/// The x-coordinate of the second intersection point with the image border will be calculated.
	///\note Currently this feature is only available if the 'bestMatchOnly' flag has been set to \c true.
	efX1Intersect = mvIMPACT_C::GAU_EDGE_X1_INTERCEPT,
	/// The y-coordinate of the second intersection point with the image border will be calculated.
	///\note Currently this feature is only available if the 'bestMatchOnly' flag has been set to \c true.
	efY1Intersect = mvIMPACT_C::GAU_EDGE_Y1_INTERCEPT,
	/// The x-coordinate of edge points will be calculated.
	///\note Currently this feature is only available if the 'bestMatchOnly' flag has been set to \c true.
	efXPoint = mvIMPACT_C::GAU_EDGE_POINT_X,
	/// The y-coordinate of edge points will be calculated.
	///\note Currently this feature is only available if the 'bestMatchOnly' flag has been set to \c true.
	efYPoint = mvIMPACT_C::GAU_EDGE_POINT_Y
};


/// Available features for edge pairs (stripes).
/// See also \ref mvi_measure_subsection_edges_features.
/// \see \ref mvi_measure_section_terms
/// \see MeasureEdgePairs::addFeature
/// \see MeasureEdgePairs::getFeature
enum EdgePairFeature
{
	/// The x-position of the first edge will be calculated.
	epfXPos1 = mvIMPACT_C::GAU_EDGE_XPOS,
	/// The x-position of the second edge will be calculated.
	epfXPos2 = mvIMPACT_C::GAU_EDGE_XPOS_2,
	/// The y-position of the first edge will be calculated.
	epfYPos1 = mvIMPACT_C::GAU_EDGE_YPOS,
	/// The y-position of the second edge will be calculated.
	epfYPos2 = mvIMPACT_C::GAU_EDGE_YPOS_2,
	/// The polarity of the first edge will be calculated.
	epfPolarity1 = mvIMPACT_C::GAU_EDGE_POLARITY,
	/// The polarity of the second edge will be calculated.
	epfPolarity2 = mvIMPACT_C::GAU_EDGE_POLARITY_2,
	/// The contrast of the first edge will be calculated.
	epfContrast1 = mvIMPACT_C::GAU_EDGE_CONTRAST,
	/// The contrast of the second edge will be calculated.
	epfContrast2 = mvIMPACT_C::GAU_EDGE_CONTRAST_2,
	/// The steepness of the first edge will be calculated.
	epfSteepness1 = mvIMPACT_C::GAU_EDGE_STEEPNESS,
	/// The steepness of the second edge will be calculated.
	epfSteepness2 = mvIMPACT_C::GAU_EDGE_STEEPNESS_2,
	/// The match score of the first edge will be calculated.
	epfScore1 = mvIMPACT_C::GAU_EDGE_SCORE,
	/// The match score of the second edge will be calculated.
	epfScore2 = mvIMPACT_C::GAU_EDGE_SCORE_2,
	/// The distance between the first and second edge will be calculated.
	epfDistance = mvIMPACT_C::GAU_EDGE_PAIR_DIST,
	/// The mean x-position between the first and second edge of an edge pair will be calculated.
	epfMeanXPos = mvIMPACT_C::GAU_EDGE_XPOS_MEAN,
	/// The mean y-position between the first and second edge of an edge pair will be calculated.
	epfMeanYPos = mvIMPACT_C::GAU_EDGE_YPOS_MEAN,
	/// The mean contrast of the first and second edge will be calculated.
	epfMeanContrast = mvIMPACT_C::GAU_EDGE_CONTRAST_MEAN,
	/// The mean steepness of the first and second edge will be calculated.
	epfMeanSteepness = mvIMPACT_C::GAU_EDGE_STEEPNESS_MEAN,
	/// The mean match score of the first and second edge will be calculated.
	epfMeanScore = mvIMPACT_C::GAU_EDGE_SCORE_MEAN
};

/// Available features for circles.
/// See also \ref mvi_measure_subsection_circles_features
/// \see MeasureCircles::addFeature
/// \see MeasureCirclePair::addFeature
/// \see MeasureCircles::getFeature
/// \see MeasureCirclePair::getFeature
enum CircleFeature
{
	/// Calculates the circle radius.
	cfRadius = mvIMPACT_C::GAU_CIRCLE_RADIUS,
	/// Calculates the x-coordinate of the circle's center point.
	cfXCenter = mvIMPACT_C::GAU_CIRCLE_CEN_X,
	/// Calculates the y-coordinate of the circle's center point.
	cfYCenter = mvIMPACT_C::GAU_CIRCLE_CEN_Y,
	/// Number of circles found.
	cfNumber = mvIMPACT_C::GAU_CIRCLE_NUMBER,


#ifndef DOXYGEN_SHOULD_SKIP_THIS  //these values should not appear in the documentation anymore
	/// Do not use, just for compatibility. 
	cfTolerance = mvIMPACT_C::GAU_CIRCLE_TOLERANCE,
	/// Do not use, just for compatibility. 
	cfIterations = mvIMPACT_C::GAU_CIRCLE_ITERATIONS,
#endif

	/// Specifies the minimum radius. It is calculated from the point which is nearest to the calculated center.
	cfMinRadius = mvIMPACT_C::GAU_CIRCLE_RADIUS_MIN,
	/// Specifies the maximum radius. It is calculated from the point which is most distant to the calculated center.
	cfMaxRadius = mvIMPACT_C::GAU_CIRCLE_RADIUS_MAX,
	/// Specifies the x-coordinate of the point on the circle which is nearest to the center.
	cfXMinRadius = mvIMPACT_C::GAU_CIRCLE_RADIUS_MIN_X,
	/// Specifies the y-coordinate of the point on the circle which is nearest to the center.
	cfYMinRadius = mvIMPACT_C::GAU_CIRCLE_RADIUS_MIN_Y,
	/// Specifies the x-coordinate of the point on the circle which is most distant to the center.
	cfXMaxRadius = mvIMPACT_C::GAU_CIRCLE_RADIUS_MAX_X,
	/// Specifies the y-coordinate of the point on the circle which is most distant to the center.
	cfYMaxRadius = mvIMPACT_C::GAU_CIRCLE_RADIUS_MAX_Y,
};


/// \brief Base class for all measurement tasks. It provides basic tasks for all measurement
/// \brief classes. These are the methods to calibrate the coordinate system.
/// \brief To do this use the following methods 
/// \brief  - mvIMPACT::measure::MeasureBase::setCoordinateSystem 
/// \brief  - mvIMPACT::measure::MeasureBase::getCoordinateSystem 
/// \brief <br>
/// \brief Please see 
/// \brief \ref mvi_measure_section_calibrate for more information on calibration.<br><br>
/// \brief This class is never instantiated directly. Always use one of the derived
/// \brief classes.
/// \brief \see MeasureEdges if you want to measure single edges.
/// \brief \see MeasureEdgePairs if you want to measure edge pairs.
/// \brief \see MeasureCircles if you want to measure single circles.
/// \brief \see MeasureCirclePair if you want to measure circle pairs.
//-----------------------------------------------------------------------------
class MeasureBase 
//-----------------------------------------------------------------------------
{
public:

        /// \brief Destructor.
	~MeasureBase();

protected:
	///\brief Allocates a MeasureBase object.
	MeasureBase();


	///\brief Create MeasureBase object from \a src.
	MeasureBase(const MeasureBase& src);

#ifndef WRAP_PYTHON
	///\brief Overwrite current object with \a src.
	MeasureBase& operator= (const MeasureBase& src);
#endif // #ifndef WRAP_PYTHON


#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY
	int getUseCount() const; 
	bool getIsUnique   () const; 
	mvIMPACT_C::GAU_BUFHANDLE getHandle  () const; 
#endif // #ifndef WRAP_ANY
#endif

protected:
	/// The buffer handle.
	mvIMPACT_C::GAU_BUFHANDLE handle_;
	/// Unassigned buffer constant.
	enum { unassigned = -1 };

private:
	/// Pointer to the reference counter.
	mutable int* counter_;

	/// Dispose the buffer. 
	/// If the reference count drops to zero
	/// and the buffer was allocated it is freed.
	void dispose();
	
	void copy(const MeasureBase& src);

public: // Properties

#ifndef WRAP_DOTNET		// this cannot be wrapped because of the set prefix and the multiple parameters
	/// \brief Set the calibration coordinate system.

	/// This function allows you to calibrate your system and to set the coordinates system's origin
	/// and rotation angle. Changing one of these parameters will influence most of the measurement 
	/// results. By default, all results will be returned in pixel coordinates. The
	/// default values of the parameters are as follows:
	/// <ul>
	/// <li>originX: 0</li>
	/// <li>originY: 0</li>
	/// <li>scaleX: 1</li>
	/// <li>scaleY: 1</li>
	/// <li>angle: 0</li>
	/// </ul>
	///
	/// \see \ref mvi_measure_section_calibrate
	MeasureBase& setCoordinateSystem 
	(
		/// Specifies the x-coordinate of the coordinate system origin. By default, the origin is
		/// located in the top left corner (0, 0). This value must be set with respect to the image
		/// origin (top left corner).
		double originX, 
		/// Specifies the y-coordinate of the coordinate system origin. By default, the origin is
		/// located in the top left corner (0, 0). This value must be set with respect to the image 
		/// origin (top left corner).
		double originY, 
		/// Specifies the scale factor in x-direction. The default value is 1.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0.01"/></mvParamMin>
		/// <mvParamStep>0.1</mvParamStep>
		/// </mvParamRange>
		/// \endxmlonly			
		double scaleX, 
		/// Specifies the scale factor in y-direction. The default value is 1.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0.01"/></mvParamMin>
		/// <mvParamStep>0.1</mvParamStep>
		/// </mvParamRange>
		/// \endxmlonly			
		double scaleY, 
		/// Specifies the rotation angle in degrees between the x-axis of the coordinate system
		/// and the horizontal axis. By default the rotation angle is set to 0.0 degrees.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="-360.0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="360.0"/></mvParamMax>
		/// <mvParamStep>0.1</mvParamStep>
		/// </mvParamRange>
		/// \endxmlonly			
		double angle
		);
#endif //WRAP_DOTNET	

#ifndef WRAP_ANY
	/// \brief Retrieve the current calibration coordinate system.
	/// \see \ref mvi_measure_section_calibrate
	void getCoordinateSystem
	(
		/// The x-coordinate of the coordinate system origin is returned here.
		double& originX, 
		/// The y-coordinate of the coordinate system origin is returned here.
		double& originY, 
		/// The scale factor in x direction is returned here.
		double& scaleX, 
		/// The scale factor in y direction is returned here.
		double& scaleY, 
		/// The rotation angle is returned here.
		double& angle
		) const;
#endif //WRAP_ANY
	
	
	/// \brief Set the calibration coordinate system.

	/// This function allows you to calibrate your system and to set the coordinates system's origin
	/// and rotation angle. Changing one of these parameters will influence most of the measurement 
	/// results. By default, all results will be returned in pixel coordinates. 
	///
	/// \see \ref mvi_measure_section_calibrate
	MeasureBase& setCoordinateSystem 
	(
		Position position
	);
	

	/// \brief Retrieve the current calibration coordinate system.
	/// \see \ref mvi_measure_section_calibrate
	Position getCoordinateSystem() const;	
};





/// \brief Base class for all edge measurement tasks.<br><br>
/// \brief This class is never instantiated directly. Always use one of the derived
/// \brief classes. The methods of this class provide retrieving and setting
/// \brief the edge property values and other global parameters when searching
/// \brief for edges. For more information on edge properties 
/// \brief see \ref mvi_measure_subsubsection_edges_classes_properties
/// \brief \see mvIMPACT::measure::MeasureEdges
/// \brief \see mvIMPACT::measure::MeasureEdgePairs
//-----------------------------------------------------------------------------
class MeasureEdgesBase : public MeasureBase
//-----------------------------------------------------------------------------
{
protected:
	/// \brief Allocates a MeasureEdgesBase object.
	MeasureEdgesBase();

	///\brief Create MeasureEdgesBase object from \a src.
	MeasureEdgesBase(const MeasureEdgesBase& src);

#ifndef WRAP_PYTHON
	///\brief Overwrite current object with \a src.
	MeasureEdgesBase& operator= (const MeasureEdgesBase& src);
#endif // #ifndef WRAP_PYTHON


public: // Properties
	/// \brief Specifies the minimum match score of an edge, that must
	/// \brief be reached, in order to be recognized by the analysis module.

	/// The score is in the range [0..100].
	/// If this function is called while measuring edge pairs it sets the 
	/// minimum score for the first edge. 
	MeasureEdgesBase& setMinScore (
		///minimal score
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="100.0"/></mvParamMax>
		/// <mvParamStep>0.1</mvParamStep>
		/// </mvParamRange>
		/// \endxmlonly			
		double minScore);
	

	/// \brief Retrieve the current minimum match score.

	/// If this function is called while measuring edge pairs it returns the 
	/// minimum score for the first edge.
	double getMinScore() const;
	

	/// \brief Specifies the interpolation mode used when placing a measurement window
	/// \brief in oblique direction. 

	/// See \ref mvi_measure_subsection_measure_edges_windows for information on measurement windows.
	/// \see mvIMPACT::measure::InterpolationMode
	MeasureEdgesBase& setInterpolationMode (InterpolationMode mode);
	

	/// \brief Retrieve the current interpolation mode for oblique measurement windows.

	/// See \ref mvi_measure_subsection_measure_edges_windows for information on measurement windows.
	/// \see mvIMPACT::measure::InterpolationMode
	InterpolationMode getInterpolationMode () const;
	

	/// \brief Enables or disables the automatic contrast adjustment.

	/// This controls how the contrast value contributes to the matching score:
	/// <ul>
	/// <li>
	///	Set to \c true: The value for the typical contrast will be calculated 
	/// automatically. The computed value is used only, if it is higher than the
	/// value set with mvIMPACT::measure::MeasureEdgesBase::setTypicalContrast). 
	/// If it is lower the value set
	/// with mvIMPACT::measure::MeasureEdgesBase::setTypicalContrast is used.
	/// </li>
	/// <li>
	/// Set to \c false: The value set with mvIMPACT::measure::MeasureEdgesBase::setTypicalContrast is always used.
	/// </li>
	/// </ul>
	/// \see \ref mvi_measure_section_terms
	/// \see \ref mvi_measure_subsubsection_edges_classes_contrast
	MeasureEdgesBase& setAutomaticContrast (bool automaticContrast);
	

	/// \brief Query if automatic contrast adjustment is switched on or off.

	/// \see \ref mvi_measure_section_terms
	/// \see \ref mvi_measure_subsubsection_edges_classes_contrast
	bool getAutomaticContrast () const;
	
	
	/// \brief Set the typical (expected) contrast of an edge. The contrast contributes
	/// \brief to the match score.

	/// If this function is called while measuring edge pairs it sets the typical contrast
	/// for the first edge. 
	/// \see \ref mvi_measure_section_terms
	/// \see \ref mvi_measure_subsubsection_edges_classes_contrast
	MeasureEdgesBase& setTypicalContrast (
		/// typical contrast
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
		/// <mvParamStep>0.1</mvParamStep>
		/// </mvParamRange>
		/// \endxmlonly			
		double typicalContrast);
	

	/// \brief Retrieve the typical contrast of an edge.

	/// If this function is called while measuring edge pairs it returns the 
	/// typical contrast for the first edge.
	/// \see \ref mvi_measure_section_terms
	/// \see \ref mvi_measure_subsubsection_edges_classes_contrast
	double getTypicalContrast () const;
	

	/// \brief Enables or disables the automatic steepness adjustment.

	/// This controls how the steepness value contributes to the matching score:
	/// <ul>
	/// <li>
	///	Set to true: The value for the typical steepness will be calculated 
	/// automatically. The computed value is used only, if it is higher than the
	/// value set with mvIMPACT::measure::MeasureEdgesBase::setTypicalSteepness. 
	/// If it is lower the value set
	/// with mvIMPACT::measure::MeasureEdgesBase::setTypicalSteepness is used.
	/// </li>
	/// <li>
	/// Set to false: The value set with mvIMPACT::measure::MeasureEdgesBase::setTypicalSteepness 
	/// is always used.
	/// </li>
	/// </ul>
	/// \see \ref mvi_measure_section_terms
	/// \see \ref mvi_measure_subsubsection_edges_classes_steepness
	MeasureEdgesBase& setAutomaticSteepness (bool automaticSteepness);
	

	/// \brief Query if automatic steepness adjustment is switched on or off.

	/// \see \ref mvi_measure_section_terms
	/// \see \ref mvi_measure_subsubsection_edges_classes_steepness
	bool getAutomaticSteepness () const;
	

	/// \brief Set the typical (expected) steepness of an edge. The steepness contributes
	/// \brief to the match score.

	/// If this function is called while measuring edge pairs it sets the typical steepness
	/// for the first edge. 
	/// \see \ref mvi_measure_section_terms 
	/// \see \ref mvi_measure_subsubsection_edges_classes_steepness
	MeasureEdgesBase& setTypicalSteepness (
		///typical steepness
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0.1"/></mvParamMin>
		/// <mvParamStep>0.1</mvParamStep>
		/// </mvParamRange>
		/// \endxmlonly			
		double typicalSteepness);
	

	/// \brief Retrieve the typical steepness of an edge.

	/// If this function is called while measuring edge pairs it returns the 
	/// typical steepness for the first edge.
	/// \see \ref mvi_measure_section_terms
	/// \see \ref mvi_measure_subsubsection_edges_classes_steepness
	double getTypicalSteepness () const;
	

	/// \brief Specifies a weight factor for the contrast match score. 

	/// The total
	/// match score is calculated from the sum of all single
	/// match scores. Please note, that all weight factors must sum up to 1.
	/// The weight is in the range [0..1].
	/// If this function is called while measuring edge pairs it sets the 
	/// weight for the first edge.
	/// \see \ref mvi_measure_section_terms
	/// \see \ref mvi_measure_subsubsection_edges_classes_contrast
	MeasureEdgesBase& setContrastWeight (
		///weight factor
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="1.0"/></mvParamMax>
		/// <mvParamStep>0.01</mvParamStep>
		/// </mvParamRange>
		/// \endxmlonly			
		double contrastWeight);
	

	/// \brief Retrieve the weight for the contrast match score.

	/// If this function is called while measuring edge pairs it returns the 
	/// weight for the first edge.
	/// \see \ref mvi_measure_section_terms 
	/// \see \ref mvi_measure_subsubsection_edges_classes_contrast
	double getContrastWeight () const;
	

	/// \brief Specifies a weight factor for the steepness match score. 

	/// The total
	/// match score is calculated from the sum of all single
	/// match scores. Please note, that all weight factors must sum up to 1.
	/// The weight is in the range [0..1].
	/// If this function is called while measuring edge pairs it sets the 
	/// weight for the first edge.
	/// \see \ref mvi_measure_section_terms
	/// \see \ref mvi_measure_subsubsection_edges_classes_steepness
	MeasureEdgesBase& setSteepnessWeight (
		/// weight factor
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="1.0"/></mvParamMax>
		/// <mvParamStep>0.1</mvParamStep>
		/// </mvParamRange>
		/// \endxmlonly					
		double steepnessWeight);
	

	/// \brief Retrieve the weight for the steepness match score.

	/// If this function is called while measuring edge pairs it returns the 
	/// weight for the first edge.
	/// \see \ref mvi_measure_section_terms
	/// \see \ref mvi_measure_subsubsection_edges_classes_steepness
	double getSteepnessWeight () const;
	
	
	/// \brief Specifies a weight factor for the polarity match score. 

	/// The total
	/// match score is calculated from the sum of all single
	/// match scores. Please note, that all weight factors must sum up to 1.
	/// The weight is in the range [0..1].
	/// If this function is called while measuring edge pairs it sets the 
	/// weight for the first edge.
	/// \see \ref mvi_measure_section_terms 
	/// \see \ref mvi_measure_subsubsection_edges_classes_polarity
	MeasureEdgesBase& setPolarityWeight (
		/// polarity weight factor
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="1.0"/></mvParamMax>
		/// <mvParamStep>0.1</mvParamStep>
		/// </mvParamRange>
		/// \endxmlonly			
		double polarityWeight);
	

	/// \brief Retrieve the weight for the polarity match score.

	/// If this function is called while measuring edge pairs it returns the 
	/// weight for the first edge.
	/// \see \ref mvi_measure_section_terms
	/// \see \ref mvi_measure_subsubsection_edges_classes_polarity
	double getPolarityWeight () const;
	

	/// \brief Specify whether or not only the edge with the best match
	/// \brief score will be returned.

	/// If multiple edges have been found, and this flag has been set \c true 
	/// only the edge with the best match score will be returned. If the flag
	/// was set to \c false all edges will be returned.
	/// \see mvIMPACT::measure::MeasureEdgesBase::getBestMatchOnly
	MeasureEdgesBase& setBestMatchOnly (bool bestMatchOnly);
	

	/// \brief Query if only the best match is returned.
	
	/// Returns \c true if only the edge with the best match is returned or
	/// \c false if all edges will be returned.
	/// \see mvIMPACT::measure::MeasureEdgesBase::setBestMatchOnly
	bool getBestMatchOnly () const;
	

	/// \brief Specifies the kernel type internally used to find extrema.
	
	///The kernel type given here will be used to detect the edges.
	/// \see mvIMPACT::measure::KernelType
	MeasureEdgesBase& setKernelType (KernelType kernelType);
	

	/// \brief Retrieve the kernel type internally used to find extrema.

	/// Returns the kernel type that was selected with 
	/// mvIMPACT::measure::MeasureEdgesBase::setKernelType
	KernelType getKernelType () const;
	

	/// \brief Specifies the number of times to smooth the profile along the
	/// \brief search line. 

	/// You should use profile smoothing, if you want to measure
	/// noisy edges. 
	/// This value must be in the range [0.10]. 0 disables smoothing.
	/// \see mvIMPACT::measure::MeasureEdgesBase::getSmoothPasses
	MeasureEdgesBase& setSmoothPasses (
		/// number of smooth passes
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="10"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int smoothPasses);
	

	/// \brief Retrieve the number of smooting passes applied to the profile along
	/// \brief the search line.
	/// \see mvIMPACT::measure::MeasureEdgesBase::getSmoothPasses
	int getSmoothPasses () const;
	
};



/// \brief This class measures single edges. It is derived from MeasureEdgesBase and contains
/// \brief all the necessary methods to perform the mesurement on single edges. For more information
/// \brief on measuring edges see \ref mvi_measure_subsection_measure_edges_windows.


/// The general approach is as follows:
/// Specify features to search for
/// with one or more calls to mvIMPACT::measure::MeasureEdges::addFeature. Then call 
/// mvIMPACT::measure::MeasureEdges::find and query the 
/// results with mvIMPACT::measure::MeasureEdges::count and 
/// mvIMPACT::measure::MeasureEdges::getFeature . 
/// Features that can be measured are defined in enumeration #EdgeFeature.
/// For more information on measuring edges see \ref mvi_measure_section_edges.
/// This little example retrieves the x-coordinate of any edge inside the search window.<br>
///
/// \xmlonly
/// <mvWrapperDoc><htmlinclude ref="code_fragment_start_cpp.html"/></mvWrapperDoc>
/// \endxmlonly
/// \htmlinclude code_fragment_start_cpp.html
/// \code
/// Image img = loadImage("sample.bmp");
/// 
/// MeasureEdges me;
/// me.addFeature (efXPos);
/// me.find(img,50,100,250,100,21);
/// 
/// int count = me.count();
/// for (int i=0; i<count; ++i)
/// {
///    double xpos = me.getFeature(efXPos,i);
/// }
/// \endcode
/// \htmlinclude code_fragment_end.html
/// \xmlonly
/// <mvWrapperDoc><htmlinclude ref="code_fragment_end.html"/></mvWrapperDoc>
/// \endxmlonly
///
/// \xmlonly
/// <mvWrapperDoc>
/// <htmlinclude ref="code_fragment_start_cs.html"/>
/// \code
/// Image img = gBase.loadImage("sample.bmp");
/// 
/// MeasureEdges me = new MeasureEdges();
/// me.addFeature (EdgeFeature.efXPos);
/// me.find(img,50,100,250,100,21);
/// 
/// int count = me.count();
/// for (int i=0; i&#x3c;count; ++i)
/// {
///    double xpos = me.getFeature(EdgeFeature.efXPos,i);
/// }
/// \endcode
/// <htmlinclude ref="code_fragment_end.html"/>
/// </mvWrapperDoc>
/// \endxmlonly
///
/// \xmlonly
/// <mvWrapperDoc>
/// <htmlinclude ref="code_fragment_start_vb_net.html"/>
/// \code
/// Dim img As Image = gBase.loadImage("sample.bmp")
/// 
/// Dim me As MeasureEdges = new MeasureEdges()
/// me.addFeature(EdgeFeature.efXPos)
/// me.find(img,50,100,250,100,21)
/// 
/// Dim count As Int = me.count()
/// For i As Integer = 0 To count-1
///    Dim xpos As Double = me.getFeature(EdgeFeature.efXPos,i)
/// Next i
/// \endcode
/// <htmlinclude ref="code_fragment_end.html"/>
/// </mvWrapperDoc>
/// \endxmlonly
///
/// \xmlonly
/// <mvWrapperDoc>
/// <htmlinclude ref="code_fragment_start_python.html"/>
/// \code
/// img = mvIMPACT.loadImage("sample.bmp")
/// 
/// me = measure.MeasureEdges()
/// me.addFeature (measure.efXPos)
/// me.find(img,50,100,250,100,21)
/// 
/// count = me.count()
/// for i in range(count):
///    xpos = me.getFeature(measure.efXPos,i)
/// 
/// \endcode
/// <htmlinclude ref="code_fragment_end.html"/>
/// </mvWrapperDoc>
/// \endxmlonly
/// \see \ref mvi_measure_section_edges
//-----------------------------------------------------------------------------
class MeasureEdges : public MeasureEdgesBase
//-----------------------------------------------------------------------------
{
public:
	/// \brief Allocates a MeasureEdges object.
	MeasureEdges();

	/// \brief Create a MeasureEdges object from \a src.
	MeasureEdges(const MeasureEdges& src);

#ifndef WRAP_PYTHON
	/// \brief Overwrite current object with \a src.
	MeasureEdges& operator= (const MeasureEdges& src);
#endif // #ifndef WRAP_PYTHON
	

	/// \brief Add a feature to search for. 

	/// After the
	/// mvIMPACT::measure::MeasureEdges::find method is called all
	/// features added with this function can be queried.
	/// For a list of features see enumeration #EdgeFeature
	void addFeature (EdgeFeature edgeFeature);
	

	/// \brief Search for edges.

	/// This function locates edges in a source image in a given search window.
	/// The search window is defined by a search line and a width:
	/// After the search is completed the results can be queried with 
	/// mvIMPACT::measure::MeasureEdges::count
	/// and mvIMPACT::measure::MeasureEdges::getFeature .
	/// \see \ref mvi_measure_subsection_measure_edges_windows
	void find 
	(
		/// The source image.
		const Image& source, 
		/// The x-coordinate of the start point.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(source)s.width-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int startX, 
		/// The y-coordinate of the start point.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(source)s.height-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int startY, 
		/// The x-coordinate of the end point.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(source)s.width-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int endX, 
		/// The y-coordinate of the end point.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(source)s.height-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int endY, 
		/// The width of the window given as the perpendicular distance from 
		/// the search line. This must be an odd number.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			
		int width
		);
	

	/// \brief Search for edges.

	/// This function locates edges in a source image in a given search window.
	/// The search window is defined by a search line and a width:
	/// After the search is completed the results can be queried with 
	/// mvIMPACT::measure::MeasureEdges::count
	/// and mvIMPACT::measure::MeasureEdges::getFeature .
	/// \see \ref mvi_measure_subsection_measure_edges_windows
	void find 
	(
		/// The source image.
		const Image& source, 
		/// The search line
		Line line,
		/// The width of the window given as the perpendicular distance from 
		/// the search line. This must be an odd number.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			
		int width
		);


	/// \brief Retrieve the number of found edges after searching.
	
	/// The results of this method are influenced by the flag that can
	/// be set with mvIMPACT::measure::MeasureEdgesBase::setBestMatchOnly
	/// See \ref mvi_measure_subsection_edges_results to for more information about
	/// getting results from mvIMPACT Measure.
	/// \see mvIMPACT::measure::MeasureEdgesBase::setBestMatchOnly
	/// \see mvIMPACT::measure::MeasureEdges::getFeature
	int count 
	(
		/// Specifies the band offset. If the image, where the edges have been found,
		/// consists of multiple bands (i.e. a color image consists of 3 bands (RGB)),
		/// this parameter allows you to select the desired band in a range 
		/// [0..number of bands-1]. If the source image consists of only one band 
		/// (monochrome image), set this parameter to 0.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.bands-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int band=0, 
		/// Specifies the slice offset. If the image, where the edges have been found, 
		/// consists of multiple slices (i.e. stack of images, that represent a data volume),
		/// this parameter allows you to select the desired slice in a range 
		/// [0..number of slices-1]. If the source image consists of only one slice,
		/// set this parameter to 0.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.slices-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int slice=0, 
		/// Specifies the time offset. If the image, where the edges have been found, 
		/// consists of multiple frames (i.e. an image sequence), this parameter allows 
		/// you to select the desired frame in a range [0..number of frames-1]. 
		/// If the source image consists of only one frame, set this parameter to 0.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.frames-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int frame=0
		) const;


	/// \brief Retrieve the number of bands.
	
	/// This method returns the number of bands a result buffer consists of.
	/// Internally this number is obtained from the source image.
	int getBands() const;

	/// \brief Retrieve the number of slices.
	
	/// This method returns the number of slices a result buffer consists of.
	/// Internally this number is obtained from the source image.
	int getSlices() const;

	/// \brief Retrieve the number of frames.
	
	/// This method returns the number of frames a result buffer consists of.
	/// Internally this number is obtained from the source image.
	int getFrames() const;


	/// \brief Retrieve a single result. 

	/// All features specified with mvIMPACT::measure::MeasureEdges::addFeature previously can 
	/// be queried now.
	/// See \ref mvi_measure_subsection_edges_results for more information about getting results
	/// from mvIMPACT Measure. The number of results that can be retrieved with this method is 
	/// influenced by mvIMPACT::measure::MeasureEdgesBase::setBestMatchOnly. You can get the number
	/// of results with mvIMPACT::measure::MeasureEdges::count.
	///
	/// If the \a edgeFeature parameter is set to \c efXPoint or \c efYPoint, the number of points obtained
	/// must be queried by calling the function with \a index set to \a -1. You can only read
	/// the coordinates of one point at a time. The \a index parameter specifies the point to be read.
	/// In this case tThe \a index parameter range is from [0..number of points - 1].
	/// \see mvIMPACT::measure::MeasureEdgesBase::setBestMatchOnly
	/// \see mvIMPACT::measure::MeasureEdges::count
	double getFeature
	(
		/// The feature to query.
		EdgeFeature edgeFeature, 
		/// The index of the edge to query. Valid indexes go from 0 to the result 
		/// of count(int,int,int) - 1. If \a edgeFeature is set to \c efXPoint or \c efYPoint
		/// the index can be -1 to query the number of points \c N or [0..N-1] to get the coordinates
		/// of such a point.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="(%(instance)s.count(%(band)s,%(slice)s,%(frame)s)-1,%(instance)s.getFeature(%(edgeFeature)s,-1,%(band)s,%(slice)s,%(frame)s))[%(edgeFeature)s == efXPoint or %(edgeFeature)s == efYPoint]"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int index=0, 
		/// If the source images consists of more than one band specify the band offset here.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.bands-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int band =0, 
		/// If the source images consists of more than one slice specify the slice offset here.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.slices-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int slice=0, 
		/// If the source images consists of more than one frame specify the frame offset here.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.frames-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int frame=0
		) const;
	
public: // Properties

	/// \brief Specifies the expected polarity of an edge. The polarity contributes to the
	/// \brief match score.

	/// The available polarity modes are reflected in enumeration #EdgePolarityMode.
	/// \see \ref mvi_measure_section_terms
	MeasureEdges& setPolarity( EdgePolarityMode polarity );
	

	/// \brief Retrieve the current expected polarity.

	/// The available polarity modes are reflected in enumeration #EdgePolarityMode.
	/// \see \ref mvi_measure_section_terms
	EdgePolarityMode getPolarity() const;

};



/// \brief This class measures edge pairs. An example for an edge pair is a bright stripe
/// \brief on a dark background. The class is derived from MeasureEdgesBase and contains all necessary
/// \brief methods to perform the mesurement.

/// The following image shows an example of a plastic piece with a bright 'stripe' 
/// in the middle. A search line in yellow is drawn over it. The edges that were found 
/// are displayed in blue:
///
/// \image html images/mvi_measure/mvi_measure_EdgePair.png
///
/// Example how to compute the distance between both edges in the above image:
///
/// \xmlonly
/// <mvWrapperDoc><htmlinclude ref="code_fragment_start_cpp.html"/></mvWrapperDoc>
/// \endxmlonly
/// \htmlinclude code_fragment_start_cpp.html
/// \code
/// Image img = mvIMPACT.loadImage("pics/plasticpiece.bmp");
///
/// MeasureEdgePair mep = MeasureEdgePair();
/// mep.addFeature(epfDistance);
/// mep.find(img,80,100,200,92,11);
/// 
/// int count = mep.count();
/// for (int i; i<count; ++i)
/// {
///     double distance = mep.getFeature(epfDistance,i);
/// }
/// \endcode
/// \htmlinclude code_fragment_end.html
/// \xmlonly
/// <mvWrapperDoc><htmlinclude ref="code_fragment_end.html"/></mvWrapperDoc>
/// \endxmlonly
///
/// \xmlonly		     
/// <mvWrapperDoc>
///
/// <htmlinclude ref="code_fragment_start_cs.html"/>
/// \code
/// Image img = gBase.loadImage("pics/plasticpiece.bmp");
///
/// MeasureEdgePair mep = new MeasureEdgePair();
/// mep.addFeature(EdgePairFeature.epfDistance);
/// mep.find(img,80,100,200,92,11);
/// 
/// int count = mep.count();
/// for (int i; i&#x3c;count; ++i)
/// {
///     double distance = mep.getFeature(EdgePairFeature.epfDistance,i);
/// }
/// \endcode
/// <htmlinclude ref="code_fragment_end.html"/>
/// </mvWrapperDoc>
/// \endxmlonly
///
/// \xmlonly		     
/// <mvWrapperDoc>
///
/// <htmlinclude ref="code_fragment_start_vb_net.html"/>
/// \code
/// Dim img As Image = gBase.loadImage("pics/plasticpiece.bmp")
///
/// Dim mep As MeasureEdgePair = new MeasureEdgePair()
/// mep.addFeature(EdgePairFeature.epfDistance)
/// mep.find(img,80,100,200,92,11)
/// 
/// Dim count As Integer = mep.count()
/// For i As Integer = 0 To count-1
///     Dim distance As Double = mep.getFeature(EdgePairFeature.epfDistance,i)
/// Next i
/// \endcode
/// <htmlinclude ref="code_fragment_end.html"/>
/// </mvWrapperDoc>
/// \endxmlonly
///
/// \xmlonly		     
/// <mvWrapperDoc>
///
/// <htmlinclude ref="code_fragment_start_python.html"/>
/// \code
/// img = mvIMPACT.loadImage("pics/plasticpiece.bmp")
///
/// mep = measure.MeasureEdgePair()
/// mep.addFeature(measure.epfDistance);
/// mep.find(img,80,100,200,92,11);
/// 
/// count = mep.count()
/// for i in range(count):
///     distance = mep.getFeature(measure.epfDistance,i);
/// 
/// \endcode
/// <htmlinclude ref="code_fragment_end.html"/>
/// </mvWrapperDoc>
/// \endxmlonly
///
/// \see \ref mvi_measure_section_edges

//-----------------------------------------------------------------------------
class MeasureEdgePairs : public MeasureEdgesBase
//-----------------------------------------------------------------------------
{
public:
	///\brief Allocates a MeasureEdgePair object.
	MeasureEdgePairs( PairMode mode = pmPosNeg );

	///\brief Creates a MeasureEdgePair object from \a src.
	MeasureEdgePairs(const MeasureEdgePairs& src);

#ifndef WRAP_PYTHON
	///\brief Overwrite current object with \a src.
	MeasureEdgePairs& operator= (const MeasureEdgePairs& src);
#endif // #ifndef WRAP_PYTHON
	

	/// \brief Add a feature to search for. 

	/// After the mvIMPACT::measure::MeasureEdgePairs::find 
	/// method is called all features added with this function can be queried.
	/// For a list of available features 
	/// see mvIMPACT::measure::EdgePairFeature.
	/// \see \ref mvi_measure_subsection_edges_features
	void addFeature (EdgePairFeature edgePairFeature);
	

	/// \brief Search for edge pairs.

	/// This function locates edge pairs in a given search window.
	/// The search window is defined by a search line and a width:
	/// After the search is completed the results can be queried with mvIMPACT::measure::MeasureEdgePairs::count and 
	/// mvIMPACT::measure::MeasureEdgePairs::getFeature .
	/// \see \ref mvi_measure_subsection_measure_edges_windows
	void find 
	(
		/// The source image.
		const Image& source, 
		/// The x-coordinate of the start point.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(source)s.width-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int startX, 
		/// The y-coordinate of the start point.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(source)s.height-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int startY, 
		/// The x-coordinate of the end point.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(source)s.width-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int endX, 
		/// The y-coordinate of the end point.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(source)s.height-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int endY, 
		/// The width of the window given as the perpendicular distance from 
		/// the search line. This must be an odd number.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			
		int width
		);
	
	/// \brief Search for edge pairs.
	///
	/// This function locates edge pairs in a given search window.
	/// The search window is defined by a search line and a width:
	/// After the search is completed the results can be queried with mvIMPACT::measure::MeasureEdgePairs::count and 
	/// mvIMPACT::measure::MeasureEdgePairs::getFeature .
	/// \see \ref mvi_measure_subsection_measure_edges_windows
	void find 
	(
		/// The source image.
		const Image& source, 
		/// The search line
		Line line,
		/// The width of the window given as the perpendicular distance from 
		/// the search line. This must be an odd number.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			
		int width
		);


	/// \brief Retrieve the number of found edge pairs after searching.
	///
	/// The result of this method is influenced by the state of the flag, that
	/// can be set with mvIMPACT::measure::MeasureEdgesBase::setBestMatchOnly.
	/// The actual results can be retrieved with mvIMPACT::measure::MeasureEdgePairs::getFeature .
	/// \see \ref mvi_measure_subsection_edges_results
	int count
	(
		/// Specifies the band offset. If the image, where the edges have been found,
		/// consists of multiple bands (i.e. a color image consists of 3 bands (RGB)),
		/// this parameter allows you to select the desired band in a range 
		/// [0..number of bands-1]. If the source image consists of only one band 
		/// (monochrome image), set this parameter to 0.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.bands-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int band=0, 
		/// Specifies the slice offset. If the image, where the edges have been found, 
		/// consists of multiple slices (i.e. stack of images, that represent a data volume),
		/// this parameter allows you to select the desired slice in a range 
		/// [0..number of slices-1]. If the source image consists of only one slice,
		/// set this parameter to 0.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.slices-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int slice=0, 
		/// Specifies the time offset. If the image, where the edges have been found, 
		/// consists of multiple frames (i.e. an image sequence), this parameter allows 
		/// you to select the desired frame in a range [0..number of frames-1]. 
		// If the source image consists of only one frame, set this parameter to 0.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.frames-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int frame=0
		) const;
	
	/// This method returns the number of bands a result buffer consists of.
	/// Internally this number is obtained from the source image.
	int getBands() const;

	/// \brief Retrieve the number of slices.
	
	/// This method returns the number of slices a result buffer consists of.
	/// Internally this number is obtained from the source image.
	int getSlices() const;

	/// \brief Retrieve the number of frames.
	
	/// This method returns the number of frames a result buffer consists of.
	/// Internally this number is obtained from the source image.
	int getFrames() const;


	/// \brief Retrieve a single result. 
	///
	///All features specified with mvIMPACT::measure::MeasureEdgePairs::addFeature
	/// previously can be queried now. Available features are listed in enumeration #EdgePairFeature.
	/// The number of results that can be retrieved can be determined by calling mvIMPACT::measure::MeasureEdgePairs::count.
	/// The number of results is influenced by the flag, that can be set with mvIMPACT::measure::MeasureEdgesBase::setBestMatchOnly
	/// \see \ref mvi_measure_subsection_edges_results
	double getFeature 
	(
		/// The feature to query.
		EdgePairFeature edgePairFeature, 
		/// The index of the edge to query. Valid indexes go from 0 to the result 
		/// of count(int,int,int) - 1.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.count(%(band)s,%(slice)s,%(frame)s)-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int index=0, 
		/// If the source images consists of more than one band specify the band offset here.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.bands-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int band =0, 
		/// If the source images consists of more than one slice specify the slice offset here.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.slices-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int slice=0, 
		/// If the source images consists of more than one frame specify the frame offset here.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.frames-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int frame=0
		) const;
	

public: // Properties

	/// \brief Set the polarity mode of the edge pair to search for. 

	/// You can search 
	/// for a bright stripe on a dark background or vice versa.
	/// \see \ref mvi_measure_section_terms
	/// For more information on edge polarities see \ref mvi_measure_subsubsection_edges_classes_polarity.
	/// \see #PairMode
	MeasureEdgePairs& setPairMode( PairMode pairMode );
	

	/// \brief Retrieve the current polarity mode used for searching.

	/// For more information on edge polarities see \ref mvi_measure_subsubsection_edges_classes_polarity.
	/// \see \ref mvi_measure_section_terms
	/// \see #PairMode
	PairMode getPairMode() const;
	

	/// \brief Specifies the minimum match score of the second edge, that must
	/// \brief be reached, in order to be recognized by the analysis module.

	/// The score is in the range [0..100].
	/// \see mvIMPACT::measure::MeasureEdgePairs::getMinScore2
	MeasureEdgePairs& setMinScore2 (
		///minimal score value
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="100.0"/></mvParamMax>
		/// <mvParamStep>0.1</mvParamStep>
		/// </mvParamRange>
		/// \endxmlonly			
		double minScore2);
	

	/// \brief Retrieve the current minimum match score of the second edge.
	
	///
	/// \see mvIMPACT::measure::MeasureEdgePairs::getMinScore2
	double getMinScore2 () const;
	

	/// \brief Specifies a weight factor for the contrast match score of the second
	/// edge. 

	/// The total match score is calculated from the sum of all single
	/// match scores. Please note, that all weight factors must sum up to 1.
	/// The weight is in the range [0..1].
	/// For more information on edge contrast see \ref mvi_measure_subsubsection_edges_classes_contrast.
	/// \see \ref mvi_measure_section_terms 
	/// \see mvIMPACT::measure::MeasureEdgesBase::setContrastWeight
	/// \see mvIMPACT::measure::MeasureEdgesBase::getContrastWeight
	/// \see mvIMPACT::measure::MeasureEdgePairs::getContrastWeight2
	MeasureEdgePairs& setContrastWeight2 (
		/// weight factor
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="1.0"/></mvParamMax>
		/// <mvParamStep>0.1</mvParamStep>
		/// </mvParamRange>
		/// \endxmlonly			
		double contrastWeight2);
	

	/// \brief Retrieve the weight for the contrast match score of the second edge.
	/// For more information on edge contrast see \ref mvi_measure_subsubsection_edges_classes_contrast.

	/// \see \ref mvi_measure_section_terms
	/// \see mvIMPACT::measure::MeasureEdgesBase::setContrastWeight
	/// \see mvIMPACT::measure::MeasureEdgesBase::getContrastWeight
	/// \see mvIMPACT::measure::MeasureEdgePairs::setContrastWeight2
	double getContrastWeight2 () const;
	

	/// \brief Specifies a weight factor for the steepness match score of the second
	/// edge. 

	/// The total match score is calculated from the sum of all single
	/// match scores. Please note, that all weight factors must sum up to 1.
	/// The weight is in the range [0..1].
	/// For more information on edge steepness see \ref mvi_measure_subsubsection_edges_classes_steepness.
	/// \see \ref mvi_measure_section_terms 
	/// \see mvIMPACT::measure::MeasureEdgesBase::setSteepnessWeight
	/// \see mvIMPACT::measure::MeasureEdgesBase::getSteepnessWeight
	/// \see mvIMPACT::measure::MeasureEdgePairs::getSteepnessWeight2
	MeasureEdgePairs& setSteepnessWeight2 (
		///weight factor
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="1.0"/></mvParamMax>
		/// <mvParamStep>0.1</mvParamStep>
		/// </mvParamRange>
		/// \endxmlonly			
		double steepnessWeight2);
	

	/// \brief Retrieve the weight for the steepness match score of the second edge.

	/// For more information on edge steepness see \ref mvi_measure_subsubsection_edges_classes_steepness.
	/// \see \ref mvi_measure_section_terms
	/// \see mvIMPACT::measure::MeasureEdgesBase::setSteepnessWeight
	/// \see mvIMPACT::measure::MeasureEdgesBase::getSteepnessWeight
	/// \see mvIMPACT::measure::MeasureEdgePairs::setSteepnessWeight2
	double getSteepnessWeight2 () const;
	

	/// \brief Specifies a weight factor for the polarity match score of the second
	/// edge. 

	/// The total match score is calculated from the sum of all single
	/// match scores. Please note, that all weight factors must sum up to 1.
	/// The weight is in the range [0..1].
	/// For more information on edge polarity see \ref mvi_measure_subsubsection_edges_classes_polarity.
	/// \see \ref mvi_measure_section_terms
	/// \see mvIMPACT::measure::MeasureEdgesBase::setPolarityWeight
	/// \see mvIMPACT::measure::MeasureEdgesBase::getPolarityWeight
	/// \see mvIMPACT::measure::MeasureEdgePairs::getPolarityWeight2
	MeasureEdgePairs& setPolarityWeight2 (
		///weight factor
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="1.0"/></mvParamMax>
		/// <mvParamStep>0.1</mvParamStep>
		/// </mvParamRange>
		/// \endxmlonly			
		double polarityWeight2);
	

	/// \brief Retrieve the weight for the polarity match score of the second edge.


	/// For more information on edge polarity see \ref mvi_measure_subsubsection_edges_classes_polarity.
	/// \see \ref mvi_measure_section_terms
	/// \see mvIMPACT::measure::MeasureEdgesBase::setPolarityWeight
	/// \see mvIMPACT::measure::MeasureEdgesBase::getPolarityWeight
	/// \see mvIMPACT::measure::MeasureEdgePairs::setPolarityWeight2
	double getPolarityWeight2 () const;
	

	/// \brief Set the typical (expected) contrast of the second edge. The contrast contributes
	/// to the match score.

	/// For more information on edge contrast see \ref mvi_measure_subsubsection_edges_classes_contrast.
	/// \see \ref mvi_measure_section_terms 
	/// \see mvIMPACT::measure::MeasureEdgesBase::setTypicalContrast
	/// \see mvIMPACT::measure::MeasureEdgesBase::getTypicalContrast
	/// \see mvIMPACT::measure::MeasureEdgePairs::getTypicalContrast2
	MeasureEdgePairs& setTypicalContrast2 (
		///contrast value
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
		/// <mvParamStep>0.1</mvParamStep>
		/// </mvParamRange>
		/// \endxmlonly			
		double typicalContrast2 );
	

	/// \brief Retrieve the typical contrast of the second edge.


	/// For more information on edge contrast see \ref mvi_measure_subsubsection_edges_classes_contrast.
	/// \see \ref mvi_measure_section_terms
	/// \see mvIMPACT::measure::MeasureEdgesBase::setTypicalContrast
	/// \see mvIMPACT::measure::MeasureEdgesBase::getTypicalContrast
	/// \see mvIMPACT::measure::MeasureEdgePairs::setTypicalContrast2
	double getTypicalContrast2 () const;
	

	/// \brief Set the typical (expected) steepness of the second edge. The contrast contributes
	/// to the match score.

	///For more information on edge steepness see \ref mvi_measure_subsubsection_edges_classes_steepness.
	/// \see \ref mvi_measure_section_terms
	/// \see mvIMPACT::measure::MeasureEdgesBase::setTypicalSteepness
	/// \see mvIMPACT::measure::MeasureEdgesBase::getTypicalSteepness
	/// \see mvIMPACT::measure::MeasureEdgePairs::getTypicalContrast2
	MeasureEdgePairs& setTypicalSteepness2 (
		///steepness value
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0.1"/></mvParamMin>
		/// <mvParamStep>0.1</mvParamStep>
		/// </mvParamRange>
		/// \endxmlonly			
		double typicalSteepness2);
	

	/// \brief Retrieve the typical steepness of the second edge.

	///For more information on edge steepness see \ref mvi_measure_subsubsection_edges_classes_steepness.
	/// \see \ref mvi_measure_section_terms
	/// \see mvIMPACT::measure::MeasureEdgesBase::setTypicalSteepness
	/// \see mvIMPACT::measure::MeasureEdgesBase::getTypicalSteepness
	/// \see mvIMPACT::measure::MeasureEdgePairs::getTypicalContrast2
	double getTypicalSteepness2 () const;
	
};




/// \brief Base class for circle measurement. This class contains methods to
/// \brief set parameters for the search algorithm, that apply to both,
/// \brief mvIMPACT::measure::MeasureCircles and mvIMPACT::measure::MeasureCirclePair.
/// \brief This class is never instantiated directly.
/// \brief Always use one of the derived classes.
//-----------------------------------------------------------------------------
class MeasureCircleBase : public MeasureBase
//-----------------------------------------------------------------------------
{
protected:

        ///\brief Creates a MeasureCircleBase object.
	MeasureCircleBase();

	///\brief Creates a MeasureCircleBase object from \a src.
	MeasureCircleBase( const MeasureCircleBase& src );

#ifndef WRAP_PYTHON
	///\brief Overwrite current object with \a src.
	MeasureCircleBase& operator= ( const MeasureCircleBase& src );
#endif // #ifndef WRAP_PYTHON


public: // Properties

	/// \brief Specifies the number of passes to remove 'false' edge pixels.

	/// After the first pass, it's possible, that 'false' edge pixels falsify the
	/// results. In each iteration, pixels with a distance greater than a given 
	/// value (specified with MeasureBase::setDistanceTolerance) are removed.
	/// Value specifies the value to be set. A value of 0 disables this feature.<br>
	/// Example : 2
	/// For more information on false pixel filtering see \ref mvi_measure_subsubsection_circles_classes_filtering.
	/// \see mvIMPACT::measure::MeasureCircleBase::getRemovalIterations
	MeasureCircleBase& setRemovalIterations( int removalIterations );
	

	/// \brief Retrieve the number of passes to remove 'false' edge pixels.

	/// For more information on false pixel filtering see \ref mvi_measure_subsubsection_circles_classes_filtering.
	/// \see MeasureCircleBase::setRemovalIterations
	int getRemovalIterations() const;
	

	/// \brief Specifies the distance when to reject 'false' edge pixels. 

	/// The distance is 
	/// calculated as the deviation from the radius of the previous iteration in 
	/// percent. It is in the range [0..100].<br>
	/// Example: If this value is 5.0 and the previous iteration yielded a radius 
	/// of 100.0 all pixels outside [95..105] are considered as false pixels and removed.
	/// For more information on false pixel filtering see \ref mvi_measure_subsubsection_circles_classes_filtering.
	/// \see mvIMPACT::measure::MeasureCircleBase::getDistanceTolerance
	MeasureCircleBase& setDistanceTolerance( 
		/// new tolerance value
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="100.0"/></mvParamMax>
		/// <mvParamStep>0.1</mvParamStep>
		/// </mvParamRange>
		/// \endxmlonly			
		double distanceTolerance);
	

	/// \brief Retrieve the current distance tolerance in percent.

	/// For more information on false pixel filtering see \ref mvi_measure_subsubsection_circles_classes_filtering.
	/// \see MeasureCircleBase::setDistanceTolerance
	double getDistanceTolerance() const;
	
#ifndef DOXYGEN_SHOULD_SKIP_THIS //these methods should not appear in the documentation anymore
	/// \brief Do not use, just for compatibility.
	MeasureCircleBase& setMaxIterations( 
		///maximal number of iterations
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			
		int maxIterations);
	

	/// \brief Do not use, just for compatibility.
	int getMaxIterations() const;
	

	/// \brief Do not use, just for compatibility.
	MeasureCircleBase& setTolerance(
		///new tolerance value
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0.00001"/></mvParamMin>
		/// <mvParamStep>0.1</mvParamStep>
		/// </mvParamRange>
		/// \endxmlonly			
		double tolerance );
	

	/// \brief Do not use, just for compatibility.
	double getTolerance() const;
#endif	

	/// \brief Specifies the gradient threshold when searching for edge pixel candidates. 

	/// The gradient threshold is the absolute value of the difference between
	/// two pixel values in the transition of the edge.<br>
	/// Example: 20
	/// For more information see \ref mvi_measure_subsubsection_circles_classes_gradient.
	/// \see mvIMPACT::measure::MeasureCircleBase::getGradientThreshold
	MeasureCircleBase& setGradientThreshold( 
		///new threshold value
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			
		int gradientThreshold);
	

	/// \brief Retrieve the current gradient threshold.

	/// For more information see \ref mvi_measure_subsubsection_circles_classes_gradient.
	/// \see mvIMPACT::measure::MeasureCircleBase::setGradientThreshold
	int getGradientThreshold() const;
	

	/// \brief Specifies the measurement direction. 

	/// The measurement direction sets the 
	/// internal direction of the measure module how to search for edge pixels of 
	/// the circular object. The measurement direction should be set with respect 
	/// to the measurement window's contents.
	/// For a detailed explanation on measurement directions see \ref mvi_measure_circles_measurement_direction.
	/// \see #CircleMeasurementDirection
	/// \see mvIMPACT::measure::MeasureCircleBase::getMeasurementDirection
	MeasureCircleBase& setMeasurementDirection( CircleMeasurementDirection measurementDirection);
	

	/// \brief Retrieve the current measurement direction.
	/// For a detailed explanation on measurement directions see \ref mvi_measure_circles_measurement_direction.
	/// \see #CircleMeasurementDirection
	/// \see mvIMPACT::measure::MeasureCircleBase::setMeasurementDirection
	CircleMeasurementDirection getMeasurementDirection() const;
	
};



/// \brief This class measures single circles. It is derived from mvIMPACT::measure::MeasureCircleBase
/// and contains all necessary methods to perform the measurement on single circles.

/// Specify features to search for
/// with one or more calls to mvIMPACT::measure::MeasureCircles::addFeature. 
/// Then call mvIMPACT::measure::MeasureCircles::find and query the 
/// results with mvIMPACT::measure::MeasureCirlces::count and mvIMPACT::measure::MeasureCircles::getFeature .
///
/// An example demonstrates this: <br>
///
/// \image html images/mvi_measure/mvi_measure_circle.png
/// The image above shows a white circle on a dark background. To measure its 
/// radius you can use the following code:
///
/// \xmlonly
/// <mvWrapperDoc><htmlinclude ref="code_fragment_start_cpp.html"/></mvWrapperDoc>
/// \endxmlonly
/// \htmlinclude code_fragment_start_cpp.html
/// \code
/// Image img = loadImage("circle.bmp");
/// 
/// MeasureCircles mc;
/// mc.addFeature (cfRadius);
/// mc.find       (img,100,20,60,210);
/// 
/// int count = mc.count();
/// for (int i=0; i<count; ++i)
/// {
///    double radius = mc.getFeature(cfRadius,i);
/// }
/// \endcode
/// \htmlinclude code_fragment_end.html
/// \xmlonly
/// <mvWrapperDoc><htmlinclude ref="code_fragment_end.html"/></mvWrapperDoc>
/// \endxmlonly
///
/// \xmlonly
/// <mvWrapperDoc>
/// <htmlinclude ref="code_fragment_start_cs.html"/>
/// \code
/// Image img = gBase.loadImage("circle.bmp");
/// 
/// MeasureCircles mc = new MeasureCircles();
/// mc.addFeature (CircleFeature.cfRadius);
/// mc.find(img,100,20,60,210);
/// 
/// int count = mc.count();
/// for (int i=0; i&#x3c;count; ++i)
/// {
///    double radius = mc.getFeature(CircleFeature.cfRadius,i);
/// }
/// \endcode
/// <htmlinclude ref="code_fragment_end.html"/>
/// </mvWrapperDoc>
/// \endxmlonly
///
/// \xmlonly
/// <mvWrapperDoc>
/// <htmlinclude ref="code_fragment_start_vb_net.html"/>
/// \code
/// Dim img As Image = gBase.loadImage("circle.bmp")
/// 
/// Dim mc As MeasureCircles = new MeasureCircles()
/// mc.addFeature (CircleFeature.cfRadius)
/// mc.find(img,100,20,60,210)
/// 
/// Dim count As Integer = mc.count()
/// For i As Integer = 0 To count-1
///    Dim radius As Double = mc.getFeature(CircleFeature.cfRadius,i)
/// Next i
/// \endcode
/// <htmlinclude ref="code_fragment_end.html"/>
/// </mvWrapperDoc>
/// \endxmlonly
///
/// \xmlonly
/// <mvWrapperDoc>
/// <htmlinclude ref="code_fragment_start_python.html"/>
/// \code
/// img = mvIMPACT.loadImage("circle.bmp")
/// 
/// mc = measure.MeasureCircles
/// mc.addFeature (measure.cfRadius)
/// mc.find(img,100,20,60,210)
/// 
/// count = mc.count()
/// for i in range(count):
///    radius = mc.getFeature(measure.cfRadius,i);
/// 
/// \endcode
/// <htmlinclude ref="code_fragment_end.html"/>
/// </mvWrapperDoc>
/// \endxmlonly
//-----------------------------------------------------------------------------
class MeasureCircles : public MeasureCircleBase
//-----------------------------------------------------------------------------
{
public:
	///\brief Allocates a MeasureCircles object.
	MeasureCircles();

	///\brief Creates a MeasureCircles object from \a src.
	MeasureCircles(const MeasureCircles& src);

#ifndef WRAP_PYTHON
	///\brief Overwrite current object with \a src.
	MeasureCircles& operator= (const MeasureCircles& src);
#endif // #ifndef WRAP_PYTHON
	

	/// \brief Add a feature to search for. 

	/// After the mvIMPACT::measure::MeasureCircles::find 
	/// method is called all features added with this function can be queried.
	/// For a list of available features see #CircleFeature.
	/// for more information on circle features see \ref mvi_measure_subsection_circles_features.
	void addFeature (CircleFeature circleFeature);
	

	/// \brief Search for circles.

	/// This function locates circles in a given search window.
	/// The search window is defined by a point and a corresponding width and 
	/// height.
	/// After the search is completed the results can be queried with mvIMPACT::measure::MeasureCircles::count and 
	/// mvIMPACT::measure::MeasureCircles::getFeature .
	/// For more information on measurement windows see \ref mvi_measure_subsection_circles_windows.
	void find 
	(
		/// The source image.
		const Image& source, 
		/// The x-coordinate of the start point.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(source)s.width-%(width)s"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int x, 
		/// The y-coordinate of the start point.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(source)s.height-%(height)s"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int y, 
		/// The width of the search window.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(source)s.width-%(x)s"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int width, 
		/// The height of the search window.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(source)s.height-%(y)s"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int height
		);
	

	/// This function locates circles in a given search window.
	/// The search window is defined by a point and a corresponding width and 
	/// height.
	/// After the search is completed the results can be queried with mvIMPACT::measure::MeasureCircles::count and 
	/// mvIMPACT::measure::MeasureCircles::getFeature .
	/// For more information on measurement windows see \ref mvi_measure_subsection_circles_windows.
	void find 
	(
		/// The source image.
		const Image& source, 
		/// The start point.
		Point & point,
		/// The width of the search window.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(source)s.width-%(point)s.x"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int width, 
		/// The height of the search window.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(source)s.height-%(point)s.y"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int height
		);

	/// This function locates circles in a given search window.
	/// The search window is defined by a rectangle.
	/// After the search is completed the results can be queried with mvIMPACT::measure::MeasureCircles::count and 
	/// mvIMPACT::measure::MeasureCircles::getFeature .
	/// For more information on measurement windows see \ref mvi_measure_subsection_circles_windows.
	void find 
	(
		/// The source image.
		const Image& source, 
		/// The search window.
		Rectangle & rectangle
		);

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY
	/// This function locates circles in multiple search windows.
	/// ....
	/// After the search is completed the results can be queried with mvIMPACT::measure::MeasureCircles::count and 
	/// mvIMPACT::measure::MeasureCircles::getFeature .
	/// For more information on measurement windows see \ref mvi_measure_subsection_circles_windows.
	void find 
	(
		/// The source image.
		const Image& source, 
		/// ...
		const std::vector<MeasureCircles>& PYTHON_CPP_SWITCH(SEQUENCE_TO_VECTOR,windows),
		/// ...
		const std::vector<Rectangle>& PYTHON_CPP_SWITCH(SEQUENCE_TO_VECTOR,rectangles)
		);
#endif // #ifndef WRAP_ANY
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

	/// \brief Retrieve the number of found circles after searching.
	
	/// The result of this method can be used as an index for mvIMPACT::measure::MeasureCircles::getFeature .
	/// For more information on getting the results from mvIMPACT Measure see \ref mvi_measure_subsection_circles_results.
	/// \see mvIMPACT::measure::MeasureCircles::getFeature 
	int count 
	(
		/// Specifies the band offset. If the image, where the edges have been found,
		/// consists of multiple bands (i.e. a color image consists of 3 bands (RGB)),
		/// this parameter allows you to select the desired band in a range 
		/// [0..number of bands-1]. If the source image consists of only one band 
		/// (monochrome image), set this parameter to 0.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.bands-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int band=0, 
		/// Specifies the slice offset. If the image, where the edges have been found, 
		/// consists of multiple slices (i.e. stack of images, that represent a data volume),
		/// this parameter allows you to select the desired slice in a range 
		/// [0..number of slices-1]. If the source image consists of only one slice,
		/// set this parameter to 0.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.slices-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int slice=0, 
		/// Specifies the time offset. If the image, where the edges have been found, 
		/// consists of multiple frames (i.e. an image sequence), this parameter allows 
		/// you to select the desired frame in a range [0..number of frames-1]. 
		// If the source image consists of only one frame, set this parameter to 0.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.frames-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int frame=0
		) const;
	
	/// This method returns the number of bands a result buffer consists of.
	/// Internally this number is obtained from the source image.
	int getBands() const;

	/// \brief Retrieve the number of slices.
	
	/// This method returns the number of slices a result buffer consists of.
	/// Internally this number is obtained from the source image.
	int getSlices() const;

	/// \brief Retrieve the number of frames.
	
	/// This method returns the number of frames a result buffer consists of.
	/// Internally this number is obtained from the source image.
	int getFrames() const;

	/// \brief Retrieve a single result. 

	/// All features specified with mvIMPACT::measure::MeasureCircles::addFeature previously can 
	/// be queried now. Use mvIMPACT::measure::MeasureCircles::count to get the number of results.
	/// For a list of valid features see enumeration #CircleFeature.
	/// For more information on getting the results from mvIMPACT Measure see \ref mvi_measure_subsection_circles_results.
	double getFeature
	(
		/// The feature to query.
		CircleFeature circleFeature, 
		/// The index of the circle to query. Valid indexes go from 0 to the result 
		/// of count(int,int,int) - 1.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.count(%(band)s,%(slice)s,%(frame)s)-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int index=0, 
		/// If the source images consists of more than one band specify the band offset here.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.bands-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int band=0, 
		/// If the source images consists of more than one slice specify the slice offset here.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.slices-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int slice=0, 
		/// If the source images consists of more than one frame specify the frame offset here.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.frames-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int frame=0
		) const;
	

public: // Properties

	/// \brief Specifies the expected polarity of a circle. 

	/// When specifying a polarity,
	/// it helps the internal algorithm to distinguish between 'true' and 'false'
	/// edge pixels. The chance to detect 'false' edge pixels decreases.
	/// For more information on circle polarity see \ref mvi_measure_subsubsection_circles_classes_polarity.
	/// \see \ref mvi_measure_section_terms
	/// \see mvIMPACT::measure::EdgePolarityMode
	MeasureCircles& setPolarity( EdgePolarityMode polarity );
	

	/// \brief Retrieve the current expected polarity.

	/// For more information on circle polarity see \ref mvi_measure_subsubsection_circles_classes_polarity.
	/// \see \ref mvi_measure_section_terms
	/// \see mvIMPACT::measure::EdgePolarityMode
	EdgePolarityMode getPolarity() const;
	
};




/// \brief This class measures a circle pair. It is derived from mvIMPACT::measure::MeasureCircleBase
/// \brief and contains all necessary functionality to perform the measurment.

/// Specify features to search for
/// with one or more calls to mvIMPACT::measure::MeasureCirclePair::addFeature. Then call 
/// mvIMPACT::measure::MeasureCirclePair::find and query the results with 
/// mvIMPACT::measure::MeasureCirclePair::getFeature . 
/// Please note that only one circle pair can be measured!
///
/// An example demonstrates this: <br>
/// This image shows a dark torus on a white background.
/// \image html images/mvi_measure/mvi_measure_torus.png
/// You can for example compute the width of this torus that is given as the 
/// difference of the radius of the outer and the inner circle like this:
///
/// \xmlonly
/// <mvWrapperDoc><htmlinclude ref="code_fragment_start_cpp.html"/></mvWrapperDoc>
/// \endxmlonly
/// \htmlinclude code_fragment_start_cpp.html
/// \code
/// Image img = loadImage("torus.bmp");
///
/// MeasureCirclePair mc;
/// mc.addFeature (cfRadius);
/// mc.find       (img,100,20,60,210);
/// 
/// double width = mc.getFeature(cfRadius,0) - mc.getFeature(cfRadius,1);
/// \endcode
/// \htmlinclude code_fragment_end.html
/// \xmlonly
/// <mvWrapperDoc><htmlinclude ref="code_fragment_end.html"/></mvWrapperDoc>
/// \endxmlonly
///
/// \xmlonly
/// <mvWrapperDoc>
///
/// <htmlinclude ref="code_fragment_start_cs.html"/>
/// \code
/// Image img = gBase.loadImage("torus.bmp");
///
/// MeasureCirclePair mc = new MeasureCriclePair();
/// mc.addFeature (CircleFeature.cfRadius);
/// mc.find(img,100,20,60,210);
/// 
/// double width = mc.getFeature(cfRadius,0) - mc.getFeature(cfRadius,1);
/// \endcode
/// <htmlinclude ref="code_fragment_end.html"/>
/// </mvWrapperDoc>
/// \endxmlonly
///
/// \xmlonly
/// <mvWrapperDoc>
///
/// <htmlinclude ref="code_fragment_start_vb_net.html"/>
/// \code
/// Dim img As Image = gBase.loadImage("torus.bmp")
///
/// Dim mc As MeasureCirclePair = new MeasureCriclePair()
/// mc.addFeature (CircleFeature.cfRadius)
/// mc.find(img,100,20,60,210)
/// 
/// Dim width As Double = mc.getFeature(cfRadius,0) - mc.getFeature(cfRadius,1)
/// \endcode
/// <htmlinclude ref="code_fragment_end.html"/>
/// </mvWrapperDoc>
/// \endxmlonly
///
/// \xmlonly
/// <mvWrapperDoc>
///
/// <htmlinclude ref="code_fragment_start_python.html"/>
/// \code
/// img = mvIMPACT.loadImage("torus.bmp");
///
/// mc = measure.MeasureCirclePair();
/// mc.addFeature (measure.cfRadius);
/// mc.find(img,100,20,60,210);
/// 
/// width = mc.getFeature(measure.cfRadius,0) - mc.getFeature(measure.cfRadius,1);
/// \endcode
/// <htmlinclude ref="code_fragment_end.html"/>
/// </mvWrapperDoc>
/// \endxmlonly

//-----------------------------------------------------------------------------
class MeasureCirclePair : public MeasureCircleBase
//-----------------------------------------------------------------------------
{
public:
	/// \brief Allocates a MeasureCirclePair object.
	MeasureCirclePair( PairMode mode = pmPosNeg );

	///\brief Create a MeasureCirclePair object from \a src.
	MeasureCirclePair(const MeasureCirclePair& src);

#ifndef WRAP_PYTHON
	///\brief Overwrite current object with \a src.
	MeasureCirclePair& operator= (const MeasureCirclePair& src);
#endif // #ifndef WRAP_PYTHON
	

	/// \brief Add a feature to search for. 

	/// After the mvIMPACT::measure::MeasureCirclePair::find 
	/// method is called all features added with this function can be queried.
	/// For a list of availabale features see #CircleFeature.
	/// for mopre information on circle features see \ref mvi_measure_subsection_circles_features.
	/// \see mvIMPACT::measure::CircleFeature
	void addFeature (CircleFeature circleFeature);
	

	/// \brief Search for a circle pair.

	/// This function locates a circle pair in a given search window.
	/// The search window is defined by a point and a corresponding width and 
	/// height.
	/// After the search is completed the results can be queried with 
	/// mvIMPACT::measure::MeasureCirclePair::getFeature .
	/// For more information on measurement windows see \ref mvi_measure_subsection_circles_windows.
	void find 
	(
		/// The source image.
		const Image& source, 
		/// The x-coordinate of the start point.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(source)s.width-%(width)s"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int x, 
		/// The y-coordinate of the start point.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(source)s.height-%(height)s"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int y, 
		/// The width of the search window.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(source)s.width-%(x)s"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int width, 
		/// The height of the search window.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(source)s.height-%(y)s"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int height
		);
	

	/// \brief Search for a circle pair.

	/// This function locates a circle pair in a given search window.
	/// The search window is defined by a point and a corresponding width and 
	/// height.
	/// After the search is completed the results can be queried with 
	/// mvIMPACT::measure::MeasureCirclePair::getFeature .
	/// For more information on measurement windows see \ref mvi_measure_subsection_circles_windows.
	void find 
	(
		/// The source image.
		const Image& source, 
		/// The start point.
		Point point,
		/// The width of the search window.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(source)s.width-%(point)s.x"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int width, 
		/// The height of the search window.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(source)s.height-%(point)s.y"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int height
		);


	/// \brief Search for a circle pair.

	/// This function locates a circle pair in a given search window.
	/// The search window is defined by a rectangle.
	/// After the search is completed the results can be queried with 
	/// mvIMPACT::measure::MeasureCirclePair::getFeature .
	/// For more information on measurement windows see \ref mvi_measure_subsection_circles_windows.
	void find 
	(
		/// The source image.
		const Image& source, 
		/// The search window.
		Rectangle& rectangle
		);





	/// This method returns the number of bands a result buffer consists of.
	/// Internally this number is obtained from the source image.
	int getBands() const;

	/// \brief Retrieve the number of slices.
	
	/// This method returns the number of slices a result buffer consists of.
	/// Internally this number is obtained from the source image.
	int getSlices() const;

	/// \brief Retrieve the number of frames.
	
	/// This method returns the number of frames a result buffer consists of.
	/// Internally this number is obtained from the source image.
	int getFrames() const;

	/// \brief Retrieve a single result. 

	/// All features specified with mvIMPACT::measure::MeasureCirclePair::addFeature 
	/// previously can  be queried now. For a list of possible features see #CircleFeature.
	/// \note The index parameter specifies whether the outer or the inner circle of the torus is
	/// \note selected.
	/// \see mvIMPACT::measure::CircleFeature
	double getFeature
	(
		/// The feature to query.
		CircleFeature circleFeature,
		/// Index can be 0 for the outer circle or 1 for the inner circle.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.count(%(band)s,%(slice)s,%(frame)s)-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int index=0, 
		/// If the source images consists of more than one band specify the band offset here.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.bands-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int band =0, 
		/// If the source images consists of more than one slice specify the slice offset here.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.slices-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int slice=0, 
		/// If the source images consists of more than one frame specify the frame offset here.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.frames-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int frame=0) const;
	

public: // properties

	/// \brief Set the polarity mode of the circle pair to search for. 

	/// You can search 
	/// for a bright torus on a dark background or vice versa.
	/// See \ref mvi_measure_subsubsection_circles_classes_polarity for more information on circle polarities.
	/// \see \ref mvi_measure_section_terms
	/// \see mvIMPACT::measure::MeasureCirclePair::getPairMode
	MeasureCirclePair& setPairMode( PairMode pairMode );
	

	/// \brief Retrieve the current polarity mode used for searching.

	/// See \ref mvi_measure_subsubsection_circles_classes_polarity for more information on circle polarities.
	/// \see \ref mvi_measure_section_terms
	/// \see mvIMPACT::measure::MeasureCirclePair::setPairMode
	PairMode getPairMode() const;
	
};

/// \brief Checks if the measure license is valid.
bool checkLicenseMeasure();

/// \brief Return version number from measure module.

/// This function returns a version string. The following format is used: 
/// "major.minor.point.build" where "major" specifies the major version number, "minor" specifies
/// the minor version number, "point" specifies a point version number and "build" specifies the build number. 
/// Example: 6.3.1.4529
//-----------------------------------------------------------------------------
std::string getVersionMeasure();

	} // namespace measure
} // namespace mvIMPACT

#include "mvIMPACT_measure_impl.h"

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_PYTHON

#ifndef MVIMPACT_USE_NAMESPACES
using namespace mvIMPACT::measure;
#endif // #ifndef MVIMPACT_USE_NAMESPACES

#endif // #ifndef WRAP_PYTHON
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

#endif // MVIMPACT_MEASURE_H_
