/**
\xmlonly
mvIMPACT Measure
\endxmlonly

\page autogauge_page Autogauge example (CPP)


\section autogauge_section_purpose Purpose
This example shows how to use the MeasureEdges class to get information about a single edge.

\section autogauge_section_intro What it does
The example shows an image and a LineOfInterest. Every time the LineOfInterest is moved or
resized the mvIMPACT Measure module is used to calculate a line, that represents the detected edge.

\section autogauge_section_description How it works
First of all the mouse handler has to be implemented. This is done by subclassing mvIMPACT::Window and 
implementing the onMousebuttonUp method. This method will be called each time a mouse button is released
and starts the measurement process. The class also stores the results and the condition, if an edge 
could be detected at all, in member variables, which are used by the drawing handler method onDraw to 
draw a line representing the edge. 

The constructor of the subclass first calls the corresponding constructor of the parent class mvIMPACT::Window.
If this is not done, no window will be opened. Then the member variables representing the measurement are initialized.

\code 
class MyWindow: public Window 
{
private:
	int edgeX0;
	int edgeY0;
	int edgeX1;
	int edgeY1;
	bool lineIsValid;

public:
	MyWindow()
		: Window(), edgeX0(0), edgeY0(0), edgeX1(0), edgeY1(0), lineIsValid(false)
	{ }

	MyWindow(const Image &image)
		: Window(image), edgeX0(0), edgeY0(0), edgeX1(0), edgeY1(0), lineIsValid(false)
	{ }

	MyWindow(const MyWindow& r)
		: Window(r), edgeX0(r.edgeX0), edgeY0(r.edgeY0), edgeX1(r.edgeX1), edgeY1(r.edgeY1), lineIsValid(r.lineIsValid)
	{ }

	virtual MyWindow* makeShallowCopy() const
	{
		return new MyWindow(*this);
	}

	MyWindow & operator=(const MyWindow & src)
	{
		Window::operator=(src);
		edgeX0 = src.edgeX0;
		edgeY0 = src.edgeY0;
		edgeX1 = src.edgeX1;
		edgeY1 = src.edgeY1;
		lineIsValid = src.lineIsValid;
		return *this;
	}
	
\endcode

The buttonUp method first retrieves the LineOfInterest object through the OoiSet object associated with the 
window. If there is no such object yet, the method returns immediately. Then the coordinates and the width of the 
LineOfInterest are read.


\code
	//implementation of the buttonUp method
	bool onMouseButtonUp (MouseButton, const PointDouble * )
	{
		//get the coordinates of the line
		Line line = loi.getLine();
		long x0 = line.getP0().getX();
		long y0 = line.getP0().getY();
		long x1 = line.getP1().getX();
		long y1 = line.getP1().getY();

		//get the width of the line and multiply by 2, since it only returns half of the width
		long width = loi.getWidth() * 2;
 		
\endcode

 Then a MeasureEdges object is created. This object is used to do the actual measuring. In the 
 example we want to calculate a line between the two intersection points of the edge with the
 image border. Thus we have to add the corresponding features. In the example these are: 
 efX0Interset, efY0Intersect, efX1Intersect and efY1Intersect. The next instructions tell the 
 MeasureEdges object to ignore edges with a match score below 40 and to deliver only the edge, that 
 has the maximum match score. Thus, there will be no or exactly one result.


\code
		printf("%ld \t %ld \t %ld \t %ld \t %ld\n",x0,y0,x1,y1,width);
		//perform measurement for single edges
		MeasureEdges measure = MeasureEdges();

		//add requested features
		measure.addFeature(efX0Intersect);
		measure.addFeature(efY0Intersect);
		measure.addFeature(efX1Intersect);
		measure.addFeature(efY1Intersect);

		//set some other properties
		measure.setMinScore(40);
		measure.setBestMatchOnly(true);

\endcode
 
 In the next step the measurement process is started. This is done by calling
 mvIMPACT::measure::MeasureEdges::find. In the example the center line and the width of the measurement window is 
 defined by the LineOfInterest. 
 The call is surrounded by a try-except clause, because the function throws an exception
 if it does not find a circle. The number of results is then set to 0.
 If the call was successful the number of results is queried by calling 
 mvIMPACT::measure::MeasureEdges::count. 


\code 
		//define measurement window and search for edges
		long edges;
		try 
		{
			measure.find(dynamic_cast<Image &>(getBuffer()), x0, y0, x1, y1, width);
			//get number of results
			edges = measure.count();
		} 
		catch(ImpactException) 
		{
			edges = 0;
		}
\endcode

 If there are any results they are queried by calling mvIMPACT::meausre::MeasureEdges::get
 and passing the corresponding feature. The four values defining the line are retrieved and the
 member variables are set appropriately, so that they can be displayed. The drawing of the line is done
 by a separate method.

\code
		if (edges > 0) 
		{
			//get results
			edgeX0 = (int)measure.getFeature(efX0Intersect);
			edgeY0 = (int)measure.getFeature(efY0Intersect);
			edgeX1 = (int)measure.getFeature(efX1Intersect);
			edgeY1 = (int)measure.getFeature(efY1Intersect);
			lineIsValid = true;
		} 
		else 
		{
			lineIsValid = false;
		}

		redraw();
		return true;
	}
\endcode

In the main program the derived class MyWindow is instantiated and the LineOfInterest object is created.
Afterwards a short help message is printed.

\code
		//load an image
		Image img = loadImage("stanz.bmp");

		//open window and display the image
		MyWindow window = MyWindow(img);

		//create a LineOfInterest
		LineOfInterest loi = LineOfInterest(window,Line(Point(100,100),Point(170,170)));
		loi.setColor(RGBColor(0,0,255));
		loi.setShowArrow(true); 
		loi.setWidth(20);
		loi.setFillVisible(true);
		loi.setFillTransparent(true);
		loi.setDragWidth(true);

		// output message
		printf("Drag the LOI over an edge to be measured.\n"
			"The LOI should be placed perpendicular to the edge.\n"
			"Releasing the left mouse button starts the measurement.\n"
		    "(The coordinates x0, y0, x1, y1 and the width of the line are listed.) \n\n");
\endcode

Finally the message loop is run and the window is destroyed after the user has pressed space.

\code
		//wait until the user presses space
		runMessageLoop();
\endcode


\section autogauge_section_source Source file

<matrixdoc>
*/


#include "mvIMPACT.h"
#include "mvIMPACT_measure.h"

LineOfInterest loi;

//this class handles the track events
class MyWindow: public Window 
{
private:
	int edgeX0;
	int edgeY0;
	int edgeX1;
	int edgeY1;
	bool lineIsValid;

public:
	MyWindow()
		: Window(), edgeX0(0), edgeY0(0), edgeX1(0), edgeY1(0), lineIsValid(false)
	{ }

	MyWindow(const Image &image)
		: Window(image), edgeX0(0), edgeY0(0), edgeX1(0), edgeY1(0), lineIsValid(false)
	{ }

	MyWindow(const MyWindow& r)
		: Window(r), edgeX0(r.edgeX0), edgeY0(r.edgeY0), edgeX1(r.edgeX1), edgeY1(r.edgeY1), lineIsValid(r.lineIsValid)
	{ }

	virtual MyWindow* makeShallowCopy() const
	{
		return new MyWindow(*this);
	}

	MyWindow & operator=(const MyWindow & src)
	{
		Window::operator=(src);
		edgeX0 = src.edgeX0;
		edgeY0 = src.edgeY0;
		edgeX1 = src.edgeX1;
		edgeY1 = src.edgeY1;
		lineIsValid = src.lineIsValid;
		return *this;
	}

	//implementation of the buttonUp method
	bool onMouseButtonUp (MouseButton /*button*/, const PointDouble * /*pPoint*/)
	{
		// get the coordinates of the line
		Line line = loi.getLine();
		long x0 = line.getP0().getX();
		long y0 = line.getP0().getY();
		long x1 = line.getP1().getX();
		long y1 = line.getP1().getY();

		//get the width of the line and multiply by 2, since it only returns half of the width
		long width = loi.getWidth() * 2;
 
		printf("%ld \t %ld \t %ld \t %ld \t %ld\n",x0,y0,x1,y1,width);
		//perform measurement for single edges
		MeasureEdges measure = MeasureEdges();

		//add requested features
		measure.addFeature(efX0Intersect);
		measure.addFeature(efY0Intersect);
		measure.addFeature(efX1Intersect);
		measure.addFeature(efY1Intersect);

		//set some other properties
		measure.setMinScore(40);
		measure.setBestMatchOnly(true);

		//define measurement window and search for edges
		long edges;
		try 
		{
			measure.find(dynamic_cast<Image &>(getBuffer()), x0, y0, x1, y1, width);
			//get number of results
			edges = measure.count();
		} 
		catch(ImpactException) 
		{
			edges = 0;
		}

		if (edges > 0) 
		{
			//get results
			edgeX0 = (int)measure.getFeature(efX0Intersect);
			edgeY0 = (int)measure.getFeature(efY0Intersect);
			edgeX1 = (int)measure.getFeature(efX1Intersect);
			edgeY1 = (int)measure.getFeature(efY1Intersect);
			lineIsValid = true;
		} 
		else 
		{
			lineIsValid = false;
		}

		redraw();

		return true;
	}

	void onDraw
		(
		DeviceContext *pDc, 
		const mvIMPACT::Rectangle* /*pSourceRect*/,
		const mvIMPACT::Rectangle* /*pDestRect*/
		)
	{
		if(lineIsValid) 
		{
			// get the current scroll position of the window
			int spx = getScrollPosX();
			int spy = getScrollPosY();

			Line line(edgeX0-spx, edgeY0-spy, edgeX1-spx, edgeY1-spy);
			pDc->drawLine(line, RGBColor(0,0,255));
		}
	}
};

int main(int , char* )
{
	try
	{

		//load an image
		Image img = loadImage("stanz.bmp");

		//open window and display the image
		MyWindow window = MyWindow(img);

		//create a LineOfInterest
		loi = LineOfInterest(window, Line(Point(100,100), Point(170,170)));
		loi.setColor(RGBColor(0,0,255));
		loi.setShowArrow(true); 
		loi.setWidth(20);
		loi.setFillVisible(true);
		loi.setFillTransparent(true);
		loi.setDragWidth(true);

		// output message
		printf("Drag the LOI over an edge to be measured.\n"
			"The LOI should be placed perpendicular to the edge.\n"
			"Releasing the left mouse button starts the measurement.\n"
		    "(The coordinates x0, y0, x1, y1 and the width of the line are listed.) \n\n");

		//wait until the user presses space
		runMessageLoop();
	}
	catch(ImpactException& e)
	{
		std::cerr << e.getErrorString() << std::endl;
	}
			
	return 0;
}

//</matrixdoc>
