/**
\xmlonly
mvIMPACT Base
\endxmlonly

\page autotrace_page Autotrace example (C++)


\section autotrace_section_purpose Purpose
This example shows how to use the Tracer class.

\section autotrace_section_intro What it does
The example shows an image and one or two lines of interest, depending on the option the user chooses at the
beginning. Two lines are showed for contuor tracing and one line is showed for tracing around the whole object.
When the user moves a line the application draws the contour of the whole object or the part of the object, that
is delimited by the two lines.

\section autotrace_section_description How it works
First of all the mouse handler has to be implemented. This is done by creating a subclass of class Window and
implementing the onMouseButtonUp method. This method will be called each time a mouse button is released
and starts the tracing process. The class also stores the tracing result and the number of points in member variables.
The trace result is used by the overloaded method onDraw to draw the contour points into
the window.

In the constructor of the subclass first the corresponding constructor of class Window is called. Then
the member variable \c nPoints is initialized.

\code
//this class handles the mouse events
class MyWindow:  public Window 
{
private:
	TraceResult result;
	int nPoints;

public:
	MyWindow() 
		: Window(), nPoints(0)
	{ }

	MyWindow(const Image &image)
		: Window(image), nPoints(0)
	{ }

	MyWindow(const MyWindow& r)
		: Window(r), nPoints(r.nPoints)
	{ }

	virtual MyWindow* makeShallowCopy() const
	{
		return new MyWindow(*this);
	}

	MyWindow & operator=(const MyWindow & src)
	{
		Window::operator=(src);
		nPoints = src.nPoints;
		return *this;
	}

\endcode

The buttonUp method first retrieves creates a Tracer object and sets the maximal number of points
the tracer will deliver before terminating.


\code
		//implementation of the buttonUp method
		bool onMouseButtonUp (MouseButton ,  const PointDouble *)
		{
			Tracer tracer = Tracer();
			// set the number of maximal points for the tracer
			tracer.setMaxPoints(10000);

\endcode

Then depending on the LOIs contained in the Window, the tracing operation is started. This is done
by creating Line objects from the LOIs and calling the tracer, providing these lines as
delimiters for the trace operation. If only one line is given, the tracer will not terminate until
the start point was reached again or the maximum number of points was reached. If two lines are given,
the tracer will not terminate until the second line was reached or the maximum number of points was
reached.

\code
			if (unassigned == loiEnd.getObjectType())
			{
				//start tracing between two lines
				result = tracer.trace(dynamic_cast<Image &>(getBuffer()), loiStart.getLine(), loiEnd.getLine());
			}
			else
			{
				//start tracing around the object
				result = tracer.trace(dynamic_cast<Image &>(getBuffer()), loiStart.getLine());
			}


\endcode

Finally the number of result points is stored in a member variable and the redraw method is called
to force execution of the onDraw method. The onDraw method draws the result points as an overlay into
the window.

\code
			nPoints = result.getNumberOfPoints();

			redraw();
			return true;
\endcode

The onDraw method checks, whether some points were found. If so, it iterates through all the points
and draws them on the device context.

\code
		void onDraw(DeviceContext *dc, const Rectangle *, const Rectangle *)
		{
			if(nPoints > 0)
			{
				for (int counter=0;counter< nPoints;counter++)
				{
					//get the point coordinates and draw them
					PointDouble point = result.getPoint(counter);
					dc->drawPoint(Point((int)point.getX(),(int)point.getY()),RGBColor(0,255,0));
				}
			}
		}
\endcode

The main program first outputs some text and gives the user the ability to enter the desired tracing
option. Then the image is loaded and an instance of the MyWindow class is created. Then a tracer object
is created.
\code
				printf("Select trace mode:\n");
				printf("1 - trace around object\n");
				printf("2 - trace along contour segment\n");
				printf("Your selection [1 or 2 followed by RETURN]: ");

				// get input from stdin
				int ch = getchar();



				//load image
				Image image = loadImage("unwind.bmp");

				//open window that handles tracer events
				MyWindow window = MyWindow(image);


\endcode

Depending on the chosen option one or two LOIs are created and attached to the window.

\code
				//create one or two LOI, depending whether we trace around the object
				//or just a contour
				LineOfInterest loiStart = LineOfInterest(window,Line(58,237,99,237));
				if (ch == '2')
				{
					loiEnd = LineOfInterest(window,Line(108,93,132,112));
				}

				printf("Release mouse to start tracing...\n\n");

\endcode
Finally the message loop is run. At this point all further functionality is done in the MyWindow class, triggered
by the onMouseButtonUp method.

\code
				//keep demo alive until space was pressed
				runMessageLoop();

\endcode

\section autotrace_section_source Source file

<matrixdoc>
*/


#include "mvIMPACT.h"

LineOfInterest loiStart;
LineOfInterest loiEnd;

//this class handles the mouse events
class MyWindow:  public Window 
{
private:
	TraceResult result;
	int nPoints;

public:
	MyWindow() 
		: Window(), nPoints(0)
	{ }

	MyWindow(const Image &image)
		: Window(image), nPoints(0)
	{ }

	MyWindow(const MyWindow& r)
		: Window(r), nPoints(r.nPoints)
	{ }

	virtual MyWindow* makeShallowCopy() const
	{
		return new MyWindow(*this);
	}

	MyWindow & operator=(const MyWindow & src)
	{
		Window::operator=(src);
		nPoints = src.nPoints;
		return *this;
	}

	//implementation of the buttonUp method
	bool onMouseButtonUp (MouseButton /*button*/,  const PointDouble* /*pPoint*/ )
	{
		Tracer tracer = Tracer();
		// set the number of maximal points for the tracer
		tracer.setMaxPoints(10000);

		if (otNone != loiEnd.getObjectType())
		{
			//start tracing between two lines
			result = tracer.trace(dynamic_cast<Image &>(getBuffer()), loiStart.getLine(), loiEnd.getLine());
		}
		else
		{
			//start tracing around the object
			result = tracer.trace(dynamic_cast<Image &>(getBuffer()), loiStart.getLine());
		}

		nPoints = result.getNumberOfPoints();

		redraw();
		return true;
	}

	void onDraw(DeviceContext *dc, const mvIMPACT::Rectangle* /*sourceRect*/, const mvIMPACT::Rectangle* /*destRect*/)
	{
		if(nPoints > 0)
		{
			for (int counter=0;counter< nPoints;counter++)
			{
				//get the point coordinates and draw them
				PointDouble point = result.getPoint(counter);
				dc->drawPoint(Point((int)point.getX(),(int)point.getY()),RGBColor(0,255,0));
			}
		}
	}
};


int main(int, char* [])
{
	try
	{

		printf("Select trace mode:\n");
		printf("1 - trace around object\n");
		printf("2 - trace along contour segment\n");
		printf("Your selection [1 or 2 followed by RETURN]: ");

		// get input from stdin
		int ch = getchar();



		//load image
		Image image = loadImage("unwind.bmp");

		//open window that handles tracer events
		MyWindow window = MyWindow(image);


		//create one or two LOI, depending whether we trace around the object
		//or just a contour
		loiStart = LineOfInterest(window,Line(58,237,99,237));
		if (ch == '2')
		{
			loiEnd = LineOfInterest(window,Line(108,93,132,112));
		}

		printf("Release mouse to start tracing...\n\n");

		//keep demo alive until space was pressed
		runMessageLoop();

	}
	catch(ImpactException e)
	{
		printf("%s\n",e.getErrorString().c_str());
	}
	return 0;

}


//</matrixdoc>

