/**
\xmlonly
mvIMPACT Barcode
\endxmlonly

\page barcode_page Barcode example (CPP)


\section barcode_section_purpose Purpose
Barcodes are a machine-readable representation of information (as bars and gaps) on a surface. 
This example shows how to extract these information from an image. 

\section barcode_section_intro What it does
The sample extracts from a UPC/EAN code the information using 
nearly the default barcode settings.

\section barcode_section_description How it works

First of all the sample loads an image and displays it in a window.

\code
		//load and display an image
		Image image = loadImage("BarCode16.bmp");
		Window window = Window(image);
\endcode

Then the barcode settings are initialised and the MinGaps are set to "11". That means, the barcode
must have minimum 11 spaces (gaps). The last line of the code fragment below enables the barcode 
print quality inspection according to ANSI X3.182. Without enabling this switch no print quality 
inspection values are calculated.

\code
		//setup bar code configuration
		BarcodeConfiguration config = BarcodeConfiguration();
		config.setContrast(10);
		config.setMinGaps(10);
		config.setNoiseThreshold(40);
		config.setCheckPrintQuality(true);
\endcode

In the next step the barcode will be decoded with the given configuration.

\code
		//decode the barcode with the given configuration
		BarcodeResults results = BarcodeResults();
		results = decodeBC(image,config);
\endcode


After that the program iterates through all barcodes and outputs parameters such as code type,
code data, checksum, location, direction and the overprint factor. This information can always be 
inquired, that means it does not depend on the flag that enables or disables print quality inspection.

\code

		int candidates = results.getNumberOfCandidates();
		printf("NumberOfCandidates: %i\n",candidates);
		if (candidates > 0) {
			double total_time = results.getTotalDecodingTime();
			printf("\tTotal decoding time: %3.2f\n",total_time);

			//iterate through all candidates
			for (int candCounter = 0; candCounter < candidates; candCounter++) {
				printf("\t\tCandidate: %i\n",candCounter);
				int codes = results.getNumberOfCodes(candCounter);

				//iterate through all codes
				for (int codeCounter = 0; codeCounter < codes; codeCounter++) {
					printf("\t\t\tCode: %i\t",codeCounter);

					//get the code type
					std::string codeType = results.getCodeType(candCounter,codeCounter);
					printf("Decoder: %s\t",codeType.c_str());

					//get the code data
					std::string codeData = results.getCode(candCounter,codeCounter);
					printf("Code: %s\n",codeData.c_str());

					//get the checksum
					std::string checksum = results.getChecksum(candCounter,codeCounter);
					printf("\t\t\tCkecksum: %s\n",checksum.c_str());

					//get the location 
					std::vector<Point> location = results.getLocation(candCounter);	
					printf("\t\t\tLocation: %i %i %i %i %i %i %i %i\n",
						location[0].getX(), location[0].getY(),
						location[1].getX(), location[1].getY(),
						location[2].getX(), location[2].getY(),
						location[3].getX(), location[3].getY());

					//overprint
					double overprint = results.getOverprint(candCounter);
					printf("\t\t\tOverprint: %f\n",overprint);

					//direction
					bool direction = results.getDirection(candCounter);
					if (direction)
						printf("\t\t\tDirection: forward\n");
					else
						printf("\t\t\tDirection: backward\n");

					//decoding time
					double time = results.getDecodingTime(candCounter);
					printf("\t\t\tDecoding time: %3.2f\n",time);

\endcode


In the next step the print quality inspection values according to ANSI X3.182 are printed
on the console. These values can only be inquired if the print quality inspection was enabled before
decoding the barcode.

\code

					// reflectance minimum
					long rmin = results.getReflectanceMinimum(candCounter);
					printf("\t\t\tX3.182 Reflectance minimum: %ld\n", rmin);

					// reflectance maximum
					long rmax = results.getReflectanceMaximum(candCounter);
					printf("\t\t\tX3.182 Reflectance maximum: %ld\n", rmax);

					// symbol contrast
					long sc = results.getSymbolContrast(candCounter);
					printf("\t\t\tX3.182 Symbol contrast: %ld\n", sc);

					// global threshold
					long gt = results.getGlobalThreshold(candCounter);
					printf("\t\t\tX3.182 Global threshold: %ld\n", gt);

					// minimum edge contrast
					long minec = results.getMinimumEdgeContrast(candCounter);
					printf("\t\t\tX3.182 Minimum edge contrast: %ld\n", minec);

					// modulation
					double mod = results.getModulation(candCounter);
					printf("\t\t\tX3.182 Modulation: %g\n", mod);

					// maximum non-uniformity
					long mnu = results.getMaximumNonUniformity(candCounter);
					printf("\t\t\tX3.182 Maximum non-uniformity: %ld\n", mnu);

					// edge determination test
					bool edt = results.getEdgeDeterminationTest(candCounter);
					if (edt)
						printf("\t\t\tX3.182 Edge determination test: passed\n");
					else
						printf("\t\t\tX3.182 Edge determination test: failed\n");

					// minimum reflectance test
					bool mrt = results.getMinimumReflectanceTest(candCounter);
					if (mrt)
						printf("\t\t\tX3.182 Miniumm reflectance test: passed\n");
					else
						printf("\t\t\tX3.182 Miniumm reflectance test: failed\n");

					// minimum edge contrast test
					bool mect = results.getMinimumEdgeContrastTest(candCounter);
					if (mect)
						printf("\t\t\tX3.182 Miniumm edge contrast test: passed\n");
					else
						printf("\t\t\tX3.182 Miniumm edge contrast test: failed\n");

					// symbol contrast grade
					long scg = results.getSymbolContrastGrade(candCounter);
					printf("\t\t\tX3.182 Symbol contrast grade: %ld", scg);
					switch (scg)
					{
					case 4:
						printf("(A)\n");
						break;
					case 3:
						printf("(B)\n");
						break;
					case 2:
						printf("(C)\n");
						break;
					case 1:
						printf("(D)\n");
						break;
					case 0:
						printf("(F)\n");
						break;
					}

					// modulation grade
					long mg = results.getModulationGrade(candCounter);
					printf("\t\t\tX3.182 Modulation grade: %ld", mg);
					switch (mg)
					{
					case 4:
						printf("(A)\n");
						break;
					case 3:
						printf("(B)\n");
						break;
					case 2:
						printf("(C)\n");
						break;
					case 1:
						printf("(D)\n");
						break;
					case 0:
						printf("(F)\n");
						break;
					}

					// defects grade
					long dg = results.getDefectsGrade(candCounter);
					printf("\t\t\tX3.182 Defects grade: %ld", dg);
					switch (dg)
					{
					case 4:
						printf("(A)\n");
						break;
					case 3:
						printf("(B)\n");
						break;
					case 2:
						printf("(C)\n");
						break;
					case 1:
						printf("(D)\n");
						break;
					case 0:
						printf("(F)\n");
						break;
					}

					// decodability grade
					long decg = results.getDecodabilityGrade(candCounter);
					printf("\t\t\tX3.182 Decodability grade: %ld", decg);
					switch (decg)
					{
					case 4:
						printf("(A)\n");
						break;
					case 3:
						printf("(B)\n");
						break;
					case 2:
						printf("(C)\n");
						break;
					case 1:
						printf("(D)\n");
						break;
					case 0:
						printf("(F)\n");
						break;
					}

					// scan grade
					long sg = results.getScanGrade(candCounter);
					printf("\t\t\tX3.182 Scan grade: %ld", sg);
					switch (sg)
					{
					case 4:
						printf("(A)\n");
						break;
					case 3:
						printf("(B)\n");
						break;
					case 2:
						printf("(C)\n");
						break;
					case 1:
						printf("(D)\n");
						break;
					case 0:
						printf("(F)\n");
						break;



\endcode




\section barcode_section_source Source file

<matrixdoc>
*/

#include "mvIMPACT.h"
#include "mvIMPACT_barcode.h"

int main(int , char* [])
{
	try 
	{
		//load and display an image
		Image image = loadImage("BarCode16.bmp");
		Window window = Window(image);

		//setup bar code configuration
		BarcodeConfiguration config = BarcodeConfiguration();
		config.setContrast(10);
		config.setMinGaps(10);
		config.setNoiseThreshold(40);
		config.setCheckPrintQuality(true);

		//decode the barcode with the given configuration
		BarcodeResults results = BarcodeResults();
		results = decodeBC(image,config);

		int candidates = results.getNumberOfCandidates();
		printf("NumberOfCandidates: %i\n",candidates);
		if (candidates > 0) {
			double total_time = results.getTotalDecodingTime();
			printf("\tTotal decoding time: %3.2f\n",total_time);

			//iterate through all candidates
			for (int candCounter = 0; candCounter < candidates; candCounter++) {
				printf("\t\tCandidate: %i\n",candCounter);
				int codes = results.getNumberOfCodes(candCounter);

				//iterate through all codes
				for (int codeCounter = 0; codeCounter < codes; codeCounter++) {
					printf("\t\t\tCode: %i\t",codeCounter);

					//get the code type
					std::string codeType = results.getCodeType(candCounter,codeCounter);
					printf("Decoder: %s\t",codeType.c_str());

					//get the code data
					std::string codeData = results.getCode(candCounter,codeCounter);
					printf("Code: %s\n",codeData.c_str());

					//get the checksum
					std::string checksum = results.getChecksum(candCounter,codeCounter);
					printf("\t\t\tCkecksum: %s\n",checksum.c_str());

					//get the location 
					std::vector<Point> location = results.getLocation(candCounter);	
					printf("\t\t\tLocation: %i %i %i %i %i %i %i %i\n",
						location[0].getX(), location[0].getY(),
						location[1].getX(), location[1].getY(),
						location[2].getX(), location[2].getY(),
						location[3].getX(), location[3].getY());

					//overprint
					double overprint = results.getOverprint(candCounter);
					printf("\t\t\tOverprint: %f\n",overprint);

					//direction
					bool direction = results.getDirection(candCounter);
					if (direction)
						printf("\t\t\tDirection: forward\n");
					else
						printf("\t\t\tDirection: backward\n");

					//decoding time
					double time = results.getDecodingTime(candCounter);
					printf("\t\t\tDecoding time: %3.2f\n",time);

					// reflectance minimum
					long rmin = results.getReflectanceMinimum(candCounter);
					printf("\t\t\tX3.182 Reflectance minimum: %ld\n", rmin);

					// reflectance maximum
					long rmax = results.getReflectanceMaximum(candCounter);
					printf("\t\t\tX3.182 Reflectance maximum: %ld\n", rmax);

					// symbol contrast
					long sc = results.getSymbolContrast(candCounter);
					printf("\t\t\tX3.182 Symbol contrast: %ld\n", sc);

					// global threshold
					long gt = results.getGlobalThreshold(candCounter);
					printf("\t\t\tX3.182 Global threshold: %ld\n", gt);

					// minimum edge contrast
					long minec = results.getMinimumEdgeContrast(candCounter);
					printf("\t\t\tX3.182 Minimum edge contrast: %ld\n", minec);

					// modulation
					double mod = results.getModulation(candCounter);
					printf("\t\t\tX3.182 Modulation: %g\n", mod);

					// maximum non-uniformity
					long mnu = results.getMaximumNonUniformity(candCounter);
					printf("\t\t\tX3.182 Maximum non-uniformity: %ld\n", mnu);

					// edge determination test
					bool edt = results.getEdgeDeterminationTest(candCounter);
					if (edt)
						printf("\t\t\tX3.182 Edge determination test: passed\n");
					else
						printf("\t\t\tX3.182 Edge determination test: failed\n");

					// minimum reflectance test
					bool mrt = results.getMinimumReflectanceTest(candCounter);
					if (mrt)
						printf("\t\t\tX3.182 Miniumm reflectance test: passed\n");
					else
						printf("\t\t\tX3.182 Miniumm reflectance test: failed\n");

					// minimum edge contrast test
					bool mect = results.getMinimumEdgeContrastTest(candCounter);
					if (mect)
						printf("\t\t\tX3.182 Miniumm edge contrast test: passed\n");
					else
						printf("\t\t\tX3.182 Miniumm edge contrast test: failed\n");

					// symbol contrast grade
					long scg = results.getSymbolContrastGrade(candCounter);
					printf("\t\t\tX3.182 Symbol contrast grade: %ld", scg);
					switch (scg)
					{
					case 4:
						printf("(A)\n");
						break;
					case 3:
						printf("(B)\n");
						break;
					case 2:
						printf("(C)\n");
						break;
					case 1:
						printf("(D)\n");
						break;
					case 0:
						printf("(F)\n");
						break;
					}

					// modulation grade
					long mg = results.getModulationGrade(candCounter);
					printf("\t\t\tX3.182 Modulation grade: %ld", mg);
					switch (mg)
					{
					case 4:
						printf("(A)\n");
						break;
					case 3:
						printf("(B)\n");
						break;
					case 2:
						printf("(C)\n");
						break;
					case 1:
						printf("(D)\n");
						break;
					case 0:
						printf("(F)\n");
						break;
					}

					// defects grade
					long dg = results.getDefectsGrade(candCounter);
					printf("\t\t\tX3.182 Defects grade: %ld", dg);
					switch (dg)
					{
					case 4:
						printf("(A)\n");
						break;
					case 3:
						printf("(B)\n");
						break;
					case 2:
						printf("(C)\n");
						break;
					case 1:
						printf("(D)\n");
						break;
					case 0:
						printf("(F)\n");
						break;
					}

					// decodability grade
					long decg = results.getDecodabilityGrade(candCounter);
					printf("\t\t\tX3.182 Decodability grade: %ld", decg);
					switch (decg)
					{
					case 4:
						printf("(A)\n");
						break;
					case 3:
						printf("(B)\n");
						break;
					case 2:
						printf("(C)\n");
						break;
					case 1:
						printf("(D)\n");
						break;
					case 0:
						printf("(F)\n");
						break;
					}

					// scan grade
					long sg = results.getScanGrade(candCounter);
					printf("\t\t\tX3.182 Scan grade: %ld", sg);
					switch (sg)
					{
					case 4:
						printf("(A)\n");
						break;
					case 3:
						printf("(B)\n");
						break;
					case 2:
						printf("(C)\n");
						break;
					case 1:
						printf("(D)\n");
						break;
					case 0:
						printf("(F)\n");
						break;
					}
				}
			}
		}

		// ask user to end the application
		std::cout << "Press <space> to end the application";
		bool waiting = true;
		while (waiting) {
			if (keyHit()) {							// check for keyboard action
				if (getChar(false) == " ") {		// get the user's input
					waiting = false;
				}
			}

			dispatchMessages();						// dispatch messages of user interface, e.g. refresh window
			Sleep(10);								// wait 10 milliseconds to reduce CPU-load
		}
	}	
	catch(ImpactException& e)	
	{		
		std::cerr << e.getErrorString() << std::endl;	
	}	

	return 0;
}

//</matrixdoc>
