/**
\xmlonly
mvIMPACT GMM
\endxmlonly

\page gmm_page Gmm example (CPP)

\section gmm_section_purpose Purpose
This example shows how to use the Geometrical Model Matcher of mvIMPACT.

\section gmm_section_intro What it does
A model image is analyzed, a subset of the model is constructed and saved to disk and
afterwards the original model is searched for in a scene image.

\section gmm_section_description How it works

First two tool functions are defined. The first one prints the features of a model. The second one
prints information about the matches in an object of class GeoMatchResult.

\code
static void printSelectedFeatures(const GeoModel &model)
{
	fprintf(stderr, "\nFeatures that are used for pattern matching:\n");

	// ----- Corners ----- //
	int nCountCorners = model.countCorners();
	fprintf(stderr, "\nNumber of corners %d\n", nCountCorners);

	std::vector<Corner> corners = model.getCorners();
	for (int i = 0; i < nCountCorners; i++)
	{
		Corner corner = corners.at(i);
		std::cout << 
			"Corner " << (int)i << std::endl <<
			"\tCorner point: " << 
				(int)corner.getCornerPoint().getX() << 
				"/" << 
				(int)corner.getCornerPoint().getY() <<
				std::endl <<
			"\tOpening angle: "	<< (int)(corner.getOpeningAngle()/M_PI*180.0f) <<
				" degrees" << std::endl <<
			"\tBisector: " << (int)(corner.getBisector()/M_PI*180.0f) << std::endl;
	}

	// ----- Arcs ----- //
	int nCountArcs = model.countArcs();
	fprintf(stderr, "\nNumber of arcs %d\n", nCountArcs);

	std::vector<Arc> arcs = model.getArcs();
	for (int i = 0; i < nCountArcs; i++)
	{
		Arc arc = arcs.at(i);
		std::cout << 
			"Arc " << (int)i << std::endl <<
			"\tCenter point: " << 
				(int)arc.getCenter().getX() << 
				"/" << 
				(int)arc.getCenter().getY() <<
				std::endl <<
			"\tradius: "	<< (int)(arc.getRadius()) << std::endl <<
			"\tNumber of Pieces: " << (int)(arc.getNumberOfArcPieces()) << std::endl <<
			"";
	}
}

static void printMatches(const GeoMatchResult & matchResult)
{
	fprintf(stderr, "\nNumber of matches: %d\n", matchResult.getNumberOfMatches());

	int nCountMatches = matchResult.getNumberOfMatches();

	for (int i = 0; i < nCountMatches; i++)
	{
		GeoMatchData currMatch = matchResult.getMatch(i);
		fprintf(stderr, "\nMatch %d: \n", i);
		fprintf(stderr, "x translation: %3.2f\n", currMatch.getTransX());
		fprintf(stderr, "y translation: %3.2f\n", currMatch.getTransY());
		fprintf(stderr, "rotation:      %3.2f degrees\n",currMatch.getRotation()/M_PI*180.0f);
		fprintf(stderr, "scaling:       %3.2f\n", currMatch.getScale());
		fprintf(stderr, "score:         %3.2f\n", currMatch.getScore());
	}
}
\endcode

Then a helper class is defined for displaying a list of corners in a window. The corners
are displayed as a cross consisting of two lines of interest (LOIs).

\code
class CornerDisplay
{
public:

	CornerDisplay(const Window &window)
	{
		m_window = window;
	}

	void clear()
	{
		m_corners.clear();
		m_loi1.clear();
		m_loi2.clear();
	}

	void addCorners(const std::vector<Corner> corners)
	{
		for(int i = 0; i < (int)corners.size(); i++)
		{
			Corner currCorner = corners.at(i);
			m_corners.push_back(currCorner);

			LineOfInterest newLoi1 = LineOfInterest(
				m_window, 
				Line(
					Point(
						(int)(currCorner.getCornerPoint().getX()-5), 
						(int)(currCorner.getCornerPoint().getY())), 
					Point(
						(int)(currCorner.getCornerPoint().getX()+5), 
						(int)(currCorner.getCornerPoint().getY()))));
			newLoi1.setColor(RGBColor(255,0,0));
			m_loi1.push_back(newLoi1);

			LineOfInterest newLoi2 = LineOfInterest(
				m_window, 
				Line(
					Point(
						(int)(currCorner.getCornerPoint().getX()), 
						(int)(currCorner.getCornerPoint().getY()-5)), 
					Point(
						(int)(currCorner.getCornerPoint().getX()), 
						(int)(currCorner.getCornerPoint().getY()+5))));
			newLoi2.setColor(RGBColor(255,0,0));
			m_loi2.push_back(newLoi2);
		}
	}


private:
	std::vector<Corner> m_corners;
	std::vector<LineOfInterest> m_loi1;
	std::vector<LineOfInterest> m_loi2;
	Window m_window;
};
\endcode

Then another helper class is defined to display the matches represented by a
GeoMatchResult object in a given window. The positions of the matches as well as
a line of text additionally describing the rotation and scaling of the matches are
displayed in the given window which should display the scene image.

\code
class MatchDisplay
{
public:

	MatchDisplay(const Window &window)
	{
		m_window = window;
	}

	void clear()
	{
		m_gmr = GeoMatchResult();
		m_loi1.clear();
		m_loi2.clear();
		m_tloi.clear();
	}

	void setGeoMatchResult(const GeoMatchResult& r, const Point &center)
	{
		clear();
		m_gmr = r;
		m_center = center;

		for(int i = 0; i < m_gmr.getNumberOfMatches(); i++)
		{
			GeoMatchData currMatch = m_gmr.getMatch(i);

			LineOfInterest newLoi1 = LineOfInterest(
				m_window, 
				Line(
				Point(
				(int)(center.getX() + currMatch.getTransX() - 5), 
				(int)(center.getY() + currMatch.getTransY())),
				Point(
				(int)(center.getX() + currMatch.getTransX() + 5), 
				(int)(center.getY() + currMatch.getTransY()))));
			newLoi1.setColor(RGBColor(0,255,0));
			m_loi1.push_back(newLoi1);

			LineOfInterest newLoi2 = LineOfInterest(
				m_window, 
				Line(
				Point(
				(int)(center.getX() + currMatch.getTransX()), 
				(int)(center.getY() + currMatch.getTransY() - 5)),
				Point(
				(int)(center.getX() + currMatch.getTransX()), 
				(int)(center.getY() + currMatch.getTransY() + 5))));
			newLoi2.setColor(RGBColor(0,255,0));
			std::stringstream no;
			no << i;
			newLoi2.setText(no.str());
			newLoi2.setTextColor(RGBColor(0,0,255));
			m_loi2.push_back(newLoi2);

			Point pt = Point(220, 20*i + 20);
			LineOfInterest newTLoi = LineOfInterest(
				m_window,
				Line(pt, pt));
			std::stringstream text;
			text << std::setw(3) << std::setprecision(2) << std::setiosflags(std::ios::fixed) << 
				"" << i << " - " <<
				"tx: " << currMatch.getTransX() << " " <<
				"ty: " << currMatch.getTransY() << " " <<
				"rot: " << currMatch.getRotation()/M_PI*180.0f << " degrees " <<
				"scal: " << currMatch.getScore()*100. ;
			newTLoi.setText(text.str()).setTextColor(RGBColor(0,255,255));
		}
	}


private:
	GeoMatchResult m_gmr;
	std::vector<LineOfInterest> m_loi1;
	std::vector<LineOfInterest> m_loi2;
	std::vector<LineOfInterest> m_tloi;
	Window m_window;
	Point m_center;
};
\endcode

In the main program, first an image is loaded and displayed in a window.

\code
		// Load model image
		Image modelImage = loadImage("model.bmp");

		// display the loaded image
		Window window = Window(modelImage);
		runMessageLoop("The Model! Click the space bar to continue\n");
\endcode

Then a GeoModel object is constructed. During construction the feature extraction
process is done. The resulting model shall contain a maximum of 7 features and
a threshold of 30 is used to choose edge pixels for model verification.

\code
		// Model
		GeoModel geoModel = GeoModel(modelImage, 7, 30);
\endcode

Afterwards the model features are printed on the terminal and displayed in the
window using the helper class CornerDisplay.

\code
		// Read edge points of model
		Image edgePointImage = geoModel.getEdgePoints();

		// Print all model features:
		printSelectedFeatures(geoModel);
		CornerDisplay cd = CornerDisplay(window);
		cd.addCorners(geoModel.getCorners());

		window.redraw();
		runMessageLoop("The 7 corners! Click the space bar to continue\n");
		cd.clear();
\endcode

Now the edge points used for model verification are displayed in the window. Afterwards
the window is changed to show again the model image.

\code
		window.setBuffer( edgePointImage );	window.redraw();
		runMessageLoop("The Edge Points! Click the space bar to continue\n");
		window.setBuffer( modelImage ); window.redraw();

\endcode

Now a new empty model is created using the same width and height as the existing one.
Then at most 5 features are copied from the existing model to the newly created one, i.e.
afterwards it is a "reduced version" of the original model.

\code
		GeoModel myModel = GeoModel(geoModel.getWidth(), geoModel.getHeight());

		myModel.setEdgePoints(geoModel.getEdgePoints());
		int nCountCorners = geoModel.countCorners();
		int nCountCopyCorners = nCountCorners;

		if(nCountCopyCorners > 5) { nCountCopyCorners = 5; }
		std::vector<Corner> corners = geoModel.getCorners();

		for(int i = 0; i < nCountCopyCorners; i++)
		{ myModel.addCorner(corners.at(i));	}
\endcode

Afterwards the new reduced model is displayed in the window.

\code
		// Print selected 5 model features from copied model:
		printSelectedFeatures(myModel);
		cd.addCorners(myModel.getCorners());

		window.redraw();
		runMessageLoop("The 5 corners! Click the space bar to continue\n");
		cd.clear();
\endcode

Then the reduced model is saved on disk.

\code
		// Save copied model to file "copiedmodel.gmm"
		myModel.save("copiedmodel.gmm");
\endcode

Then the rotation center of the model is set. This point is used to specify the
translation position of a model in the scene as well.

\code
		// Set rotation center to the middle of the model image
		double dx = geoModel.getWidth()/2.0;
		geoModel.setRotationCenterX(dx);
		double dy = geoModel.getHeight()/2.0;
		geoModel.setRotationCenterY(dy);
\endcode

Then the object representing the matcher is created and some configuration
settings are set.

\code
		// generate Matcher object
		GeoMatcher matcher;

		// Set some options for pattern matching. It is not obligatory to set options.
		// You can also use default parameters.
		matcher.setMinScore(0.8);
		matcher.setMaxNumberOfMatches(mmMatcherFindAll);
\endcode

Afterwards the scene image is loaded from disk and the pattern matching process
is started using this scene.

\code
		// Load scene image
		Image sceneImage = loadImage("scene.bmp");

		// Start pattern matching
		GeoMatchResult matchResult = matcher.findGeoModel(geoModel, sceneImage);
\endcode

Finally the results of the matching process are displayed on the terminal as
well as in a newly created window.

\code
		printMatches(matchResult);
		MatchDisplay md(sceneWindow);
		md.setGeoMatchResult(matchResult, Point((int)dx, (int)dy));

		// display the scene image
		runMessageLoop("The Scene! Click the space bar to continue\n");
\endcode

\section gmm_section_source Source file

<matrixdoc>
*/

#include "mvIMPACT.h"
#include "mvIMPACT_gmm.h"

#include <iomanip>

#ifndef M_PI
#	define M_PI       3.14159265358979323846
#endif

static void printSelectedFeatures(const GeoModel &model)
{
	fprintf(stderr, "\nFeatures that are used for pattern matching:\n");

	// ----- Corners ----- //
	int nCountCorners = model.countCorners();

	fprintf(stderr, "\nNumber of corners %d\n", nCountCorners);

	std::vector<Corner> corners = model.getCorners();
	for (size_t i = 0; i < corners.size(); i++)
	{
		Corner corner = corners.at(i);
		std::cout << 
			"Corner " << (int)i << std::endl <<
			"\tCorner point: " << 
				(int)corner.getCornerPoint().getX() << 
				"/" << 
				(int)corner.getCornerPoint().getY() <<
				std::endl <<
			"\tOpening angle: "	<< (int)(corner.getOpeningAngle()/M_PI*180.0f) <<
				" degrees" << std::endl <<
			"\tBisector: " << (int)(corner.getBisector()/M_PI*180.0f) << std::endl;
			
	}

	// ----- Arcs ----- //
	int          nCountArcs = model.countArcs();

	fprintf(stderr, "\nNumber of arcs %d\n", nCountArcs);

	std::vector<mvIMPACT::gmm::Arc> arcs = model.getArcs();
	for (size_t i = 0; i < arcs.size(); i++)
	{
		mvIMPACT::gmm::Arc arc = arcs.at(i);
		std::cout << 
			"Arc " << (int)i << std::endl <<
			"\tCenter point: " << 
				(int)arc.getCenter().getX() << 
				"/" << 
				(int)arc.getCenter().getY() <<
				std::endl <<
			"\tradius: "	<< (int)(arc.getRadius()) << std::endl <<
			"\tNumber of Pieces: " << (int)(arc.getNumberOfArcPieces()) << std::endl <<
			"";
	}
}

static void printMatches(const GeoMatchResult & matchResult)
{
	fprintf(stderr, "\nNumber of matches: %d\n", matchResult.getNumberOfMatches());

	int nCountMatches = matchResult.getNumberOfMatches();

	for (int i = 0; i < nCountMatches; i++)
	{
		GeoMatchData currMatch = matchResult.getMatch(i);
		fprintf(stderr, "\nMatch %d: \n", i);
		fprintf(stderr, "x translation: %3.2f\n", currMatch.getTransX());
		fprintf(stderr, "y translation: %3.2f\n", currMatch.getTransY());
		fprintf(stderr, "rotation:      %3.2f degrees\n",currMatch.getRotation()/M_PI*180.0f);
		fprintf(stderr, "scaling:       %3.2f\n", currMatch.getScale());
		fprintf(stderr, "score:         %3.2f\n", currMatch.getScore());
	}
}


class CornerDisplay
{
public:

	CornerDisplay(const Window &window)
	{
		m_window = window;
	}

	void clear()
	{
		m_corners.clear();
		m_loi1.clear();
		m_loi2.clear();
	}

	void addCorners(const std::vector<Corner> corners)
	{
		for(int i = 0; i < (int)corners.size(); i++)
		{
			Corner currCorner = corners.at(i);
			m_corners.push_back(currCorner);

			LineOfInterest newLoi1 = LineOfInterest(
				m_window, 
				Line(
					Point(
						(int)(currCorner.getCornerPoint().getX()-5), 
						(int)(currCorner.getCornerPoint().getY())), 
					Point(
						(int)(currCorner.getCornerPoint().getX()+5), 
						(int)(currCorner.getCornerPoint().getY()))));
			newLoi1.setColor(RGBColor(255,0,0));
			m_loi1.push_back(newLoi1);

			LineOfInterest newLoi2 = LineOfInterest(
				m_window, 
				Line(
					Point(
						(int)(currCorner.getCornerPoint().getX()), 
						(int)(currCorner.getCornerPoint().getY()-5)), 
					Point(
						(int)(currCorner.getCornerPoint().getX()), 
						(int)(currCorner.getCornerPoint().getY()+5))));
			newLoi2.setColor(RGBColor(255,0,0));
			m_loi2.push_back(newLoi2);
		}
	}


private:
	std::vector<Corner> m_corners;
	std::vector<LineOfInterest> m_loi1;
	std::vector<LineOfInterest> m_loi2;
	Window m_window;
};

class MatchDisplay
{
public:

	MatchDisplay(const Window &window)
	{
		m_window = window;
	}

	void clear()
	{
		m_gmr = GeoMatchResult();

		m_loi1.clear();
		m_loi2.clear();
		m_tloi.clear();
	}

	void setGeoMatchResult(const GeoMatchResult& r, const Point &center)
	{
		clear();
		m_gmr = r;
		m_center = center;

		for(int i = 0; i < m_gmr.getNumberOfMatches(); i++)
		{
			GeoMatchData currMatch = m_gmr.getMatch(i);

			LineOfInterest newLoi1 = LineOfInterest(
				m_window, 
				Line(
				Point(
				(int)(center.getX() + currMatch.getTransX() - 5), 
				(int)(center.getY() + currMatch.getTransY())),
				Point(
				(int)(center.getX() + currMatch.getTransX() + 5), 
				(int)(center.getY() + currMatch.getTransY()))));
			newLoi1.setColor(RGBColor(0,255,0));
			m_loi1.push_back(newLoi1);

			LineOfInterest newLoi2 = LineOfInterest(
				m_window, 
				Line(
				Point(
				(int)(center.getX() + currMatch.getTransX()), 
				(int)(center.getY() + currMatch.getTransY() - 5)),
				Point(
				(int)(center.getX() + currMatch.getTransX()), 
				(int)(center.getY() + currMatch.getTransY() + 5))));
			newLoi2.setColor(RGBColor(0,255,0));
			std::stringstream no;
			no << i;
			newLoi2.setText(no.str());
			newLoi2.setTextColor(RGBColor(0,0,255));
			m_loi2.push_back(newLoi2);

			Point pt = Point(220, 20*i + 20);
			LineOfInterest newTLoi = LineOfInterest(
				m_window,
				Line(pt, pt));
			std::stringstream text;
			text << std::setw(3) << std::setprecision(2) << std::setiosflags(std::ios::fixed) << 
				"" << i << " - " <<
				"tx: " << currMatch.getTransX() << " " <<
				"ty: " << currMatch.getTransY() << " " <<
				"rot: " << currMatch.getRotation()/M_PI*180.0f << " degrees " <<
				"scal: " << currMatch.getScore()*100. ;
			newTLoi.setText(text.str()).setTextColor(RGBColor(0,255,255));
		}
	}


private:
	GeoMatchResult m_gmr;
	std::vector<LineOfInterest> m_loi1;
	std::vector<LineOfInterest> m_loi2;
	std::vector<LineOfInterest> m_tloi;
	Window m_window;
	Point m_center;
};

int main(int /*argc*/, char* /*argv*/[])
{
	try
	{
		// Load model image
		Image modelImage = loadImage("model.bmp");

		// display the loaded image
		Window window = Window(modelImage);
		runMessageLoop("The Model! Click the space bar to continue\n");

		// Model
		GeoModel geoModel = GeoModel(modelImage, 7, 30);

		// Read edge points of model
		Image edgePointImage = geoModel.getEdgePoints();

		// Print all model features:
		printSelectedFeatures(geoModel);
		CornerDisplay cd = CornerDisplay(window);
		cd.addCorners(geoModel.getCorners());

		window.redraw();
		runMessageLoop("The 7 corners! Click the space bar to continue\n");
		cd.clear();

		window.setBuffer( edgePointImage );	window.redraw();
		runMessageLoop("The Edge Points! Click the space bar to continue\n");
		window.setBuffer( modelImage ); window.redraw();

		GeoModel myModel = GeoModel(geoModel.getWidth(), geoModel.getHeight());

		myModel.setEdgePoints(geoModel.getEdgePoints());
		int nCountCorners = geoModel.countCorners();
		int nCountCopyCorners = nCountCorners;

		if(nCountCopyCorners > 5) { nCountCopyCorners = 5; }
		std::vector<Corner> corners = geoModel.getCorners();

		for(int i = 0; i < nCountCopyCorners; i++)
		{ myModel.addCorner(corners.at(i));	}

		// Print selected 5 model features from copied model:
		printSelectedFeatures(myModel);
		cd.addCorners(myModel.getCorners());

		window.redraw();
		runMessageLoop("The 5 corners! Click the space bar to continue\n");
		cd.clear();

		// Save copied model to file "copiedmodel.gmm"
		myModel.save("copiedmodel.gmm");

		// Set rotation center to the middle of the model image
		double dx = geoModel.getWidth()/2.0;
		geoModel.setRotationCenterX(dx);
		double dy = geoModel.getHeight()/2.0;
		geoModel.setRotationCenterY(dy);

		// generate Matcher object
		GeoMatcher matcher;

		// Set some options for pattern matching. It is not obligatory to set options.
		// You can also use default parameters.
		matcher.setMinScore(0.8);
		matcher.setMaxNumberOfMatches(mmMatcherFindAll);

		// Load scene image
		Image sceneImage = loadImage("scene.bmp");

		// Start pattern matching
		GeoMatchResult matchResult = matcher.findGeoModel(geoModel, sceneImage);

		Window sceneWindow( sceneImage );

		printMatches(matchResult);
		MatchDisplay md(sceneWindow);
		md.setGeoMatchResult(matchResult, Point((int)dx, (int)dy));

		// display the scene image
		runMessageLoop("The Scene! Click the space bar to continue\n");
	}
	catch(ImpactException& e)
	{
		std::cerr << e.getErrorString() << std::endl;
	}

	return 0;
}

//</matrixdoc>
