/**
\xmlonly
mvIMPACT Base
\endxmlonly

\page imageproperties_page Image properties example (CPP)

\section imageproperties_section_purpose Purpose
This sample shows a lot of properties for a given image.

\section imageproperties_section_intro What it does
This sample loads an image from disk and prints all available properties to the terminal.

\section imageproperties_section_description How it works
First the image file name is saved in a variable and the image is loaded from disk.

\code
		string image_name = "pcb.bmp";
		
		// load an image
		Image img = loadImage(image_name);
\endcode

Then some very basic properties are displayed: If the image is a child or a parent image,
the dimensions of the image, its datatype, buffer attribute, the bits per sample, the size
of the used datatype in byte and the duration of the image sequence in a multiple-frame image is
printed.

\code
		// print image properties
		cout << "image file: " << image_name << endl;
		
		cout << "isChild: " << img.getIsChild() << endl;
		cout << "isParent: " << img.getIsParent() << endl;
		
		cout << "Width: " << img.getWidth() << endl;
		cout << "Height: " << img.getHeight() << endl;
		cout << "Bands: " << img.getBands() << endl;
		cout << "Slices: " << img.getSlices() << endl;
		cout << "Frames: " << img.getFrames() << endl;
		
		cout << "Datatype: " << img.getDatatype() << endl;
		cout << "Attribute: " << img.getAttribute() << endl;
		
		//cout << "NumDevice: " << img.getNumDevice() << endl;
		cout << "IsChild: " << img.getIsChild() << endl;
		
		cout << "BitsPerSample: " << img.getBitsPerSample() << endl;
		
		cout << "SequenceDuration: " << img.getSequenceDuration() << endl;
\endcode

Then properties applying only to child images are printed. For parent
images these properties are all set to zero, though.

\code
		cout << "XOffset: " << img.getXOffset() << endl;
		cout << "YOffset: " << img.getYOffset() << endl;
		cout << "BandsOffset: " << img.getBandsOffset() << endl;
		cout << "SlicesOffset: " << img.getSlicesOffset() << endl;
		cout << "FramesOffset: " << img.getFramesOffset() << endl;
\endcode

Then some properties not available on all platforms are retrieved. The
exception thrown if they are not available is caught and a message is
displayed.

\code
		try {
			 cout << "isThreadSafe: " << img.getIsThreadSafe() << endl;
		} catch(ImpactException e) {
			 cout << "trying to get property isThreadSafe led to error: " << e.what() << endl;
		}
		
\endcode

Afterwards the calibration values for the four dimension where calibration
is possible are displayed.

\code
		cout << "isHorizontalCalibrated: " << img.getIsHorizontalCalibrated() << endl;
		Calibrator xcal = img.getXCalibrator();
		cout << "XCalibrator: " << (std::string)xcal.getUnit() << " 10^" << xcal.getPower() << " " << xcal.getOffset() << endl;
		
		cout << "isVerticalCalibrated: " << img.getIsVerticalCalibrated() << endl;
		Calibrator ycal = img.getYCalibrator();
		cout << "YCalibrator: " << (std::string)ycal.getUnit() << " 10^" << ycal.getPower() << " " << ycal.getOffset() << endl;
		
		cout << "isSlicesCalibrated: " << img.getIsSlicesCalibrated() << endl;
		Calibrator zcal = img.getSlicesCalibrator();
		cout << "SlicesCalibrator: " << (std::string)zcal.getUnit() << " 10^" << zcal.getPower() << " " << zcal.getOffset() << endl;
		for (int z=0; z<img.getSlices(); ++z)
			cout << zcal[z] << ", ";
		cout << std::endl;
		
		cout << "isFramesCalibrated: " << img.getIsFramesCalibrated() << endl;
		Calibrator tcal = img.getFramesCalibrator();
		cout << "FramesCalibrator: " << (std::string)tcal.getUnit() << " 10^" << tcal.getPower() << " " << tcal.getOffset() << endl;
		for (int t=0; t<img.getFrames(); ++t)
			cout << tcal[t] << ", ";
		cout << std::endl;
\endcode

Finally a window is opened to display the image and the user is requested to press space.

\code
		// open window to display image
		Window wnd = Window(img);

		// keep sample alive until console window is closed or space bar is pressed
		runMessageLoop();
\endcode

\section imageproperties_section_source Source file

<matrixdoc>
*/

#include "mvIMPACT.h"

#include "iostream"
using namespace std;

int main(int /*argc*/, char* /*argv*/[])
{
	try
	{
		string image_name = "pcb.bmp";
		
		// load an image
		Image img = loadImage(image_name);
		
		// print image properties
		cout << "image file: " << image_name << endl;
		
		cout << "isChild: " << img.getIsChild() << endl;
		cout << "isParent: " << img.getIsParent() << endl;
		
		cout << "Width: " << img.getWidth() << endl;
		cout << "Height: " << img.getHeight() << endl;
		cout << "Bands: " << img.getBands() << endl;
		cout << "Slices: " << img.getSlices() << endl;
		cout << "Frames: " << img.getFrames() << endl;
		
		cout << "Datatype: " << img.getDatatype() << endl;
		cout << "Attribute: " << img.getAttribute() << endl;
		
		//cout << "NumDevice: " << img.getNumDevice() << endl;
		cout << "IsChild: " << img.getIsChild() << endl;
		
		cout << "BitsPerSample: " << img.getBitsPerSample() << endl;
		cout << "DatatypeSize: " << img.getDatatypeSize() << endl;
		
		cout << "SequenceDuration: " << img.getSequenceDuration() << endl;

		cout << "XOffset: " << img.getXOffset() << endl;
		cout << "YOffset: " << img.getYOffset() << endl;
		cout << "BandsOffset: " << img.getBandsOffset() << endl;
		cout << "SlicesOffset: " << img.getSlicesOffset() << endl;
		cout << "FramesOffset: " << img.getFramesOffset() << endl;
		
		try {
			 cout << "isThreadSafe: " << img.getIsThreadSafe() << endl;
		} catch(ImpactException e) {
			 cout << "trying to get property isThreadSafe led to error: " << e.what() << endl;
		}
		
	
		cout << "isHorizontalCalibrated: " << img.getIsHorizontalCalibrated() << endl;
		Calibrator xcal = img.getXCalibrator();
		cout << "XCalibrator: " << (std::string)xcal.getUnit() << " 10^" << xcal.getPower() << " " << xcal.getOffset() << endl;
		
		cout << "isVerticalCalibrated: " << img.getIsVerticalCalibrated() << endl;
		Calibrator ycal = img.getYCalibrator();
		cout << "YCalibrator: " << (std::string)ycal.getUnit() << " 10^" << ycal.getPower() << " " << ycal.getOffset() << endl;
		
		cout << "isSlicesCalibrated: " << img.getIsSlicesCalibrated() << endl;
		Calibrator zcal = img.getSlicesCalibrator();
		cout << "SlicesCalibrator: " << (std::string)zcal.getUnit() << " 10^" << zcal.getPower() << " " << zcal.getOffset() << endl;
		for (int z=0; z<img.getSlices(); ++z)
			cout << zcal[z] << ", ";
		cout << std::endl;
		
		cout << "isFramesCalibrated: " << img.getIsFramesCalibrated() << endl;
		Calibrator tcal = img.getFramesCalibrator();
		cout << "FramesCalibrator: " << (std::string)tcal.getUnit() << " 10^" << tcal.getPower() << " " << tcal.getOffset() << endl;
		for (int t=0; t<img.getFrames(); ++t)
			cout << tcal[t] << ", ";
		cout << std::endl;
		
		// open window to display image
		Window wnd = Window(img);

		// keep sample alive until console window is closed or space bar is pressed
		runMessageLoop();
	}
	catch(ImpactException& e)
	{
		std::cerr << e.getErrorString() << std::endl;
	}

	return 0;
}

//</matrixdoc>
