/**
\xmlonly
mvIMPACT Base
\endxmlonly

\page keyboard_page Direct keyboard input example (CPP)

\section keyboard_section_purpose Purpose
This sample shows how direct keyboard input from the terminal (console, shell) window is handled.

\section keyboard_section_intro What it does
This sample shows how blocking and non-blocking functions for getting keyboard inputs work.


\section keyboard_section_description How it works

First the use of non-blocking keyboard input is shown. The keyHit function is
run in a while loop where actually nothing is done, but where the programmer
can run his processing loop which can then be aborted by the user by pressing some
key.

\code
		// Running non-blocking keyHit in a loop
		std::cout << "Hit some key, please";
		std::cout << std::endl << std::endl;
		while (!keyHit())
		{ 
			// working hard here
		}
		std::cout << "Thanks for hitting a key!" << std::endl;
\endcode

Afterwards the key which was pressed is read via the function mvIMPACT::getChar and
printed. This function returns the first unprocessed key in the keyboard buffer. If there
is no such key, it blocks until there is one available. In this case the first alternative applies.

\code
		// we have a key here now that should be eaten
		std::string c = getChar();
		std::cout << "Thanks for hitting: " << c;
		std::cout << std::endl << std::endl;
\endcode

Then getChar is called again, but blocks, because the user has not pressed any further
key until now. Afterwards the key is printed. The keyboard echo is turned off by default.

\code
		// Running blocking getChar without echo
		std::cout << "Hit some more key, please";
		std::cout << std::endl << std::endl;
		c = getChar();
		std::cout << "Thanks for hitting: " << c;
		std::cout << std::endl << std::endl;

\endcode

Now the same procedure is done with the only difference that the keyboard echo is turned on.

\code
		// Running blocking getChar with echo
		std::cout << "Hit some more key, please";
		std::cout << std::endl << std::endl;
		c = getChar(true);
		std::cout << " <-- that's the getChar echo\n";
		std::cout << "Thanks for hitting: " << c;
		std::cout << std::endl << std::endl;
\endcode

Finally the user is requested to press a key to quit the program.

\code
		std::cout << "press any key to quit";
		c = getChar();
\endcode


\section keyboard_section_source Source file

<matrixdoc>
*/

#include "mvIMPACT.h"

int main(int /*argc*/, char* /*argv*/[])
{
	try
	{
		// Running non-blocking keyHit in a loop
		std::cout << "Hit some key, please";
		std::cout << std::endl << std::endl;
		while (!keyHit())
		{ 
			// working hard here
		}
		std::cout << "Thanks for hitting a key!" << std::endl;

		// we have a key here now that should be eaten
		std::string c = getChar();
		std::cout << "Thanks for hitting: " << c;
		std::cout << std::endl << std::endl;

		// Running blocking getChar without echo
		std::cout << "Hit some more key, please";
		std::cout << std::endl << std::endl;
		c = getChar();
		std::cout << "Thanks for hitting: " << c;
		std::cout << std::endl << std::endl;

		// Running blocking getChar with echo
		std::cout << "Hit some more key, please";
		std::cout << std::endl << std::endl;
		c = getChar(true);
		std::cout << " <-- that's the getChar echo\n";
		std::cout << "Thanks for hitting: " << c;
		std::cout << std::endl << std::endl;

		std::cout << "press any key to quit";
		c = getChar();

	}	
	catch(ImpactException& e)	
	{		
		std::cerr << e.getErrorString() << std::endl;	
	}	
	return 0;
}

//</matrixdoc>
