//----------------------------------------------------------------------
//----------------------------------------------------------------------
// MATRIX VISION GmbH
// mvIMPACT - demo
// Mon Feb 14 11:50:05 CET 2005
//----------------------------------------------------------------------
#ifndef __ocr_h__
#define __ocr_h__

//----------------------------------------------------------------------
//----------------------------------------------------------------------
#include <stdio.h>
#include <vl_base.h>
#include <vl_ocr.h>

#include "ocrtool.h"

//----------------------------------------------------------------------
//----------------------------------------------------------------------
/*! \brief Segmentation types. */
typedef enum SEG_TYPE
{
	SEG_NONE = 0,			/*!< \brief No segmentation. */
	SEG_SPLIT,				/*!< \brief Segmentation by split. */
	SEG_MERGE,				/*!< \brief Segmentation by merge. */
	SEG_SPLIT_MERGE,		/*!< \brief Segmentation by split & merge. */
	SEG_MERGE_SPLIT,		/*!< \brief Segmentation by merge & split. */

};

/*! \brief Classification methode. */
typedef enum CLS_METHOD
{
	CLSM_MATCH = 0,
	CLSM_FOURIER
};

//----------------------------------------------------------------------
//----------------------------------------------------------------------
/*! \class CDmoOCR
 *  \brief This is the image processing class \c OCR .
 *  \author MATRIX VISION GmbH (td)
 *  \date 14.02.2005 */
class CDmoOCR
{
public:
	/*----------------------------------------------------------------*/
	/*! \brief The constructor. */
	CDmoOCR( )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		m_hOcrHandle = IPL_DONT_CARE;

		m_SegCount   = 0;
		m_MinQuality = DEFAULT_QMIN;

		memset( m_Rect, 0, sizeof( IPL_RECT ) * MAX_OBJ_CNT );

		memset( m_DisplayText, 0, MAX_DISP_CNT );
		memset( m_Quality, 0, sizeof(int) * MAX_OBJ_CNT );
		memset( m_ReadText, '?', MAX_OBJ_CNT );

		m_WMin = DEFAULT_WMIN;
		m_WMax = DEFAULT_WMAX;
		m_HMin = DEFAULT_HMIN;
		m_HMax = DEFAULT_HMAX;

		m_WMinC = DEFAULT_WMINC;
		m_WMaxC = DEFAULT_WMAXC;
		m_HMinC = DEFAULT_HMINC;
		m_HMaxC = DEFAULT_HMAXC;

		m_bgrd = BGRD_WHITE;

		m_BlankValue = DEFAULT_BLANK_WIDTH;

		m_bDefaultSegSize   = false;
		m_bAutoSegmentation = false;

		m_SegType = SEG_NONE;
		m_ClsM    = CLSM_MATCH;

		dprintf(( " -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
	}

	/*----------------------------------------------------------------*/
	/*! \brief The destructor. */
	virtual  ~CDmoOCR( )
	{
		dprintf(( " ++ ~%s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		closeOCR( );

		dprintf(( " -- ~%s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
	}

	/*----------------------------------------------------------------*/
	/*! \brief Close the OCR. */
	void closeOCR( )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		if( m_hOcrHandle != IPL_DONT_CARE )
		{
			OCR_Close( m_hOcrHandle );
			m_hOcrHandle = IPL_DONT_CARE;
		}

		dprintf(( " -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
	}

	/*----------------------------------------------------------------*/
	/*! \brief Initialize the OCR.
	 *  \param pOCRFile name of the pattern file
	 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE . */
	int initOCR( char* pOCRFile = DEFAULT_OCR_FILE )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		int ret;
		/* open the ocr pattern and make a ocr buffer handle */
		DMO_CHECK_ERR(( ret = OCR_Open( &m_hOcrHandle, pOCRFile, 1 )), IPL_GetErrorCode2( ));
		/* set the classification methode */
		DMO_CHECK_ERR(( ret = setClsMethod( )), -1 );

		/* some other OCR settings  */
		DMO_CHECK_ERR(( ret = setBackground( m_bgrd )), -1 );

		DMO_CHECK_ERR(( ret = setWMin( m_WMin )), -1 );
		DMO_CHECK_ERR(( ret = setWMax( m_WMax )), -1 );
		DMO_CHECK_ERR(( ret = setHMin( m_HMin )), -1 );
		DMO_CHECK_ERR(( ret = setHMax( m_HMax )), -1 );

		DMO_CHECK_ERR(( ret = setMinCharW( m_WMinC )), -1 );
		DMO_CHECK_ERR(( ret = setMaxCharW( m_WMaxC )), -1 );
		DMO_CHECK_ERR(( ret = setMinCharH( m_HMinC )), -1 );
		DMO_CHECK_ERR(( ret = setMaxCharH( m_HMaxC )), -1 );

		dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Reset the OCR settings
	 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE . */
	int resetOCR( )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret;
		closeOCR( );
		DMO_CHECK_ERR(( ret = initOCR( )), -1 );
		dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Set the OCR classification methode.
	 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE . */
	int setClsMethod( )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_FALSE;
		DMO_CHECK_ERR( m_hOcrHandle != IPL_DONT_CARE, IPL_HANDLE_ERROR );

		int tmp;
		switch( m_ClsM )
		{
		case CLSM_MATCH:
			tmp = OCR_KOR_CLAS;
			break;
		case CLSM_FOURIER:
			tmp = OCR_DFT_CLAS;
			break;
		}

		DMO_CHECK_ERR(( ret = OCR_Control( m_hOcrHandle, OCR_CLASSIFICATIONTYPE, &tmp )), IPL_GetErrorCode2( ));

		dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Set the image background.
	 *  \param value the new value ( \sa BGRD_WHITE of \sa BGRD_BLACK )
	 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE . */
	int setBackground( const int value )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_FALSE;
		DMO_CHECK_ERR( m_hOcrHandle != IPL_DONT_CARE, IPL_HANDLE_ERROR );

		dprintf(( " =+ %s - bgrd: %s\n", __FUNCTION__, m_bgrd == BGRD_WHITE ? "WHITE" : "BLACK" ));
		int tmp = value;
		DMO_CHECK_ERR(( ret = OCR_Control( m_hOcrHandle, OCRSEG_CHAR_COLOR, &tmp )), IPL_GetErrorCode2( ));
		m_bgrd = value;

		dprintf(( " =- %s - bgrd: %s\n", __FUNCTION__, m_bgrd == BGRD_WHITE ? "WHITE" : "BLACK" ));

		dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Set minimum allowed width of the pattern segment.
	 *  \param value the new value
	 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE . */
	int setWMin( const int value )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_TRUE;
		if( m_hOcrHandle != IPL_DONT_CARE )
		{
			int tmp = value;
			DMO_CHECK_ERR(( ret = OCR_Control( m_hOcrHandle, OCRSEG_MINWIDTH, &tmp )), IPL_GetErrorCode2( ));
			m_bDefaultSegSize = false;
		}
		m_WMin = value;

		dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Set minimum allowed hight of the pattern segment.
	 *  \param value the new value
	 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE . */
	int setHMin( const int value )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_TRUE;
		if( m_hOcrHandle != IPL_DONT_CARE )
		{
			int tmp = value;
			DMO_CHECK_ERR(( ret = OCR_Control( m_hOcrHandle, OCRSEG_MINHEIGHT, &tmp )), IPL_GetErrorCode2( ));
			m_bDefaultSegSize = false;
		}

		m_HMin = value;

		dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Set maximum allowed width of the pattern segment.
	 *  \param value the new value
	 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE . */
	int setWMax( const int value )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_TRUE;
		if( m_hOcrHandle != IPL_DONT_CARE )
		{
			int tmp = value;
			DMO_CHECK_ERR(( ret = OCR_Control( m_hOcrHandle, OCRSEG_MAXWIDTH, &tmp )), IPL_GetErrorCode2( ));
			m_bDefaultSegSize = false;
		}

		m_WMax = value;

		dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Set maximum allowed hight of the pattern segment.
	 *  \param value the new value
	 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE . */
	int setHMax( const int value )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_TRUE;
		if( m_hOcrHandle != IPL_DONT_CARE )
		{
			int tmp = value;
			DMO_CHECK_ERR(( ret = OCR_Control( m_hOcrHandle, OCRSEG_MAXHEIGHT, &tmp )), IPL_GetErrorCode2( ));
			m_bDefaultSegSize = false;
		}

		m_HMax = value;

		dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Set minimum allowed hight of the character.
	 *  \param value the new value
	 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE . */
	int setMinCharH( const int value )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_TRUE;
		if( m_hOcrHandle != IPL_DONT_CARE )
		{
			int tmp = value;
			DMO_CHECK_ERR(( ret = OCR_Control( m_hOcrHandle, OCRCHAR_MINHEIGHT, &tmp )), IPL_GetErrorCode2( ));
		}

		m_HMinC = value;

		dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Set maximum allowed hight of the character.
	 *  \param value the new value
	 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE . */
	int setMaxCharH( const int value )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_TRUE;
		if( m_hOcrHandle != IPL_DONT_CARE )
		{
			int tmp = value;
			DMO_CHECK_ERR(( ret = OCR_Control( m_hOcrHandle, OCRCHAR_MAXHEIGHT, &tmp )), IPL_GetErrorCode2( ));
		}

		m_HMaxC = value;

		dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Set minimum allowed width of the character.
	 *  \param value the new value
	 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE . */
	int setMinCharW( const int value )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_TRUE;
		if( m_hOcrHandle != IPL_DONT_CARE )
		{
			int tmp = value;
			DMO_CHECK_ERR(( ret = OCR_Control( m_hOcrHandle, OCRCHAR_MINWIDTH, &tmp )), IPL_GetErrorCode2( ));
		}

		m_WMinC = value;

		dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Set maximum allowed width of the character.
	 *  \param value the new value
	 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE . */
	int setMaxCharW( const int value )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_TRUE;
		if( m_hOcrHandle != IPL_DONT_CARE )
		{
			int tmp = value;
			DMO_CHECK_ERR(( ret = OCR_Control( m_hOcrHandle, OCRCHAR_MAXWIDTH, &tmp )), IPL_GetErrorCode2( ));
		}

		m_WMaxC = value;

		dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Set threshold value for the OCR
	 *  \param threshold the new value
	 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE . */
	int setThershold( IPL_LONG& threshold )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_FALSE;
		DMO_CHECK_ERR( m_hOcrHandle != IPL_DONT_CARE, IPL_HANDLE_ERROR );
		DMO_CHECK_ERR(( ret = OCR_Control( m_hOcrHandle, OCRSEG_THRESHOLD, &threshold )), IPL_GetErrorCode2( ));

		dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Get the segments from the image
	 *  \param imageHandle the input image handle
	 *  \return If succeeds the number of the found segments will be returned else \c IPL_FALSE will be returned. */
	IPL_LONG getSegments( const IPL_BUFHANDLE imageHandle )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		m_SegCount = -1;

		IPL_LONG ret = IPL_TRUE;
		DMO_CHECK_ERR( m_hOcrHandle != IPL_DONT_CARE, IPL_HANDLE_ERROR );
		DMO_CHECK_ERR(( ret = OCR_Segment( m_hOcrHandle, imageHandle, MAX_OBJ_CNT, m_Rect, &m_SegCount )), IPL_GetErrorCode2( ));
		ret = m_SegCount;
		
		dprintf(( " -- %s (%s: %d) (%ld)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Sort the found segments
	 *  \param imageHandle the input image handle
	 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE . */
	int sortSegments( const IPL_BUFHANDLE imageHandle )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_FALSE;
		DMO_CHECK_ERR( m_hOcrHandle != IPL_DONT_CARE, IPL_HANDLE_ERROR );
		DMO_CHECK_ERR( imageHandle != IPL_DONT_CARE, IPL_HANDLE_ERROR );
		DMO_CHECK_ERR(( ret = OCR_SortSegment( m_hOcrHandle, imageHandle, m_Rect, m_SegCount )), IPL_GetErrorCode2( ));

		dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Read the character
	 *  \param imageHandle the input image handle
	 *  \param index index of the segment
	 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE . */
	int readChar( const IPL_BUFHANDLE imageHandle, const int index )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_TRUE;
		cleanResults( );
		DMO_CHECK_ERR( m_hOcrHandle != IPL_DONT_CARE, IPL_HANDLE_ERROR );
		DMO_CHECK_ERR( imageHandle != IPL_DONT_CARE, IPL_HANDLE_ERROR );
		DMO_CHECK_ERR( index >= 0, -1 );
		IPL_LONG maxres = 1;
		ret = OCR_ReadChar( m_hOcrHandle, imageHandle, &m_Rect[index], &maxres, m_Result );

		dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Teach the character
	 *  \param imageHandle the input image handle
	 *  \param index index of the segment
	 *  \param c character
	 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE . */
	int teachChar( const IPL_BUFHANDLE imageHandle, const int index, const int c )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_FALSE;
		DMO_CHECK_ERR( m_hOcrHandle != IPL_DONT_CARE, IPL_HANDLE_ERROR );
		DMO_CHECK_ERR( index >= 0, -1 );
		DMO_CHECK_ERR(( ret = OCR_Teach( m_hOcrHandle, imageHandle, &m_Rect[index], c )), IPL_GetErrorCode2( ));

		dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Delete the character
	 *  \param index index of the pattern
	 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE . */
	int deleteOCR( const int index )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_FALSE;
		DMO_CHECK_ERR( m_hOcrHandle != IPL_DONT_CARE, IPL_HANDLE_ERROR );
		DMO_CHECK_ERR( index >= 0, -1 );
		DMO_CHECK_ERR(( ret = OCR_DelPattern( m_hOcrHandle, index )), IPL_GetErrorCode2( ));

		dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Get the pattern number
	 *  \return If succeeds the number of the pattern will be returned else \c IPL_FALSE will be returned. */
	IPL_LONG getPatternNo( )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		IPL_LONG ret = -1;
		DMO_CHECK_ERR( m_hOcrHandle != IPL_DONT_CARE, IPL_HANDLE_ERROR );
		DMO_CHECK_ERR( OCR_Inquire( m_hOcrHandle, OCR_PATTERNNUMBER, &ret ), IPL_GetErrorCode2( ));

		dprintf(( " -- %s (%s: %d) (%ld)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Get a pattern
	 *  \param index index of the pattern
	 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE . */
	int getPattern( const int index, OCR_PATTERN& pat  )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_TRUE;
		if( m_hOcrHandle != IPL_DONT_CARE )
			DMO_CHECK_ERR(( ret = OCR_GetPattern( m_hOcrHandle, index, &pat )), IPL_GetErrorCode2( ));

		dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Read all charcter
	 *  \param imageHandle the input image handle
	 *  \return If succeeds the pattern will be returned else \c IPL_FALSE . */
	int readOCR( const IPL_BUFHANDLE imageHandle )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret;
		int txt_cnt = 0;
		memset( m_DisplayText, 0, MAX_DISP_CNT );

		DMO_CHECK_ERR( m_hOcrHandle != IPL_DONT_CARE, IPL_HANDLE_ERROR );
		/* sort the segments */
		DMO_CHECK_ERR(( ret = sortSegments( imageHandle )), -1 );

		/* read the character */
		for( int i = 0; i < m_SegCount; i++ )
		{
			/* read single character */
			if( readChar( imageHandle, i ))
			{
				/* save the data */
				m_ReadText[i] = m_Result[0].ascii;
				m_ReadText[i+1] = 0;
				m_Quality[i] = (int)( m_Result[0].quality * 100 );
				m_Quality[i+1] = 0;

				/* check the read data quality */
				if( m_Quality[i] >= m_MinQuality )
				{
					/* check for line */
					txt_cnt = get_new_line( m_DisplayText, m_Result[0].ascii, i, txt_cnt );
					dprintf((" == " TOOL_PREFIX ": %s - %d: >>%s<<\n", __FUNCTION__, txt_cnt, m_DisplayText ));
				}
			}
		}
#ifdef DEBUG
		if( m_SegCount > 0 )
			printf(" == " TOOL_PREFIX ": %s - readed(%d): >>%s<<!!\n--->>%s<<\n", __FUNCTION__, txt_cnt, m_ReadText, m_DisplayText );
#endif	// DEBUG

		dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Split large segments.
	 *  \param imageHandle input image handle
	 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE .*/
	int splitSeg( const IPL_BUFHANDLE imageHandle )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		int ret = m_SegCount;

		DMO_CHECK_ERR( m_hOcrHandle != IPL_DONT_CARE, IPL_HANDLE_ERROR );
		DMO_CHECK_ERR( imageHandle != IPL_DONT_CARE, IPL_HANDLE_ERROR );

		if( m_SegCount )
		{
			switch( m_SegType )
			{
			case SEG_SPLIT:
			case SEG_SPLIT_MERGE:
			case SEG_MERGE_SPLIT:
				DMO_CHECK_ERR(( ret = OCR_SplitSegment( m_hOcrHandle, imageHandle, MAX_OBJ_CNT, m_Rect, &m_SegCount )), IPL_GetErrorCode2( ));
				ret = m_SegCount;
				break;
			default:
				break;
			}
		}

		dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Merge small segments.
	 *  \param imageHandle input image handle
	 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE .*/
	int mergeSeg( const IPL_BUFHANDLE imageHandle )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = m_SegCount;
		DMO_CHECK_ERR( m_hOcrHandle != IPL_DONT_CARE, IPL_HANDLE_ERROR );
		DMO_CHECK_ERR( imageHandle != IPL_DONT_CARE, IPL_HANDLE_ERROR );

		if( m_SegCount )
		{
			switch( m_SegType )
			{
			case SEG_MERGE:
			case SEG_SPLIT_MERGE:
			case SEG_MERGE_SPLIT:
				DMO_CHECK_ERR(( ret = OCR_MergeSegment( m_hOcrHandle, imageHandle, MAX_OBJ_CNT, m_Rect, &m_SegCount )), IPL_GetErrorCode2( ));
				ret = m_SegCount;
				break;
			default:
				break;
			}
		}

		dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	IPL_LONG m_WMin, m_WMax, m_HMin, m_HMax;
	IPL_LONG m_WMinC, m_WMaxC, m_HMinC, m_HMaxC;

	IPL_RECT m_Rect[MAX_OBJ_CNT];
	char m_ReadText[MAX_OBJ_CNT];
	char m_DisplayText[ MAX_DISP_CNT];
	int m_Quality[MAX_OBJ_CNT];
	int m_MinQuality;
	int m_bgrd;
	int m_BlankValue;
	OCR_RESULT m_Result[2];
	bool m_bAutoSegmentation;

	SEG_TYPE m_SegType;						/*!< \brief \b Segmentation type.*/
	CLS_METHOD m_ClsM;						/*!< \brief Classification method.*/

	/*----------------------------------------------------------------*/
	long getBestSegMinWidth( )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		long ret = -1;

		if( setDefaultSegSize( ) == IPL_TRUE && m_hOcrHandle != IPL_DONT_CARE )
		{
//			getSegments( const IPL_BUFHANDLE imageHandle )
			if( OCR_Inquire( m_hOcrHandle, OCRSEG_MINWIDTH, &ret ) == IPL_FALSE )
				ret = -1;
		}

		dprintf(( " -- %s (%s: %d) (%ld)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	long getBestSegMaxWidth( )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		long ret = -1;

		if( setDefaultSegSize( ) == IPL_TRUE && m_hOcrHandle != IPL_DONT_CARE )
		{
			if( OCR_Inquire( m_hOcrHandle, OCRSEG_MAXWIDTH, &ret ) == IPL_FALSE )
				ret = -1;
		}

		dprintf(( " -- %s (%s: %d) (%ld)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	long getBestSegMinHeight( )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		long ret = -1;

		if( setDefaultSegSize( ) == IPL_TRUE && m_hOcrHandle != IPL_DONT_CARE )
		{
			if( OCR_Inquire( m_hOcrHandle, OCRSEG_MINHEIGHT, &ret ) == IPL_FALSE )
				ret = -1;
		}

		dprintf(( " -- %s (%s: %d) (%ld)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	long getBestSegMaxHeight( )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		long ret = -1;

		if( setDefaultSegSize( ) == IPL_TRUE && m_hOcrHandle != IPL_DONT_CARE )
		{
			if( OCR_Inquire( m_hOcrHandle, OCRSEG_MAXHEIGHT, &ret ) == IPL_FALSE )
				ret = -1;
		}

		dprintf(( " -- %s (%s: %d) (%ld)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
private:
	/*----------------------------------------------------------------*/
	OCR_BUFHANDLE m_hOcrHandle;
	IPL_LONG m_SegCount;
	bool m_bDefaultSegSize;

	/*----------------------------------------------------------------*/
	int setDefaultSegSize( )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_TRUE;

		if( !m_bDefaultSegSize )
		{
			int tmp = 1;
			DMO_CHECK_ERR(( ret = OCR_Control( m_hOcrHandle, OCRSEG_MINWIDTH, &tmp )), IPL_GetErrorCode2( ));
			DMO_CHECK_ERR(( ret = OCR_Control( m_hOcrHandle, OCRSEG_MINHEIGHT, &tmp )), IPL_GetErrorCode2( ));

			tmp = 0;
			DMO_CHECK_ERR(( ret = OCR_Control( m_hOcrHandle, OCRSEG_MAXWIDTH, &tmp )), IPL_GetErrorCode2( ));
			DMO_CHECK_ERR(( ret = OCR_Control( m_hOcrHandle, OCRSEG_MAXHEIGHT, &tmp )), IPL_GetErrorCode2( ));

			m_bDefaultSegSize = true;
		}

		dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Check for the new line
	 *  \param displaytext the hole text
	 *  \param ascii the new ascii value
	 *  \param pos the position of the new character
	 *  \param cnt the count of already read data
	 *  \return The new count of the hole text will be returned.*/
	int get_new_line( char *displaytext, int ascii, int pos, int cnt )
	{
		dprintf(( " ++ %s (%s: %d) - ascii: %c, pos: %d, cnt: %d\n",
				__FUNCTION__, __FILE__, __LINE__, ascii, pos, cnt ));
		int next_flag = 0;
		int delta = 0;

		if(( m_Rect[pos].left < m_Rect[pos - 1].left ) ||
		((int)m_Rect[pos].top > m_Rect[pos - 1].top + ( m_BlankValue >> 1 )))
		{
			displaytext[cnt++] = '\n' ;
			next_flag = 1;
		}

		if( !next_flag && pos != 0 )
			delta = m_Rect[pos].left - m_Rect[pos - 1].left;
		else
			delta = 0;

		if( delta >= m_BlankValue )
			displaytext[cnt++] = ' ';

		displaytext[cnt] = ascii ;

		if( displaytext[cnt] == UNKNOWN_CHAR )
		{
			displaytext[cnt] = '_' ;
		}
		cnt++;

		dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, cnt ));
		return cnt;
	}

	/*----------------------------------------------------------------*/
	void cleanResults()	{ memset( m_Result, 0, sizeof( OCR_RESULT ) * 2 ); }

	/*----------------------------------------------------------------*/
};

//----------------------------------------------------------------------
//----------------------------------------------------------------------
#endif	// __ocr_h__
