//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
/*! \file senddata.h
 *  \brief Sent the data on the serial port.
 *  \author MATRIX VISION GmbH (td)
 */
//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
#ifndef __senddata_h__
#define __senddata_h__

//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
#include <stdio.h>
#include <unistd.h>
#include <termios.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "ocrtool.h"

//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
#define TTYS0_DEV					"/dev/ttyS0"			/*!< \brief Serial device \c ttyS0 .*/
#define TTYS1_DEV					"/dev/ttyS1"			/*!< \brief Serial device \c ttyS1 .*/

//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
/*! \class CBaseSendData
 *  \brief Send data base class.
 *  \author MATRIX VISION GmbH (td)
 *
 * \li This is the base class for the send data classes. */
class CBaseSendData
{
public:
	/*------------------------------------------------------------------*/
	/*! \brief The constructor. All members will be initialized here. */
	CBaseSendData( )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		m_fd = -1;

		dprintf(( " -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
	}

	/*------------------------------------------------------------------*/
	/*! \brief The destructor. The opened device will be closed. */
	virtual ~CBaseSendData( )
	{
		dprintf(( " ++ ~%s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		if( m_fd > 0 )
		{
			close( m_fd );
			m_fd = -1;
		}

		dprintf(( " -- ~%s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
	}

	/*------------------------------------------------------------------*/
	/*! \brief Write the data.
	 *  \param vptr data pointer
	 *  \param n data size
	 *  \return If succeeds \c IPL_TRUE will be returned, else \c IPL_FALSE . */
	ssize_t Writen( const void *vptr, const size_t n )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		DMO_CHECK_ERR( vptr, -1 );
		DMO_CHECK_ERR( n, -1 );

		size_t nleft;
		ssize_t nwritten;
		const char *ptr;

		ptr = (const char*) vptr;
		nleft = n;
		while( nleft > 0 )
		{
			if(( nwritten = write( m_fd, ptr, nleft )) <= 0 )
			{
				if( errno == EINTR )
					nwritten = 0;				/* and call write() again */
				else
					return IPL_FALSE;			/* error */
			}

			nleft -= nwritten;
			ptr   += nwritten;
		}

		dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, n ));
		return n;
	}

	/*------------------------------------------------------------------*/
	/*! \brief Get the baudrate value.
	 *  \param BrType baudrate type
	 *  \return The value of the baudrate will be returned. */
	long GetBaudrate( const long BrType )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		long ret = IPL_TRUE;

		if( BrType == 0 )
			ret = B0;
		else if( BrType == 50 )
			ret = B50;
		else if( BrType == 75 )
			ret = B75;
		else if( BrType == 110 )
			ret = B110;
		else if( BrType == 134 )
			ret = B134;
		else if( BrType == 150 )
			ret = B150;
		else if( BrType == 200 )
			ret = B200;
		else if( BrType == 300 )
			ret = B300;
		else if( BrType == 600 )
			ret = B600;
		else if( BrType == 1200 )
			ret = B1200;
		else if( BrType == 1800 )
			ret = B1800;
		else if( BrType == 2400 )
			ret = B2400;
		else if( BrType == 4800 )
			ret = B4800;
		else if( BrType == 9600 )
			ret = B9600;
		else if( BrType == 19200 )
			ret = B19200;
		else if( BrType == 38400 )
			ret = B38400;
		else if( BrType == 57600 )
			ret = B57600;
		else if( BrType == 115200 )
			ret = B115200;
		else if( BrType == 230400 )
			ret = B230400;


		dprintf(( " -- %s (%s: %d) (%ld)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*------------------------------------------------------------------*/
protected:
	int m_fd;

	/*------------------------------------------------------------------*/
};

//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
/*! \class CSendDatattySX
 *  \brief Send serial data class.
 *  \author MATRIX VISION GmbH (td)
 *
 * \li This class is used to send the data via serial connection. */
class CSendDatattySX : public CBaseSendData
{
public:
	/*------------------------------------------------------------------*/
	/*! \brief The constructor. */
	CSendDatattySX( )
	: CBaseSendData( )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		dprintf(( " -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
	}

	/*------------------------------------------------------------------*/
	/*! \brief The destructor. The opened device will be closed. */
	virtual ~CSendDatattySX( )
	{
		dprintf(( " ++ ~%s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		if( m_fd > 0 )
		{
			close( m_fd );
			m_fd = -1;
		}
		dprintf(( " -- ~%s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
	}

	/*------------------------------------------------------------------*/
	/*! \brief Initialize the serial port.
	 *  \param pttySX The serial device
	 *  \param Baudrate The baudrate
	 *  \return If succeeds \c IPL_TRUE will be returned, else \c IPL_FALSE . */
	int InitSerial( const char* pttySX = TTYS0_DEV, const int Baudrate = 115200  )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_TRUE;

		DMO_CHECK_ERR( pttySX, -1 );
		DMO_CHECK_ERR( Baudrate >= 0, -1 );

		dprintf(( " == %s (%s: %d) - open '%s' with %d\n", __FUNCTION__, __FILE__, __LINE__, pttySX, Baudrate ));

		DMO_CHECK_ERR(( m_fd = open( pttySX, O_RDWR )) >= 0, -1 );

		DMO_CHECK_ERR(( ret = SetSerial( GetBaudrate( Baudrate ))) >= 0, -1 );

		dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*------------------------------------------------------------------*/
	/*! \brief Make the settings for the opened serial device.
	 *  \param Baudrate The baudrate
	 *  \return If succeeds \c IPL_TRUE will be returned, else \c IPL_FALSE . */
	int SetSerial( const int Baudrate )
	{
		dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_TRUE;

		DMO_CHECK_ERR( Baudrate >= 0, -1 );

		struct termios newtio;

		bzero( &newtio, sizeof( newtio )); /* clear struct for new port settings */

		/*
		* BAUDRATE: Set bps rate. You could also use cfsetispeed and cfsetospeed.
		* CRTSCTS : output hardware flow control (only used if the cable has
					all necessary lines. See sect. 7 of Serial-HOWTO)
		* CS8     : 8n1 (8bit,no parity,1 stopbit)
		* CLOCAL  : local connection, no modem contol
		* CREAD   : enable receiving characters
		*/

		newtio.c_cflag = Baudrate|CS8|CLOCAL|CREAD;

		/*
		* IGNPAR  : ignore bytes with parity errors
		* ICRNL   : map CR to NL (otherwise a CR input on the other computer
					will not terminate input)
		* otherwise make device raw (no other input processing)
		*/
		newtio.c_iflag = 0;	//&= ~(IGNBRK|BRKINT|PARMRK|ISTRIP|INLCR|IGNCR|IXON);		//= 0;

		/*
		* Raw output.
		*/
		newtio.c_oflag = 0;//~OPOST;		// = 0;

		/*
		* ICANON  : enable canonical input
		* disable all echo functionality, and don't send signals to calling program
		*/
		newtio.c_lflag = 0;	//&= ~(ECHO|ECHONL|ICANON|ISIG|IEXTEN);	// = 0;

		/*
		* initialize all control characters
		* default values can be found in /usr/include/termios.h, and are given
		* in the comments, but we don't need them here
		*/
		newtio.c_cc[VINTR]    = 0;     /* 003, ETX, Ctrl-C, or also 0177, DEL, rubout */
		newtio.c_cc[VQUIT]    = 0;     /* 034, FS, Ctrl-\ */
		newtio.c_cc[VERASE]   = 0;     /* 0177, DEL, rubout, or 010, BS, Ctrl-H, or also # */
		newtio.c_cc[VKILL]    = 0;     /* 025, NAK, Ctrl-U, or Ctrl-X, or also @ */
		newtio.c_cc[VEOF]     = 4;     /* 004, EOT, Ctrl-D */
		newtio.c_cc[VTIME]    = 0;     /* inter-character timer unused */
		newtio.c_cc[VMIN]     = 1;     /* blocking read until 1 character arrives */
		//newtio.c_cc[VSWTC]    = 0;     /* '\0' */
		newtio.c_cc[VSTART]   = 0;     /* 021, DC1, Ctrl-Q */
		newtio.c_cc[VSTOP]    = 0;     /* 023, DC3, Ctrl-S */
		newtio.c_cc[VSUSP]    = 0;     /* 032, SUB, Ctrl-Z */
		newtio.c_cc[VEOL]     = 0;     /* 0, NUL */
		newtio.c_cc[VREPRINT] = 0;     /* 022, DC2, Ctrl-R */
		//newtio.c_cc[VDISCARD] = 0;     /* Ctrl-u */
		newtio.c_cc[VWERASE]  = 0;     /* 027, ETB, Ctrl-W */
		newtio.c_cc[VLNEXT]   = 0;     /* 026, SYN, Ctrl-V */
		newtio.c_cc[VEOL2]    = 0;     /* 0, NUL */

		/*
		* now clean the modem line and activate the settings for the port
		*/
		tcflush( m_fd, TCIOFLUSH );
		tcsetattr( m_fd, TCSANOW, &newtio );

		dprintf(( " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*------------------------------------------------------------------*/
};

//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
#endif	// __senddata_h__
